@@ -899,6 +899,23 @@ Value* RTBuilder::getInstanceContributionToHitGroupIndex(
     return {};
 }
 
+Value* RTBuilder::getRayMask(
+    RTBuilder::StackPointerVal* perLaneStackPtr)
+{
+    switch (getMemoryStyle())
+    {
+#define STYLE(X)                                            \
+    case RTMemoryStyle::X:                                  \
+        return _getRayMask_##X(                             \
+            perLaneStackPtr,                                \
+            VALUE_NAME("RayMask"));
+#include "RayTracingMemoryStyle.h"
+#undef STYLE
+    }
+    IGC_ASSERT(0);
+    return {};
+}
+
 Value* RTBuilder::getObjToWorld(
     RTBuilder::StackPointerVal* perLaneStackPtr,
     uint32_t dim,
@@ -1044,15 +1061,21 @@ Value* RTBuilder::getTraceRayPayload(
     return bitField;
 }
 
-Value* RTBuilder::emitStateRegID(uint32_t BitStart, uint32_t BitEnd)
+GenIntrinsicInst* RTBuilder::getSr0_0()
 {
     Module* module = this->GetInsertBlock()->getModule();
 
-    Value* sr0_0 = this->CreateCall(
+    auto* sr0_0 = this->CreateCall(
         GenISAIntrinsic::getDeclaration(
             module, GenISAIntrinsic::GenISA_getSR0_0),
         None,
         VALUE_NAME("sr0.0"));
+    return cast<GenIntrinsicInst>(sr0_0);
+}
+
+Value* RTBuilder::emitStateRegID(uint32_t BitStart, uint32_t BitEnd)
+{
+    Value* sr0_0 = getSr0_0();
 
     uint32_t and_imm = BITMASK_RANGE(BitStart, BitEnd);
     uint32_t shr_imm = BitStart;
@@ -1411,14 +1434,23 @@ Value* RTBuilder::getGlobalBufferPtr()
 
 Value* RTBuilder::getSyncStackID()
 {
-    Module* module = this->GetInsertBlock()->getModule();
-    Value* stackID = this->CreateCall(
-        GenISAIntrinsic::getDeclaration(
-            module, GenISAIntrinsic::GenISA_SyncStackID),
-        None,
-        VALUE_NAME("SyncStackID"));
+    auto& PlatformInfo = Ctx.platform.getPlatformInfo();
 
-    return stackID;
+    if (PlatformInfo.eProductFamily == IGFX_DG2        ||
+        PlatformInfo.eProductFamily == IGFX_METEORLAKE)
+    {
+        return _getSyncStackID_Xe(VALUE_NAME("SyncStackID"));
+    }
+    else if (PlatformInfo.eRenderCoreFamily == IGFX_XE_HPC_CORE)
+    {
+        return _getSyncStackID_Xe_HPC(VALUE_NAME("SyncStackID"));
+    }
+    else
+    {
+        IGC_ASSERT_MESSAGE(0, "Invalid Product Family for SyncStackID");
+    }
+
+    return {};
 }
 
 bool RTBuilder::checkAlign(
