@@ -265,6 +265,18 @@ RETVAL CGen8OpenCLProgramBase::GetProgramDebugData(char* dstBuffer, size_t dstBu
     return retValue;
 }
 
+static void dumpZEInfo(
+    const IGC::CodeGenContext &Ctx, ZEBinaryBuilder &ZEBuilder)
+{
+    auto filename = IGC::Debug::DumpName(
+        IGC::Debug::GetShaderOutputName())
+        .Hash(Ctx.hash)
+        .Type(ShaderType::OPENCL_SHADER)
+        .Extension("zeinfo");
+
+    ZEBuilder.printZEInfo(filename.str());
+}
+
 void dumpOCLKernelBinary(
     const IGC::COpenCLKernel *Kernel,
     const KernelData &data)
@@ -458,6 +470,8 @@ bool CGen8OpenCLProgram::GetZEBinary(
         (const uint8_t*)buildOptions, buildOptionsSize);
     zebuilder.setProductFamily(m_Platform.eProductFamily);
     zebuilder.setGfxCoreFamily(m_Platform.eRenderCoreFamily);
+    zebuilder.setVISAABIVersion(m_Context.platform.getVISAABIVersion());
+    zebuilder.setGmdID(m_Platform.sRenderBlockID);
 
     //
     // Creating ZE binary requires linking individual ELF files,
@@ -669,7 +683,17 @@ bool CGen8OpenCLProgram::GetZEBinary(
                 llvm::raw_string_ostream linkOut(linkOutStr);
 
                 constexpr bool canExitEarly = false;
-                if (IGCLLD::elf::link(elfArrRef, canExitEarly, linkOut, linkErr))
+                bool linked = false;
+                {
+                    // LLD is not assured to be thread-safe.
+                    // Mutex can be removed as soon as thread-safety is implemented in future versions of LLVM.
+                    static std::mutex linkerMtx;
+                    std::lock_guard<std::mutex> lck(linkerMtx);
+                    linked =
+                        IGCLLD::elf::link(elfArrRef, canExitEarly, linkOut, linkErr);
+                }
+
+                if(linked)
                 {
                     // Multiple ELF files linked.
                     // Copy the data from the linked file to a memory, what will be a source location
@@ -765,6 +789,11 @@ bool CGen8OpenCLProgram::GetZEBinary(
     }
 
     zebuilder.getBinaryObject(programBinary);
+
+    // dump .ze_info to a file
+    if (IGC_IS_FLAG_ENABLED(ShaderDumpEnable))
+        dumpZEInfo(m_Context, zebuilder);
+
     return retValue;
 }
 
