@@ -37,6 +37,7 @@ SPDX-License-Identifier: MIT
 #include "common/secure_mem.h"
 #include "common/shaderOverride.hpp"
 #include "common/ModuleSplitter.h"
+#include "common/IGCSPIRVParser.h"
 
 #include "CLElfLib/ElfReader.h"
 
@@ -476,6 +477,9 @@ bool TranslateSPIRVToLLVM(
             Context,
             *LLVMModule,
             llvm::StringRef(InputArgs.pOptions, InputArgs.OptionsSize));
+
+      if (IGC_IS_FLAG_ENABLED(ShaderDumpTranslationOnly))
+          LLVMModule->dump();
     }
 
     return success;
@@ -734,15 +738,13 @@ bool ProcessElfInput(
                 IGC_ASSERT(hasVISALinking);
             }
 
-            // Create a copy of the string to return to the caller. The output type
-            // determines how the buffer gets managed
-            char* pBufResult = static_cast<char*>(operator new(OutputString.size(), std::nothrow));
-            if (pBufResult != NULL)
+            if (outType == TB_DATA_FORMAT_LLVM_BINARY)
             {
-                if (outType == TB_DATA_FORMAT_LLVM_BINARY)
+                // Create a copy of the string to return to the caller.
+                char* pBufResult = static_cast<char*>(operator new(OutputString.size(), std::nothrow));
+                if (pBufResult != NULL)
                 {
                     memcpy_s(pBufResult, OutputString.size(), OutputString.data(), OutputString.size());
-
                     // The buffer is returned to the runtime. When the buffer is not
                     // needed anymore the runtime ir responsible to call the module for
                     // destroying it
@@ -759,17 +761,17 @@ bool ProcessElfInput(
                         QWORD prevAsmHash = previousHash.getAsmHash();
                         std::ostringstream oss1(std::ostringstream::ate);
                         oss1 << std::hex
-                             << std::setfill('0')
-                             << std::setw(sizeof(prevAsmHash) * CHAR_BIT / 4)
-                             << prevAsmHash;
+                            << std::setfill('0')
+                            << std::setw(sizeof(prevAsmHash) * CHAR_BIT / 4)
+                            << prevAsmHash;
                         const std::string prevHashString = oss1.str();
 
                         QWORD newAsmHash = ShaderHashOCL((const UINT*)OutputArgs.pOutput, OutputArgs.OutputSize / 4).getAsmHash();
                         std::ostringstream oss2(std::ostringstream::ate);
                         oss2 << std::hex
-                             << std::setfill('0')
-                             << std::setw(sizeof(newAsmHash) * CHAR_BIT / 4)
-                             << newAsmHash;
+                            << std::setfill('0')
+                            << std::setw(sizeof(newAsmHash) * CHAR_BIT / 4)
+                            << newAsmHash;
                         const std::string newHashString = oss2.str();
 
                         llvm::Twine outputPath = outputDir;
@@ -790,80 +792,90 @@ bool ProcessElfInput(
                     }
 #endif // defined(IGC_SPIRV_ENABLED)
 
-                    if (success == true)
+                    // if -dump-opt-llvm is enabled dump the llvm output to the file
+                    std::string options = "";
+                    if ((InputArgs.pOptions != nullptr) && (InputArgs.OptionsSize > 0))
                     {
-                        // if -dump-opt-llvm is enabled dump the llvm output to the file
-                        std::string options = "";
-                        if ((InputArgs.pOptions != nullptr) && (InputArgs.OptionsSize > 0))
+                        options.append(InputArgs.pOptions, InputArgs.pOptions + InputArgs.OptionsSize);
+                    }
+                    size_t dumpOptPosition = options.find("-dump-opt-llvm");
+                    if (dumpOptPosition != std::string::npos)
+                    {
+                        std::string dumpFileName;
+                        std::istringstream iss(options.substr(dumpOptPosition));
+                        iss >> dumpFileName;
+                        size_t equalSignPosition = dumpFileName.find('=');
+                        if (equalSignPosition != std::string::npos)
                         {
-                            options.append(InputArgs.pOptions, InputArgs.pOptions + InputArgs.OptionsSize);
-                        }
-                        size_t dumpOptPosition = options.find("-dump-opt-llvm");
-                        if (dumpOptPosition != std::string::npos)
-                        {
-                            std::string dumpFileName;
-                            std::istringstream iss(options.substr(dumpOptPosition));
-                            iss >> dumpFileName;
-                            size_t equalSignPosition = dumpFileName.find('=');
-                            if (equalSignPosition != std::string::npos)
+                            dumpFileName = dumpFileName.substr(equalSignPosition + 1);
+                            // dump the buffer
+                            FILE* file = fopen(dumpFileName.c_str(), "wb");
+                            if (file != NULL)
                             {
-                                dumpFileName = dumpFileName.substr(equalSignPosition + 1);
-                                // dump the buffer
-                                FILE* file = fopen(dumpFileName.c_str(), "wb");
-                                if (file != NULL)
-                                {
-                                    fwrite(pBufResult, OutputString.size(), 1, file);
-                                    fclose(file);
-                                }
-                            }
-                            else
-                            {
-                                std::string errorString = "File name not specified with the -dump-opt-llvm option.";
-                                SetWarningMessage(errorString, OutputArgs);
+                                fwrite(pBufResult, OutputString.size(), 1, file);
+                                fclose(file);
                             }
                         }
-                    }
-                }
-                else if (IsDeviceBinaryFormat(outType))
-                {
-                    if (hasVISALinking)
-                    {
-                        ShaderHash hash = ShaderHashOCL(
-                            reinterpret_cast<const UINT*>(InputArgs.pInput),
-                            InputArgs.InputSize / 4);
-                        std::string errorMessage;
-
-                        // Temporary workaround for invoke_sycl case.
-                        std::reverse(SPIRVToLink.begin(), SPIRVToLink.end());
-
-                        success = VLD::TranslateBuildSPMDAndESIMD(
-                            SPIRVToLink, &OutputArgs, TB_DATA_FORMAT_SPIR_V,
-                            Context.platform, profilingTimerResolution, hash,
-                            errorMessage);
-
-                        if (!success)
+                        else
                         {
-                            SetErrorMessage(errorMessage, OutputArgs);
-                            return false;
+                            std::string errorString = "File name not specified with the -dump-opt-llvm option.";
+                            SetWarningMessage(errorString, OutputArgs);
                         }
                     }
-                    else
-                    {
-                        InputArgs.pInput = OutputString.data();
-                        InputArgs.InputSize = OutputString.size();
-                        success = TC::TranslateBuild(
-                            &InputArgs, &OutputArgs, TB_DATA_FORMAT_LLVM_BINARY,
-                            Context.platform, profilingTimerResolution);
-                    }
                 }
                 else
                 {
-                    IGC_ASSERT_MESSAGE(0, "Unrecognized output format when processing ELF input");
                     success = false;
                 }
             }
+            else if (IsDeviceBinaryFormat(outType))
+            {
+                if (hasVISALinking)
+                {
+                    if (!Context.enableZEBinary()) {
+                        SetErrorMessage(
+                            "vISA linking can be used only with ZeBinary "
+                            "compiler output format. It seems that it is "
+                            "currently disabled for your platform. You can "
+                            "experiment with EnableZEBinary environmental "
+                            "flag to turn it on.",
+                            OutputArgs);
+                        return false;
+                    }
+
+                    ShaderHash hash = ShaderHashOCL(
+                        reinterpret_cast<const UINT*>(InputArgs.pInput),
+                        InputArgs.InputSize / 4);
+                    std::string errorMessage;
+
+                    // Temporary workaround for invoke_sycl case.
+                    std::reverse(SPIRVToLink.begin(), SPIRVToLink.end());
+
+                    success = VLD::TranslateBuildSPMDAndESIMD(
+                        SPIRVToLink, &OutputArgs, TB_DATA_FORMAT_SPIR_V,
+                        Context.platform, profilingTimerResolution, hash,
+                        errorMessage);
+
+                    if (!success)
+                    {
+                        SetErrorMessage(errorMessage, OutputArgs);
+                        return false;
+                    }
+                }
+                else
+                {
+                    InputArgs.pInput = OutputString.data();
+                    InputArgs.InputSize = OutputString.size();
+                    success = TC::TranslateBuild(
+                        &InputArgs, &OutputArgs, TB_DATA_FORMAT_LLVM_BINARY,
+                        Context.platform, profilingTimerResolution);
+                    InputArgs.pInput = nullptr;
+                    InputArgs.InputSize = 0;
+                }
+            }
             else
             {
+                IGC_ASSERT_MESSAGE(0, "Unrecognized output format when processing ELF input");
                 success = false;
             }
         }
@@ -1000,6 +1012,11 @@ void RebuildGlobalAnnotations(IGC::OpenCLProgramContext& oclContext, Module* pKe
     // Remove old "llvm.global.annotations" that refers to kernels not requiring recompilation
     globalAnnotations->eraseFromParent();
 
+    if (newGlobalAnnotations.empty())
+    {
+        return;
+    }
+
     // Create new "llvm.global.annotations" that refers only to kernels that need to be recompiled
     Constant* Array =
         ConstantArray::get(ArrayType::get(newGlobalAnnotations[0]->getType(),
@@ -1168,6 +1185,7 @@ bool TranslateBuildSPMD(
     llvm::Module* pKernelModule = nullptr;
     LLVMContextWrapper* llvmContext = new LLVMContextWrapper;
     RegisterComputeErrHandlers(*llvmContext);
+    RegisterErrHandlers();
 
     if (IGC_IS_FLAG_ENABLED(ShaderDumpEnable))
     {
@@ -1298,7 +1316,7 @@ bool TranslateBuildSPMD(
                          IGC_IS_FLAG_ENABLED(CompileOneAtTime);
     // set retry manager
     bool retry = false;
-    oclContext.m_retryManager.Enable();
+    oclContext.m_retryManager.Enable(ShaderType::OPENCL_SHADER);
     do
     {
         llvm::TinyPtrVector<const llvm::Function*> kernelFunctions;
@@ -1663,12 +1681,12 @@ bool TranslateBuild(
 {
     ShaderHash inputShHash;
     if (IGC_IS_FLAG_ENABLED(EnableKernelNamesBasedHash))
+    {
         // Create the hash based on kernel names.
         // This takes the names and concatenates them into a string
         // which is then used to calculate the hash
-    {
-        llvm::Expected<VLD::SPVMetadata> parsedInput = VLD::SpvSplitter().Parse(pInputArgs->pInput, pInputArgs->InputSize);
-        std::vector<std::string> &entryPoints = parsedInput->EntryPointNames;
+        StringRef inputBin(pInputArgs->pInput, pInputArgs->InputSize);
+        const std::vector<std::string> &entryPoints = IGC::SPIRVParser::getEntryPointNames(inputBin);
 
         std::string entryPointsString = std::accumulate(entryPoints.begin(), entryPoints.end(), std::string(""));
 
