@@ -26,6 +26,8 @@ SPDX-License-Identifier: MIT
 #include "cif/macros/enable.h"
 #include <spirv-tools/libspirv.h>
 
+#include "Compiler/CISACodeGen/OpenCLKernelCodeGen.hpp"
+
 namespace TC{
 
 // Taken from dllInterfaceCompute
@@ -67,6 +69,11 @@ bool TranslateBuildVC(
   float profilingTimerResolution,
   const ShaderHash& inputShHash);
 
+void RebuildGlobalAnnotations(
+  IGC::OpenCLProgramContext& oclContext,
+  llvm::Module* pKernelModule
+);
+
 bool ReadSpecConstantsFromSPIRV(
     std::istream &IS,
     std::vector<std::pair<uint32_t, uint32_t>> &OutSCInfo);
@@ -263,16 +270,22 @@ CIF_DECLARE_INTERFACE_PIMPL(IgcOclTranslationCtx) : CIF::PimplBase
         std::string RegKeysFlagsFromOptions;
         if (inputArgs.pOptions != nullptr)
         {
-            const std::string optionsWithFlags = inputArgs.pOptions;
-            std::size_t found = optionsWithFlags.find("-igc_opts");
-            if (found != std::string::npos)
+            std::string_view optionsWithFlags = inputArgs.pOptions;
+
+            // check if there are more instances of '-igc_opts'
+            while (!optionsWithFlags.empty())
             {
+                std::size_t found = optionsWithFlags.find("-igc_opts");
+                if (found == std::string::npos)
+                    break;
+
                 std::size_t foundFirstSingleQuote = optionsWithFlags.find('\'', found);
                 std::size_t foundSecondSingleQuote = optionsWithFlags.find('\'', foundFirstSingleQuote + 1);
                 if (foundFirstSingleQuote != std::string::npos && foundSecondSingleQuote)
                 {
-                    RegKeysFlagsFromOptions = optionsWithFlags.substr(foundFirstSingleQuote + 1, (foundSecondSingleQuote - foundFirstSingleQuote - 1));
+                    RegKeysFlagsFromOptions += optionsWithFlags.substr(foundFirstSingleQuote + 1, (foundSecondSingleQuote - foundFirstSingleQuote - 1));
                     RegKeysFlagsFromOptions = RegKeysFlagsFromOptions + ',';
+                    optionsWithFlags = optionsWithFlags.substr(foundSecondSingleQuote + 1, optionsWithFlags.length());
                 }
             }
         }
@@ -389,6 +402,38 @@ CIF_DECLARE_INTERFACE_PIMPL(IgcOclTranslationCtx) : CIF::PimplBase
         return outputInterface.release();
     }
 
+    OclTranslationOutputBase* GetErrorOutput(CIF::Version_t outVersion, unsigned int code) const
+    {
+        auto outputInterface = CIF::RAII::UPtr(CIF::InterfaceCreator<OclTranslationOutput>::CreateInterfaceVer(outVersion, this->outType));
+        if (outputInterface == nullptr) {
+            return nullptr; // OOM
+        }
+        const char* outputMessage;
+        switch (code) {
+#if defined(WIN32)
+        case EXCEPTION_ACCESS_VIOLATION:
+            outputMessage = "IGC: Internal Compiler Error: Access violation";
+            break;
+        case EXCEPTION_DATATYPE_MISALIGNMENT:
+            outputMessage = "IGC: Internal Compiler Error: Datatype misalignement";
+            break;
+        case EXCEPTION_FLT_DIVIDE_BY_ZERO:
+        case EXCEPTION_INT_DIVIDE_BY_ZERO:
+            outputMessage = "IGC: Internal Compiler Error: Divide by zero";
+            break;
+        case EXCEPTION_STACK_OVERFLOW:
+            outputMessage = "IGC: Internal Compiler Error: Stack overflow";
+            break;
+#endif
+        default:
+            outputMessage = "IGC: Internal Compiler Error: Signal caught";
+        }
+        if (outputInterface->GetImpl()->SetError(TranslationErrorType::Internal, outputMessage)) {
+            return outputInterface.release();
+        }
+        return nullptr;
+    }
+
 protected:
     CIF_PIMPL(IgcOclDeviceCtx) &globalState;
     CodeType::CodeType_t inType;
@@ -400,3 +445,27 @@ CIF_DEFINE_INTERFACE_TO_PIMPL_FORWARDING_CTOR_DTOR(IgcOclTranslationCtx);
 }
 
 #include "cif/macros/disable.h"
+
+#if defined(NDEBUG)
+#if defined(WIN32)
+int ex_filter(unsigned int code, struct _EXCEPTION_POINTERS* ep);
+
+#define EX_GUARD_BEGIN                                                        \
+__try                                                                         \
+{                                                                             \
+
+#define EX_GUARD_END                                                          \
+}                                                                             \
+__except (ex_filter(GetExceptionCode(), GetExceptionInformation()))           \
+{                                                                             \
+    res = CIF_GET_PIMPL()->GetErrorOutput(outVersion, GetExceptionCode());    \
+}                                                                             \
+
+#else
+#define EX_GUARD_BEGIN
+#define EX_GUARD_END
+#endif
+#else
+#define EX_GUARD_BEGIN
+#define EX_GUARD_END
+#endif
\ No newline at end of file
