@@ -12,6 +12,7 @@ SPDX-License-Identifier: MIT
 #include "Compiler/CISACodeGen/CVariable.hpp"
 #include "Compiler/CISACodeGen/PatternMatchPass.hpp"
 #include "Compiler/CISACodeGen/helper.h"
+#include "Compiler/CISACodeGen/GenCodeGenModule.h"
 #include "visa_wa.h"
 #include "inc/common/sku_wa.h"
 
@@ -114,7 +115,7 @@ namespace IGC
     class CEncoder
     {
     public:
-        void InitEncoder(bool canAbortOnSpill, bool hasStackCall, bool hasInlineAsmCall, bool hasAdditionalVisaAsmToLink, VISAKernel* prevKernel);
+        void InitEncoder(bool canAbortOnSpill, bool hasStackCall, bool hasInlineAsmCall, bool hasAdditionalVisaAsmToLink, int numThreadsPerEU, VISAKernel* prevKernel);
         void InitBuildParams(llvm::SmallVector<std::unique_ptr< char, std::function<void(char*)>>, 10> & params);
         void InitVISABuilderOptions(TARGET_PLATFORM VISAPlatform, bool canAbortOnSpill, bool hasStackCall, bool enableVISA_IR);
         SEncoderState CopyEncoderState();
@@ -131,18 +132,18 @@ namespace IGC
         void GetVISAPredefinedVar(CVariable* pVar, PreDefined_Vars var);
         void CreateVISAVar(CVariable* var);
         void DeclareInput(CVariable* var, uint offset, uint instance);
+        void DeclarePred(CVariable* var, uint offset);
         void MarkAsOutput(CVariable* var);
         void MarkAsPayloadLiveOut(CVariable* var);
-        void Compile(bool hasSymbolTable = false);
+        void Compile(bool hasSymbolTable, GenXFunctionGroupAnalysis*& pFGA);
         std::string GetShaderName();
-        void ReportCompilerStatistics(VISAKernel* pMainKernel, SProgramOutput* pOutput);
-        int GetThreadCount(SIMDMode simdMode);
 
         CEncoder();
         ~CEncoder();
         void SetProgram(CShader* program);
         void Jump(CVariable* flag, uint label);
-        void Label(uint label);
+        void AddLabel(uint label);
+        void AddDivergentResourceLoopLabel(uint label);
         uint GetNewLabelID(const CName &name);
         void DwordAtomicRaw(AtomicOp atomic_op,
             const ResourceDescriptor& bindingTableIndex,
@@ -165,7 +166,6 @@ namespace IGC
         void Cmp(e_predicate p, CVariable* dst, CVariable* src0, CVariable* src1);
         void Select(CVariable* flag, CVariable* dst, CVariable* src0, CVariable* src1);
         void GenericAlu(e_opcode opcode, CVariable* dst, CVariable* src0, CVariable* src1, CVariable* src2 = nullptr);
-        void URBWrite(CVariable* src, const int payloadElementOffset, CVariable* offset, CVariable* urbHandle, CVariable* dynamicMask);
         void Send(CVariable* dst, CVariable* src, uint exDesc, CVariable* messDescriptor, bool isSendc = false);
         void Send(CVariable* dst, CVariable* src, uint ffid, CVariable* exDesc, CVariable* messDescriptor, bool isSendc = false);
         void Sends(CVariable* dst, CVariable* src0, CVariable* src1, uint ffid, CVariable* exDesc, CVariable* messDescriptor, bool isSendc = false, bool hasEOT = false);
@@ -255,18 +255,20 @@ namespace IGC
         static LSC_DATA_ELEMS LSC_GetElementNum(unsigned eNum);
         static LSC_ADDR_TYPE getLSCAddrType(const ResourceDescriptor * resource);
         static LSC_ADDR_TYPE getLSCAddrType(e_predefSurface surfaceType);
-        void LSC_LoadGather(
-            LSC_OP subOp, CVariable* dst, CVariable* offset,
-            LSC_DATA_SIZE elemSize, LSC_DATA_ELEMS numElems,
-            unsigned blockOffset, ResourceDescriptor* resource,
-            LSC_ADDR_SIZE addr_size, LSC_DATA_ORDER data_order,
-            int immOffset, LSC_CACHE_OPTS cacheOpts);
-        void LSC_StoreScatter(
-            LSC_OP subOp, CVariable * src, CVariable * offset,
-            LSC_DATA_SIZE elemSize, LSC_DATA_ELEMS numElems,
-            unsigned blockOffset, ResourceDescriptor * resource,
-            LSC_ADDR_SIZE addr_size, LSC_DATA_ORDER data_order,
-            int immOffset, LSC_CACHE_OPTS cacheOpts);
+        void LSC_LoadGather(LSC_OP subOp, CVariable *dst,
+                            CVariable *offset, LSC_DATA_SIZE elemSize,
+                            LSC_DATA_ELEMS numElems, unsigned blockOffset,
+                            ResourceDescriptor *resource,
+                            LSC_ADDR_SIZE addr_size, LSC_DATA_ORDER data_order,
+                            int immOffset, LSC_CACHE_OPTS cacheOpts);
+        void LSC_StoreScatter(LSC_OP subOp,
+                              CVariable *src, CVariable *offset,
+                              LSC_DATA_SIZE elemSize, LSC_DATA_ELEMS numElems,
+                              unsigned blockOffset,
+                              ResourceDescriptor *resource,
+                              LSC_ADDR_SIZE addr_size,
+                              LSC_DATA_ORDER data_order, int immOffset,
+                              LSC_CACHE_OPTS cacheOpts);
         void LSC_LoadBlock1D(
             CVariable* dst, CVariable* offset,
             LSC_DATA_SIZE elemSize, LSC_DATA_ELEMS numElems,
@@ -299,12 +301,14 @@ namespace IGC
         void NamedBarrier(e_barrierKind BarrierKind, CVariable* src0, CVariable* src1);
         void LSC_TypedReadWrite(
             LSC_OP subOp, ResourceDescriptor* resource,
-            CVariable* pU, CVariable* pV, CVariable* pR, CVariable* pLOD,
+            CVariable* pU, CVariable* pV, CVariable* pR,
+            CVariable* pLOD,
             CVariable* pSrcDst,
             unsigned elemSize, unsigned numElems,
             LSC_ADDR_SIZE addr_size, int chMask,
             LSC_CACHE_OPTS cacheOpts = { LSC_CACHING_DEFAULT, LSC_CACHING_DEFAULT });
 
+        void AppendBreakpoint();
         void ScatterA64(CVariable* val, CVariable* offset, unsigned elementSize, unsigned numElems);
         void ByteGather(CVariable* dst, const ResourceDescriptor& resource, CVariable* offset, unsigned elementSize, unsigned numElems);
         void ByteScatter(CVariable* src, const ResourceDescriptor& resource, CVariable* offset, unsigned elementSize, unsigned numElems);
@@ -374,8 +378,8 @@ namespace IGC
         inline void IShr(CVariable* dst, CVariable* src0, CVariable* src1);
         inline void Min(CVariable* dst, CVariable* src0, CVariable* src1);
         inline void Max(CVariable* dst, CVariable* src0, CVariable* src1);
-        inline void UAddC(CVariable* dst, CVariable* src0, CVariable* src1);
-        inline void USubB(CVariable* dst, CVariable* src0, CVariable* src1);
+        inline void UAddC(CVariable* dst, CVariable* dstCarryBorrow, CVariable* src0, CVariable* src1);
+        inline void USubB(CVariable* dst, CVariable* dstCarryBorrow, CVariable* src0, CVariable* src1);
         inline void IEEESqrt(CVariable* dst, CVariable* src0);
         inline void IEEEDivide(CVariable* dst, CVariable* src0, CVariable* src1);
         void AddPair(CVariable* Lo, CVariable* Hi, CVariable* L0, CVariable* H0, CVariable* L1, CVariable* H1 = nullptr);
@@ -506,6 +510,9 @@ namespace IGC
 
         std::string GetUniqueInlineAsmLabel();
 
+        bool IsVisaCompiledSuccessfully() const { return m_vIsaCompileStatus == VISA_SUCCESS; }
+        bool IsVisaCompileStatusFailure() const { return m_vIsaCompileStatus == VISA_FAILURE; }
+
     private:
         // helper functions
         VISA_VectorOpnd* GetSourceOperand(CVariable* var, const SModifier& mod);
@@ -516,7 +523,7 @@ namespace IGC
         VISA_PredOpnd* GetFlagOperand(const SFlag& flag);
         VISA_StateOpndHandle* GetVISASurfaceOpnd(e_predefSurface surfaceType, CVariable* bti);
         VISA_StateOpndHandle* GetVISASurfaceOpnd(const ResourceDescriptor& resource);
-        VISA_LabelOpnd* GetLabel(uint label);
+        VISA_LabelOpnd* GetOrCreateLabel(uint label, VISA_Label_Kind kind = LABEL_BLOCK);
         VISA_LabelOpnd* GetFuncLabel(llvm::Function* F);
         void InitLabelMap(const llvm::Function* F);
         CName CreateVisaLabelName(const llvm::StringRef &L = "");
@@ -553,7 +560,9 @@ namespace IGC
             CVariable* src0 = nullptr,
             CVariable* src1 = nullptr,
             CVariable* src2 = nullptr);
-        void CarryBorrowArith(ISA_Opcode opcode, CVariable* dst, CVariable* src0, CVariable* src1);
+        // dst may be special `null` CVariable, if only carry/borrow bits matter
+        void CarryBorrowArith(ISA_Opcode opcode, CVariable* dst, CVariable* dstCarryBorrow,
+            CVariable* src0, CVariable* src1);
         void ScatterGather(
             ISA_Opcode opcode,
             CVariable* srcdst,
@@ -624,7 +633,8 @@ namespace IGC
         void CreateRelocationTable(VISAKernel* pMainKernel, SProgramOutput::RelocListTy& relocations);
 
         // CreateFuncAttributeTable
-        void CreateFuncAttributeTable(VISAKernel* pMainKernel, void*& buffer, unsigned& bufferSize, unsigned& tableEntries, SProgramOutput::FuncAttrListTy& attrs);
+        void CreateFuncAttributeTable(VISAKernel *pMainKernel,
+                                      GenXFunctionGroupAnalysis *pFga);
 
         // CreateGlobalHostAccessTable
         typedef std::vector<vISA::HostAccessEntry> HostAccessList;
@@ -684,6 +694,48 @@ namespace IGC
         void SaveOption(vISAOptions option, const char* val);
         void SetBuilderOptions(VISABuilder* pbuilder);
 
+    private:
+        // helper functions for compile flow
+        /// shaderOverrideVISAFirstPass - pre-process shader overide dir for visa shader override.
+        /// return if there are visa files to be overriden
+        bool shaderOverrideVISAFirstPass(
+            std::vector<std::string> &visaOverrideFiles, std::string& kernelName);
+
+        /// shaderOverrideVISASecondPassOrInlineAsm - handle visa inputs of shader override
+        /// or inline asm
+        /// Return the VISAKernel built from overrided visa or inline asm. Return nullptr if
+        /// failed to create VISAKernel
+        VISAKernel* shaderOverrideVISASecondPassOrInlineAsm(
+            bool visaAsmOverride, bool hasSymbolTable, bool emitVisaOnly,
+            const std::vector<const char *> *additionalVISAAsmToLink,
+            const std::vector<std::string> &visaOverrideFiles,
+            const std::string kernelName);
+
+        // setup m_retryManager according to jitinfo and other factors
+        void SetKernelRetryState(CodeGenContext* context, vISA::FINALIZER_INFO* jitInfo, GenXFunctionGroupAnalysis*& pFGA);
+
+        /// create symbol tables and GlobalHostAccessTable according to if it's zebin
+        /// or patch-token formats
+        void createSymbolAndGlobalHostAccessTables(bool hasSymbolTable,
+                                                   VISAKernel &pMainKernel);
+
+        /// create relocation according to if it's zebin or patch-token formats
+        void createRelocationTables(VISAKernel &pMainKernel);
+
+        /// Estimate vISA stack size according to FunctionGroup information.
+        /// Conservatively increase the stack size when having recursive or indirect
+        /// calls if the given function is a Function Group Head. Function group
+        /// heads are kernels/entry functions those their stack size will be used to
+        /// reported back to the Runtime, so we set a conservative value. For other
+        /// non-head functions, we set the precise stack usage of the function.
+        ///
+        /// For group head functions, when having the stack call, the spillSize
+        /// returned by vISA is the sum of required stack size of all potential
+        /// callee and without considering recursive or indirect calls.
+        uint32_t getSpillMemSizeWithFG(const llvm::Function& curFunc,
+                                       uint32_t curSize,
+                                       GenXFunctionGroupAnalysis *fga);
+
     protected:
         // encoder states
         SEncoderState m_encoderState = {};
@@ -749,15 +801,16 @@ namespace IGC
         CName currFunctionName;
 
         /// Keep a map between a function and its label, per kernel state.
-        llvm::SmallDenseMap<llvm::Function*, VISA_LabelOpnd*> funcLabelMap;
+        llvm::MapVector<llvm::Function*, VISA_LabelOpnd*> funcLabelMap;
         /// Keep a map between a stack-called function and the corresponding vISA function
-        llvm::SmallDenseMap<llvm::Function*, VISAFunction*> stackFuncMap;
+        llvm::MapVector<llvm::Function*, VISAFunction*> stackFuncMap;
 
         // dummy variables
         VISA_SurfaceVar* dummySurface;
         VISA_SamplerVar* samplervar;
 
         CShader* m_program;
+        int m_vIsaCompileStatus = VISA_FAILURE;
 
         // Keep a map between a function and its per-function attributes needed for function pointer support
         struct FuncAttrib
@@ -767,7 +820,7 @@ namespace IGC
             unsigned argumentStackSize = 0;
             unsigned allocaStackSize = 0;
         };
-        llvm::SmallDenseMap<llvm::Function*, FuncAttrib> funcAttributeMap;
+        llvm::MapVector<llvm::Function*, FuncAttrib> funcAttributeMap;
 
     public:
         // Used by EmitVISAPass to set function attributes
@@ -997,14 +1050,14 @@ namespace IGC
         MinMax(CISA_DM_FMAX, dst, src0, src1);
     }
 
-    inline void CEncoder::UAddC(CVariable* dst, CVariable* src0, CVariable* src1)
+    inline void CEncoder::UAddC(CVariable* dst, CVariable* dstCarryBorrow, CVariable* src0, CVariable* src1)
     {
-        CarryBorrowArith(ISA_ADDC, dst, src0, src1);
+        CarryBorrowArith(ISA_ADDC, dst, dstCarryBorrow, src0, src1);
     }
 
-    inline void CEncoder::USubB(CVariable* dst, CVariable* src0, CVariable* src1)
+    inline void CEncoder::USubB(CVariable* dst, CVariable* dstCarryBorrow, CVariable* src0, CVariable* src1)
     {
-        CarryBorrowArith(ISA_SUBB, dst, src0, src1);
+        CarryBorrowArith(ISA_SUBB, dst, dstCarryBorrow, src0, src1);
     }
 
     inline void CEncoder::LoadMS(EOPCODE subOpcode, uint writeMask, CVariable* offset,
