@@ -135,6 +135,9 @@ namespace IGC
         /// Return true if IGC needs FP64 emulation. (Valid if platform has no double inst.)
         virtual bool NeedFP64(PRODUCT_FAMILY productFamily) const { IGC_UNUSED(productFamily); return false; }
 
+        /// Needs fp64 to fp16 conversion emulation
+        virtual bool NeedFP64toFP16Conv() const { return false; }
+
         /// Needs IEEE fp64 div/sqrt
         virtual bool NeedFP64DivSqrt() const { return false; }
 
@@ -168,9 +171,6 @@ namespace IGC
         /// Configure vISA pre-RA scheduler. Not tested on all APIs
         virtual unsigned getVISAPreRASchedulerCtrl() const { return 4; }
 
-        /// Turn on sampler clustering. Hopefully VISA PreRA scheduler with latency hiding can replace it.
-        virtual bool enableSampleClustering() const { return true; }
-
         /// Make sure optimization are consistent to avoid Z-fighting issue
         virtual bool PreventZFighting() const { return false; }
 
@@ -313,6 +313,7 @@ namespace IGC
         virtual bool UseScratchSpaceForATSPlus() const { return false; }
         /// Enables HWGenerateThreadID from API level. To help debug, we must enable it from both API level AND IGC Core level.
         virtual bool SupportHWGenerateTID() const { return false; }
+        virtual bool supportHWDispatchWalkOrder() const { return false; }
         // Enables the use of simple push constants when on platforms with local (device) memory
         virtual bool supportsSimplePushForLocalMem() const { return false; }
         // disable dual8 with discard
