@@ -13,6 +13,8 @@ SPDX-License-Identifier: MIT
 #include "llvm/Pass.h"
 #include <llvm/IR/InstVisitor.h>
 #include <llvm/ADT/StringRef.h>
+#include "llvm/ADT/DenseSet.h"
+#include "llvm/Support/ScaledNumber.h"
 #include "common/LLVMWarningsPop.hpp"
 #include <cstddef>
 #include "Probe/Assertion.h"
@@ -71,15 +73,19 @@ namespace IGC {
             IGC_ASSERT(ECG.count(F));
             return static_cast<T*>(ECG[F]);
         }
-
+        void performImplArgsAnalysis();
         void initializeTopologicalVisit(llvm::Function* root, std::unordered_map<void*, uint32_t>& FunctionsInKernel, std::deque<void*>& BottomUpQueue, bool ignoreStackCallBoundary);
         uint32_t updateExpandedUnitSize(llvm::Function* F, bool ignoreStackCallBoundary);
+        void updateInlineCnt(llvm::Function* root);
         uint32_t bottomUpHeuristic(llvm::Function* F, uint32_t& stackCall_cnt);
         void partitionKernel();
+        void runStaticAnalysis();
         void reduceCompilationUnitSize();
         void trimCompilationUnit(llvm::SmallVector<void*, 64> &unitHeads, uint32_t threshold, bool ignoreStackCallBoundary);
+        void performTrimming(llvm::Function *head, llvm::SmallVector<void*, 64>& functions_to_trim, uint32_t threshold, bool ignoreStackCallBoundary);
         uint32_t getMaxUnitSize();
-        void getFunctionsToTrim(llvm::Function* root, llvm::SmallVector<void*, 64> &functions_to_trim, bool ignoreStackCallBoundary, uint32_t& func_cnt);
+        void getFunctionsToTrim(llvm::Function* root, llvm::SmallVector<void*, 64> &trimming_pool, bool ignoreStackCallBoundary, uint32_t& func_cnt);
+        void updateStaticFuncFreq();
 
         /// \brief The module being analyzed.
         llvm::Module* M;
@@ -104,6 +110,7 @@ namespace IGC {
         const llvm::StringRef GET_STAGE_IN_GRID_ORIGIN = "__builtin_IB_get_stage_in_grid_origin";
         const llvm::StringRef GET_STAGE_IN_GRID_SIZE = "__builtin_IB_get_stage_in_grid_size";
         const llvm::StringRef GET_SYNC_BUFFER = "__builtin_IB_get_sync_buffer";
+        const llvm::StringRef GET_ASSERT_BUFFER = "__builtin_IB_get_assert_buffer";
 
         bool HasRecursion;
         bool EnableSubroutine;
@@ -116,6 +123,8 @@ namespace IGC {
         //Functions that are assigned stackcalls
         llvm::SmallVector<void*, 64> stackCallFuncs;
         llvm::SmallVector<void*, 64> addressTakenFuncs;
+        llvm::ScaledNumber<uint64_t> threshold_func_freq;
+        llvm::ScaledNumber<uint64_t> thresholdForTrimming;
     };
 
     llvm::ModulePass* createEstimateFunctionSizePass();
