@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2021 Intel Corporation
+Copyright (C) 2017-2022 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -42,9 +42,9 @@ namespace IGC {
 
         ~LowerGEPForPrivMem() {}
 
-        virtual llvm::StringRef getPassName() const override
+        virtual StringRef getPassName() const override
         {
-            return "LowerGEPForPrivMem";
+            return IGCOpts::LowerGEPForPrivMemPass;
         }
 
         virtual void getAnalysisUsage(llvm::AnalysisUsage& AU) const override
@@ -152,6 +152,9 @@ bool LowerGEPForPrivMem::runOnFunction(llvm::Function& F)
     m_ctx = pCtxWrapper->getCodeGenContext();
     m_DT = &getAnalysis<DominatorTreeWrapperPass>().getDomTree();
 
+    if (isOptDisabledForFunction(m_ctx->getModuleMetaData(), getPassName(), &F))
+        return false;
+
     pMdUtils = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
     IGC_ASSERT(nullptr != pMdUtils);
     if (pMdUtils->findFunctionsInfoItem(&F) == pMdUtils->end_FunctionsInfo())
@@ -187,8 +190,6 @@ bool LowerGEPForPrivMem::runOnFunction(llvm::Function& F)
         }
     }
 
-    if (!allocaToHande.empty())
-        DumpLLVMIR(m_ctx, "AfterLowerGEP");
     // IR changed only if we had alloca instruction to optimize
     return !allocaToHande.empty();
 }
@@ -285,31 +286,29 @@ StatusPrivArr2Reg LowerGEPForPrivMem::CheckIfAllocaPromotable(llvm::AllocaInst*
     bool useAssumeUniform = pAlloca->getMetadata("UseAssumeUniform") != nullptr;
     unsigned int allocaSize = extractConstAllocaSize(pAlloca);
     unsigned int allowedAllocaSizeInBytes = MAX_ALLOCA_PROMOTE_GRF_NUM * 4;
+    unsigned int SIMDSize = numLanes(m_ctx->platform.getMinDispatchMode());
+
+    // consider GRF width in alloca register promotion limit
+    allowedAllocaSizeInBytes = allowedAllocaSizeInBytes * m_ctx->platform.getGRFSize() / 32;
 
     // scale alloc size based on the number of GRFs we have
     float grfRatio = m_ctx->getNumGRFPerThread() / 128.0f;
     allowedAllocaSizeInBytes = (uint32_t)(allowedAllocaSizeInBytes * grfRatio);
 
-    if (m_ctx->type == ShaderType::COMPUTE_SHADER)
-    {
-        ComputeShaderContext* ctx = static_cast<ComputeShaderContext*>(m_ctx);
-        SIMDMode simdMode = ctx->GetLeastSIMDModeAllowed();
-        unsigned d = simdMode == SIMDMode::SIMD32 ? 4 : 1;
-
-        allowedAllocaSizeInBytes = allowedAllocaSizeInBytes / d;
-    }
-    else if (m_ctx->type == ShaderType::OPENCL_SHADER)
+    if (m_ctx->type == ShaderType::OPENCL_SHADER)
     {
         FunctionInfoMetaDataHandle funcInfoMD = pMdUtils->getFunctionsInfoItem(m_pFunc);
         SubGroupSizeMetaDataHandle subGroupSize = funcInfoMD->getSubGroupSize();
         if (subGroupSize->hasValue())
         {
-            auto simdSize = (uint32_t)subGroupSize->getSIMD_size();
-            allowedAllocaSizeInBytes = (allowedAllocaSizeInBytes * 8) / simdSize;
+            SIMDSize = std::max((uint32_t)subGroupSize->getSIMD_size(), SIMDSize);
         }
+
+        allowedAllocaSizeInBytes = (allowedAllocaSizeInBytes * 8) / SIMDSize;
     }
     Type* baseType = nullptr;
-    if (!CanUseSOALayout(pAlloca, baseType))
+    bool allUsesAreVector = false;
+    if (!CanUseSOALayout(pAlloca, baseType, allUsesAreVector))
     {
         return StatusPrivArr2Reg::CannotUseSOALayout;
     }
@@ -319,9 +318,9 @@ StatusPrivArr2Reg LowerGEPForPrivMem::CheckIfAllocaPromotable(llvm::AllocaInst*
     }
     if (isUniformAlloca)
     {
-        // Heuristic: for uniform alloca we divide the size by 8 to adjust the pressure
+        // Heuristic: for uniform alloca we divide the size by SIMDSize to adjust the pressure
         // as they will be allocated as uniform array
-        allocaSize = iSTD::Round(allocaSize, SIMD_PRESSURE_MULTIPLIER) / SIMD_PRESSURE_MULTIPLIER;
+        allocaSize = iSTD::Round(allocaSize, SIMDSize) / SIMDSize;
     }
 
     if (useAssumeUniform || allocaSize <= IGC_GET_FLAG_VALUE(ByPassAllocaSizeHeuristic))
@@ -379,22 +378,25 @@ StatusPrivArr2Reg LowerGEPForPrivMem::CheckIfAllocaPromotable(llvm::AllocaInst*
     liverange.lowId = lowestAssignedNumber;
     liverange.highId = highestAssignedNumber;
     liverange.varSize = allocaSize;
+    liverange.LR = nullptr;
     m_promotedLiveranges.push_back(liverange);
     return StatusPrivArr2Reg::OK;
 }
 
-static bool CheckUsesForSOAAlyout(Instruction* I, bool& vectorSOA)
+static bool CheckUsesForSOAAlyout(Instruction* I, bool& vectorSOA, bool& allUsesAreVector)
 {
     for (Value::user_iterator use_it = I->user_begin(), use_e = I->user_end(); use_it != use_e; ++use_it)
     {
         if (GetElementPtrInst * gep = dyn_cast<GetElementPtrInst>(*use_it))
         {
-            if (CheckUsesForSOAAlyout(gep, vectorSOA))
+            if (CheckUsesForSOAAlyout(gep, vectorSOA, allUsesAreVector))
                 continue;
         }
         if (llvm::LoadInst * pLoad = llvm::dyn_cast<llvm::LoadInst>(*use_it))
         {
-            vectorSOA &= pLoad->getType()->isVectorTy();
+            bool isVectorLoad = pLoad->getType()->isVectorTy();
+            vectorSOA &= isVectorLoad;
+            allUsesAreVector &= isVectorLoad;
             if (!pLoad->isSimple())
                 return false;
         }
@@ -403,7 +405,9 @@ static bool CheckUsesForSOAAlyout(Instruction* I, bool& vectorSOA)
             if (!pStore->isSimple())
                 return false;
             llvm::Value* pValueOp = pStore->getValueOperand();
-            vectorSOA &= pStore->getValueOperand()->getType()->isVectorTy();
+            bool isVectorStore = pStore->getValueOperand()->getType()->isVectorTy();
+            vectorSOA &= isVectorStore;
+            allUsesAreVector &= isVectorStore;
             if (pValueOp == I)
             {
                 // GEP instruction is the stored value of the StoreInst (not supported case)
@@ -412,8 +416,8 @@ static bool CheckUsesForSOAAlyout(Instruction* I, bool& vectorSOA)
         }
         else if (llvm::BitCastInst * pBitCast = llvm::dyn_cast<llvm::BitCastInst>(*use_it))
         {
-            Type* baseT = GetBaseType(pBitCast->getType()->getPointerElementType());
-            Type* sourceType = GetBaseType(pBitCast->getOperand(0)->getType()->getPointerElementType());
+            Type* baseT = GetBaseType(IGCLLVM::getNonOpaquePtrEltTy(pBitCast->getType()));
+            Type* sourceType = GetBaseType(IGCLLVM::getNonOpaquePtrEltTy(pBitCast->getOperand(0)->getType()));
             if (pBitCast->use_empty())
             {
                 continue;
@@ -423,7 +427,7 @@ static bool CheckUsesForSOAAlyout(Instruction* I, bool& vectorSOA)
                 baseT->getScalarSizeInBits() == sourceType->getScalarSizeInBits())
             {
                 vectorSOA &= (unsigned int)baseT->getPrimitiveSizeInBits() == sourceType->getPrimitiveSizeInBits();
-                if (CheckUsesForSOAAlyout(pBitCast, vectorSOA))
+                if (CheckUsesForSOAAlyout(pBitCast, vectorSOA, allUsesAreVector))
                     continue;
             }
             else if (IsBitCastForLifetimeMark(pBitCast))
@@ -453,7 +457,7 @@ static bool CheckUsesForSOAAlyout(Instruction* I, bool& vectorSOA)
 }
 
 
-bool IGC::CanUseSOALayout(AllocaInst* I, Type*& base)
+bool IGC::CanUseSOALayout(AllocaInst* I, Type*& base, bool& allUsesAreVector)
 {
     // Do not allow SOA layout for vla which will be stored on the stack.
     // We don't support SOA layout for privates on stack at all so this is just to make
@@ -463,7 +467,7 @@ bool IGC::CanUseSOALayout(AllocaInst* I, Type*& base)
 
     // Don't even look at non-array allocas.
     // (extractAllocaDim can not handle them anyway, causing a crash)
-    llvm::Type* pType = I->getType()->getPointerElementType();
+    llvm::Type* pType = IGCLLVM::getNonOpaquePtrEltTy(I->getType());
     if (pType->isStructTy() && pType->getStructNumElements() == 1)
     {
         pType = pType->getStructElementType(0);
@@ -478,7 +482,8 @@ bool IGC::CanUseSOALayout(AllocaInst* I, Type*& base)
     if (!(base->getScalarType()->isFloatingPointTy() || base->getScalarType()->isIntegerTy()))
         return false;
     bool vectorSOA = true;
-    bool useSOA = CheckUsesForSOAAlyout(I, vectorSOA);
+    allUsesAreVector = true;
+    bool useSOA = CheckUsesForSOAAlyout(I, vectorSOA, allUsesAreVector);
     if (!vectorSOA)
     {
         base = base->getScalarType();
@@ -559,7 +564,7 @@ public:
 void LowerGEPForPrivMem::handleAllocaInst(llvm::AllocaInst* pAlloca)
 {
     // Extract the Alloca size and the base Type
-    Type* pType = pAlloca->getType()->getPointerElementType();
+    Type* pType = IGCLLVM::getNonOpaquePtrEltTy(pAlloca->getType());
     Type* pBaseType = GetBaseType(pType)->getScalarType();
     IGC_ASSERT(pBaseType);
     llvm::AllocaInst* pVecAlloca = createVectorForAlloca(pAlloca, pBaseType);
@@ -619,7 +624,7 @@ void TransposeHelper::handleGEPInst(
     //
     IRBuilder<> IRB(pGEP);
     Value* pScalarizedIdx = IRB.getInt32(0);
-    Type* T = pGEP->getPointerOperandType()->getPointerElementType();
+    Type* T = IGCLLVM::getNonOpaquePtrEltTy(pGEP->getPointerOperandType());
     for (unsigned i = 0, e = pGEP->getNumIndices(); i < e; ++i)
     {
         auto GepOpnd = IRB.CreateZExtOrTrunc(pGEP->getOperand(i + 1), IRB.getInt32Ty());
