@@ -304,6 +304,11 @@ bool hasSLMFence() const
     return (m_platformInfo.eRenderCoreFamily >= IGFX_GEN11_CORE);
 }
 
+bool hasIndependentSharedMemoryFenceFunctionality() const
+{
+    return (m_platformInfo.eProductFamily != IGFX_DG2);
+}
+
 bool supportRotateInstruction() const { return m_platformInfo.eRenderCoreFamily >= IGFX_GEN11_CORE; }
 bool supportLRPInstruction() const { return m_platformInfo.eRenderCoreFamily < IGFX_GEN11_CORE; }
 bool support16bitMSAAPayload() const { return m_platformInfo.eRenderCoreFamily >= IGFX_GEN11_CORE; }
@@ -724,7 +729,7 @@ bool hasFullInt64() const
 
 bool hasInt64DstMul() const
 {
-    return m_platformInfo.eProductFamily == IGFX_PVC;
+    return (m_platformInfo.eProductFamily == IGFX_PVC && m_platformInfo.usRevId < REVISION_B);
 }
 
 bool hasExecSize16DPAS() const
