@@ -83,6 +83,7 @@ SPDX-License-Identifier: MIT
 #include "Compiler/Optimizer/OpenCLPasses/UnreachableHandling/UnreachableHandling.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/WIFuncs/WIFuncResolution.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/ScalarArgAsPointer/ScalarArgAsPointer.hpp"
+#include "Compiler/Optimizer/OpenCLPasses/GEPLoopStrengthReduction/GEPLoopStrengthReduction.hpp"
 #include "Compiler/Optimizer/MCSOptimization.hpp"
 #include "Compiler/Optimizer/GatingSimilarSamples.hpp"
 #include "Compiler/Optimizer/IntDivConstantReduction.hpp"
@@ -355,6 +356,15 @@ static void UpdateInstTypeHint(CodeGenContext& ctx)
 // forward declaration
 llvm::ModulePass* createPruneUnusedArgumentsPass();
 
+static bool useStatelessToStateful(CodeGenContext& ctx)
+{
+    return (ctx.m_instrTypes.hasLoadStore &&
+        ctx.m_DriverInfo.SupportsStatelessToStatefulBufferTransformation() &&
+        !ctx.getModuleMetaData()->compOpt.GreaterThan4GBBufferRequired &&
+        IGC_IS_FLAG_ENABLED(EnableStatelessToStateful) &&
+        !ctx.m_instrTypes.hasInlineAsmPointerAccess);
+}
+
 void AddLegalizationPasses(CodeGenContext& ctx, IGCPassManager& mpm, PSSignature* pSignature)
 {
     COMPILER_TIME_START(&ctx, TIME_CG_Add_Legalization_Passes);
@@ -705,12 +715,7 @@ void AddLegalizationPasses(CodeGenContext& ctx, IGCPassManager& mpm, PSSignature
         mpm.add(new PromoteStatelessToBindless());
     }
 
-    if (!isOptDisabled &&
-        ctx.m_instrTypes.hasLoadStore &&
-        ctx.m_DriverInfo.SupportsStatelessToStatefulBufferTransformation() &&
-        !ctx.getModuleMetaData()->compOpt.GreaterThan4GBBufferRequired &&
-        IGC_IS_FLAG_ENABLED(EnableStatelessToStateful) &&
-        !ctx.m_instrTypes.hasInlineAsmPointerAccess)
+    if (!isOptDisabled && useStatelessToStateful(ctx))
     {
         mpm.add(new StatelessToStateful());
     }
@@ -1377,8 +1382,8 @@ void OptimizeIR(CodeGenContext* const pContext)
 
         bool disableGOPT = ( (IsStage1FastestCompile(pContext->m_CgFlag, pContext->m_StagingCtx) ||
                                IGC_GET_FLAG_VALUE(ForceFastestSIMD)) &&
-                             ((IGC_GET_FLAG_VALUE(FastestS1Experiments) & FCEXP_DISABLE_GOPT) ||
-                               IGC_GET_FLAG_VALUE(FastestS1Experiments) == FCEXP_NO_EXPRIMENT ||
+                             ((FastestS1Options(pContext) & FCEXP_DISABLE_GOPT) ||
+                               FastestS1Options(pContext) == FCEXP_NO_EXPRIMENT ||
                                pContext->getModuleMetaData()->compOpt.DisableFastestGopt));
 
         if (pContext->m_instrTypes.hasMultipleBB && !disableGOPT)
@@ -1442,8 +1447,8 @@ void OptimizeIR(CodeGenContext* const pContext)
                 bool hasIndexTemp = (pContext->m_indexableTempSize[0] > 0);
                 bool disableLoopUnrollStage1 =
                     IsStage1FastestCompile(pContext->m_CgFlag, pContext->m_StagingCtx) &&
-                       (IGC_GET_FLAG_VALUE(FastestS1Experiments) == FCEXP_NO_EXPRIMENT ||
-                        (IGC_GET_FLAG_VALUE(FastestS1Experiments) & FCEXP_DISABLE_UNROLL));
+                       (FastestS1Options(pContext) == FCEXP_NO_EXPRIMENT ||
+                        (FastestS1Options(pContext) & FCEXP_DISABLE_UNROLL));
                 if ((LoopUnrollThreshold > 0 &&
                      unroll &&
                      !disableLoopUnrollStage1)
@@ -1485,6 +1490,15 @@ void OptimizeIR(CodeGenContext* const pContext)
                         mpm.add(createSROAPass());
                     }
                 }
+
+                if (pContext->type == ShaderType::OPENCL_SHADER &&
+                    pContext->platform.isCoreChildOf(IGFX_XE_HPC_CORE) &&
+                    !useStatelessToStateful(*pContext) &&
+                    pContext->m_retryManager.IsFirstTry())
+                {
+                    mpm.add(createGEPLoopStrengthReductionPass(IGC_IS_FLAG_ENABLED(allowLICM) &&
+                            pContext->m_retryManager.AllowLICM()));
+                }
             }
 
             // Note:
@@ -1629,8 +1643,8 @@ void OptimizeIR(CodeGenContext* const pContext)
                     // Enable loop unrolling for stage 1 for now due to persisting regressions
                     bool disableLoopUnrollStage1 =
                         IsStage1FastestCompile(pContext->m_CgFlag, pContext->m_StagingCtx) &&
-                           (//IGC_GET_FLAG_VALUE(FastestS1Experiments) == FCEXP_NO_EXPRIMENT ||
-                            (IGC_GET_FLAG_VALUE(FastestS1Experiments) & FCEXP_DISABLE_UNROLL));
+                           (//FastestS1Options(pContext) == FCEXP_NO_EXPRIMENT ||
+                            (FastestS1Options(pContext) & FCEXP_DISABLE_UNROLL));
                     if ((LoopUnrollThreshold > 0 &&
                          unroll &&
                          !disableLoopUnrollStage1)
