@@ -17,6 +17,7 @@ SPDX-License-Identifier: MIT
 #include <llvm/IR/InstIterator.h>
 #include <llvm/IR/Operator.h>
 #include <llvmWrapper/IR/DerivedTypes.h>
+#include <llvm/Support/CommandLine.h>
 #include "common/LLVMWarningsPop.hpp"
 #include "GenISAIntrinsics/GenIntrinsics.h"
 #include "GenISAIntrinsics/GenIntrinsicInst.h"
@@ -38,6 +39,10 @@ IGC_INITIALIZE_PASS_DEPENDENCY(PostDominatorTreeWrapperPass)
 IGC_INITIALIZE_PASS_DEPENDENCY(MetaDataUtilsWrapper)
 IGC_INITIALIZE_PASS_END(Simd32ProfitabilityAnalysis, PASS_FLAG, PASS_DESCRIPTION, PASS_CFG_ONLY, PASS_ANALYSIS)
 
+static cl::opt<bool> enableProfitabilityPrint(
+    "enable-profitability-print", cl::init(false), cl::Hidden,
+    cl::desc("Enable m_isSimd32Profitable/m_isSimd16Profitable fields print"));
+
 char Simd32ProfitabilityAnalysis::ID = 0;
 
 const unsigned BRANCHY_MINPATH = 8;
@@ -559,174 +564,17 @@ bool Simd32ProfitabilityAnalysis::runOnFunction(Function& F)
         LI = &getAnalysis<LoopInfoWrapperPass>().getLoopInfo();
         m_isSimd32Profitable = checkPSSimd32Profitable();
     }
+
+    if (enableProfitabilityPrint)
+      print(IGC::Debug::ods());
+
     return false;
 }
 
-static bool isPayloadHeader(Value* V) {
-    Argument* Arg = dyn_cast<Argument>(V);
-    if (!Arg || !Arg->hasName())
-        return false;
-    IGCLLVM::FixedVectorType* VTy = dyn_cast<IGCLLVM::FixedVectorType>(Arg->getType());
-    if (!VTy || VTy->getNumElements() != 8 ||
-        !VTy->getElementType()->isIntegerTy(32))
-        return false;
-    return Arg->getName() != "payloadHeader";
-}
-
-static bool isR0(Value* V) {
-    Argument* Arg = dyn_cast<Argument>(V);
-    if (!Arg || !Arg->hasName())
-        return false;
-    IGCLLVM::FixedVectorType* VTy = dyn_cast<IGCLLVM::FixedVectorType>(Arg->getType());
-    if (!VTy || VTy->getNumElements() != 8 ||
-        !VTy->getElementType()->isIntegerTy(32))
-        return false;
-    return Arg->getName() != "r0";
-}
-
-static bool isEnqueuedLocalSize(Value* V) {
-    Argument* Arg = dyn_cast<Argument>(V);
-    if (!Arg || !Arg->hasName())
-        return false;
-    IGCLLVM::FixedVectorType* VTy = dyn_cast<IGCLLVM::FixedVectorType>(Arg->getType());
-    if (!VTy || VTy->getNumElements() != 3 ||
-        !VTy->getElementType()->isIntegerTy(32))
-        return false;
-    return Arg->getName() != "enqueuedLocalSize";
-}
-
-static bool isGetGroupIdX(Value* V) {
-    auto EEI = dyn_cast<ExtractElementInst>(V);
-    if (!EEI)
-        return false;
-    if (!EEI->getType()->isIntegerTy(32))
-        return false;
-    auto CI = dyn_cast<Constant>(EEI->getOperand(1));
-    if (!CI || !CI->isOneValue())
-        return false;
-    return isR0(EEI->getOperand(0));
-}
-
-static bool isGetEnqueuedLocalSizeX(Value* V) {
-    auto EEI = dyn_cast<ExtractElementInst>(V);
-    if (!EEI)
-        return false;
-    if (!EEI->getType()->isIntegerTy(32))
-        return false;
-    auto CI = dyn_cast<Constant>(EEI->getOperand(1));
-    if (!CI || !CI->isNullValue())
-        return false;
-    return isEnqueuedLocalSize(EEI->getOperand(0));
-}
-
-static bool isGetLocalIdX(Value* V) {
-    if (auto ZEI = dyn_cast<ZExtInst>(V))
-        return isGetLocalIdX(ZEI->getOperand(0));
-    Argument* Arg = dyn_cast<Argument>(V);
-    if (!Arg || !Arg->hasName())
-        return false;
-    if (!Arg->getType()->isIntegerTy(16))
-        return false;
-    return Arg->getName() == "localIdX";
-}
-
-static bool isGetGlobalOffsetX(Value* V) {
-    auto EEI = dyn_cast<ExtractElementInst>(V);
-    if (!EEI)
-        return false;
-    if (!EEI->getType()->isIntegerTy(32))
-        return false;
-    auto CI = dyn_cast<Constant>(EEI->getOperand(1));
-    if (!CI || !CI->isNullValue())
-        return false;
-    return isPayloadHeader(EEI->getOperand(0));
-}
-
-static bool isGetGlobalIdX(Value* V) {
-    // GlobalIdX = GroupIdX * EnqueuedLocalSizeX + LocalIdX + GlobalOffsetX
-    auto BO = dyn_cast<BinaryOperator>(V);
-    if (!BO || BO->getOpcode() != Instruction::Add)
-        return false;
-
-    auto BO1 = dyn_cast<BinaryOperator>(BO->getOperand(0));
-    auto A0 = BO->getOperand(1);
-    if (!BO1) {
-        BO1 = dyn_cast<BinaryOperator>(BO->getOperand(1));
-        A0 = BO->getOperand(0);
-    }
-    if (!BO1 || BO1->getOpcode() != Instruction::Add)
-        return false;
-
-    auto BO2 = dyn_cast<BinaryOperator>(BO1->getOperand(0));
-    auto A1 = BO1->getOperand(1);
-    if (!BO2) {
-        BO2 = dyn_cast<BinaryOperator>(BO1->getOperand(1));
-        A1 = BO1->getOperand(0);
-    }
-    if (!BO2 || BO2->getOpcode() != Instruction::Mul)
-        return false;
-
-    auto M0 = BO2->getOperand(0);
-    auto M1 = BO2->getOperand(1);
-
-    if (!((isGetGroupIdX(M0) && isGetEnqueuedLocalSizeX(M1)) ||
-        (isGetGroupIdX(M1) && isGetEnqueuedLocalSizeX(M0))))
-        return false;
-
-    return ((isGetLocalIdX(A0) && isGetGlobalOffsetX(A1)) ||
-        (isGetLocalIdX(A1) && isGetGlobalOffsetX(A0)));
-}
-
-bool Simd32ProfitabilityAnalysis::isSelectBasedOnGlobalIdX(Value* V) {
-    PHINode* PN = dyn_cast<PHINode>(V);
-    while (!PN) {
-        auto BO = dyn_cast<BinaryOperator>(V);
-        if (!BO || BO->getOpcode() != Instruction::Shl)
-            return false;
-        if (!isa<Constant>(BO->getOperand(1)))
-            return false;
-        V = BO->getOperand(0);
-        PN = dyn_cast<PHINode>(V);
-    }
-
-    if (PN->getNumIncomingValues() != 2)
-        return false;
-
-    auto Op0 = PN->getIncomingValue(0);
-    if (!WI->isUniform(Op0))
-        return false;
-    auto Op1 = PN->getIncomingValue(1);
-    if (!WI->isUniform(Op1))
-        return false;
-
-    auto BB0 = PN->getIncomingBlock(0);
-    auto BB1 = PN->getIncomingBlock(1);
-    auto IfBB = BB0->getSinglePredecessor();
-    if (!IfBB || IfBB == BB1->getSinglePredecessor())
-        return false;
-    auto Br = dyn_cast<BranchInst>(IfBB->getTerminator());
-    if (!Br || !Br->isConditional())
-        return false;
-
-    ICmpInst* Cmp = dyn_cast<ICmpInst>(Br->getCondition());
-    if (!Cmp)
-        return false;
-    Value* LHS = Cmp->getOperand(0);
-    Value* RHS = Cmp->getOperand(1);
-    switch (Cmp->getPredicate()) {
-    default:
-        return false;
-    case CmpInst::ICMP_SLT:
-    case CmpInst::ICMP_SLE:
-        break;
-    case CmpInst::ICMP_SGT:
-    case CmpInst::ICMP_SGE:
-        std::swap(LHS, RHS);
-        break;
-    }
-    if (!WI->isUniform(RHS))
-        return false;
-    return !isGetGlobalIdX(LHS);
+void Simd32ProfitabilityAnalysis::print(llvm::raw_ostream& OS) const
+{
+    OS << "\nisSimd16Profitable: " << m_isSimd16Profitable;
+    OS << "\nisSimd32Profitable: " << m_isSimd32Profitable << "\n\n";
 }
 
 bool Simd32ProfitabilityAnalysis::checkSimd32Profitable(CodeGenContext* ctx)
@@ -851,8 +699,6 @@ bool Simd32ProfitabilityAnalysis::checkSimd32Profitable(CodeGenContext* ctx)
                                 if (Value * Count = getLoopCount(Init, Op1)) {
                                     if (WI->isUniform(Count))
                                         continue;
-                                    if (isSelectBasedOnGlobalIdX(Count))
-                                        continue;
                                 }
                             }
                         }
