@@ -274,6 +274,7 @@ void WIAnalysisRunner::dump() const
 
 void WIAnalysisRunner::init(
     llvm::Function* F,
+    llvm::LoopInfo* LI,
     llvm::DominatorTree* DT,
     llvm::PostDominatorTree* PDT,
     IGC::IGCMD::MetaDataUtils* MDUtils,
@@ -282,6 +283,7 @@ void WIAnalysisRunner::init(
     IGC::TranslationTable* TransTable)
 {
     m_func = F;
+    this->LI = LI;
     this->DT = DT;
     this->PDT = PDT;
     m_pMdUtils = MDUtils;
@@ -381,14 +383,12 @@ bool WIAnalysis::runOnFunction(Function& F)
     auto* MDUtils = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
     auto* DT = &getAnalysis<DominatorTreeWrapperPass>().getDomTree();
     auto* PDT = &getAnalysis<PostDominatorTreeWrapperPass>().getPostDomTree();
-    auto* CGCtx = getAnalysis<CodeGenContextWrapper>().getCodeGenContext();
+    auto *LI = &getAnalysis<LoopInfoWrapperPass>().getLoopInfo();
+    auto *CGCtx = getAnalysis<CodeGenContextWrapper>().getCodeGenContext();
     auto* ModMD = getAnalysis<MetaDataUtilsWrapper>().getModuleMetaData();
     auto* pTT = &getAnalysis<TranslationTable>();
 
-    isComputeProgram = CGCtx->type == ShaderType::COMPUTE_SHADER ||
-        CGCtx->type == ShaderType::OPENCL_SHADER;
-
-    Runner.init(&F, DT, PDT, MDUtils, CGCtx, ModMD, pTT);
+    Runner.init(&F, LI, DT, PDT, MDUtils, CGCtx, ModMD, pTT);
     return Runner.run();
 }
 
@@ -844,6 +844,12 @@ void WIAnalysisRunner::calculate_dep(const Value* val)
             dep = WIAnalysis::UNIFORM_THREAD;
         }
 
+        // Spec enforces subgroup broadcast to use thread-uniform local ID.
+        if (isWaveBroadcastIndex(inst))
+        {
+            dep = WIAnalysis::UNIFORM_THREAD;
+        }
+
         // If the value was changed in this calculation
         if (!hasOriginal || dep != orig)
         {
@@ -901,6 +907,18 @@ bool WIAnalysisRunner::isRegionInvariant(const llvm::Instruction* defi, BranchIn
     return true;
 }
 
+bool WIAnalysisRunner::isWaveBroadcastIndex(const llvm::Instruction* inst)
+{
+    for (auto it = inst->users().begin(); it != inst->users().end(); ++it)
+    {
+        const GenIntrinsicInst* GII = dyn_cast<GenIntrinsicInst>(*it);
+        if (GII && GII->getIntrinsicID() == GenISAIntrinsic::GenISA_WaveBroadcast && GII->getOperand(1) == inst)
+            return true;
+    }
+
+    return false;
+}
+
 void WIAnalysisRunner::update_cf_dep(const IGCLLVM::TerminatorInst* inst)
 {
     IGC_ASSERT(hasDependency(inst));
@@ -914,17 +932,52 @@ void WIAnalysisRunner::update_cf_dep(const IGCLLVM::TerminatorInst* inst)
     BranchInfo br_info(inst, ipd);
     // debug: dump influence region and partial-joins
     // br_info.print(ods());
-
+    auto* CbrLoop = LI->getLoopFor(blk);
+    // Loop* IPDLoop = nullptr;
     // check dep-type for every phi in the full join
     if (ipd)
     {
         updatePHIDepAtJoin(ipd, &br_info);
+        // IPDLoop = LI->getLoopFor(ipd);
     }
     // check dep-type for every phi in the partial-joins
     for (SmallPtrSet<BasicBlock*, 4>::iterator join_it = br_info.partial_joins.begin(),
         join_e = br_info.partial_joins.end(); join_it != join_e; ++join_it)
     {
-        updatePHIDepAtJoin(*join_it, &br_info);
+        auto *PJ = *join_it;
+        // skip the special loop-entry case
+        if (DT->dominates(PJ, blk)) {
+            int NumPreds = 0;
+            for (auto *pred : predecessors(PJ)) {
+                if (br_info.influence_region.count(pred)) {
+                    NumPreds++;
+                }
+            }
+            if (NumPreds <= 1)
+                continue;
+        }
+        auto PJDom = DT->getNode(PJ)->getIDom()->getBlock();
+        // If both partial-join and it IDom are in partial-join region
+        // there are cases in which phi-nodes in partial-joins are not
+        // relevant to the cbr under the investigation
+        auto LoopA = LI->getLoopFor(PJDom);
+        auto LoopB = LI->getLoopFor(PJ);
+        if (br_info.partial_joins.count(PJDom))
+        {
+            // both PJ and its IDom are outside the CBR loop
+            if (!CbrLoop || !CbrLoop->contains(LoopA))
+                continue;
+            // CbrLoop contains both PJDom and PJ
+            if (CbrLoop->contains(LoopB))
+            {
+                // Either Cbr-block strongly dominates the PJDom
+                // Or PJ dominates Cbr-block
+                if ((blk != PJDom && DT->dominates(blk, PJDom)) ||
+                    DT->dominates(PJ, blk))
+                    continue;
+            }
+        }
+        updatePHIDepAtJoin(PJ, &br_info);
     }
 
     // walk through all the instructions in the influence-region
@@ -973,6 +1026,7 @@ void WIAnalysisRunner::update_cf_dep(const IGCLLVM::TerminatorInst* inst)
             // 1) if use is in the full-join
             // 2) if use is even outside the full-join
             // 3) if use is in partial-join but def is not in partial-join
+            // 4) if def and use are in partial-join but def inside loop
             Value::use_iterator use_it = defi->use_begin();
             Value::use_iterator use_e = defi->use_end();
             for (; use_it != use_e; ++use_it)
@@ -992,10 +1046,14 @@ void WIAnalysisRunner::update_cf_dep(const IGCLLVM::TerminatorInst* inst)
                     // local def-use, not related to control-dependence
                     continue; // skip
                 }
+                auto DefLoop = LI->getLoopFor(def_blk);
+                auto UseLoop = LI->getLoopFor(user_blk);
                 if (user_blk == br_info.full_join ||
                     !br_info.influence_region.count(user_blk) ||
                     (br_info.partial_joins.count(user_blk) &&
-                     !br_info.partial_joins.count(def_blk))
+                      (!br_info.partial_joins.count(def_blk) ||
+                       (DefLoop && !DefLoop->contains(UseLoop)))
+                    )
                    )
                 {
                     updateDepMap(defi, instDep);
@@ -1090,6 +1148,10 @@ void WIAnalysisRunner::updateDepMap(const Instruction* inst, WIAnalysis::WIDepen
         {
             updateInsertElements((const InsertElementInst*)inst);
         }
+        else if (const InsertValueInst* IVI = dyn_cast<const InsertValueInst>(inst))
+        {
+            updateInsertValues(IVI);
+        }
     }
 }
 
@@ -1118,6 +1180,39 @@ void WIAnalysisRunner::updateInsertElements(const InsertElementInst* inst)
     }
 }
 
+/// If one of an insertvalue chain is random, turn all into random
+///   The change of insertvalue is like the following:
+///       st0 = insertvalue %INIT, %e0, 0
+///       st1 = insertvalue %st0,  %e1, 1
+///       st2 = insertvalue %st1,  %e2, 2
+///       st3 = insertvalue %st2,  %e3, 3
+///    here, {st0, st1, st2, st3} is one insertvalue chain, in which st0, st1
+///    and st2 are all single use, st3 is either not a single use or used
+///    in a non-insertvalue instruction.
+void WIAnalysisRunner::updateInsertValues(const InsertValueInst* Inst)
+{
+    /// find the first one in the sequence
+    const InsertValueInst* pI = Inst;
+    const InsertValueInst* aI = dyn_cast<const InsertValueInst>(pI->getOperand(0));
+    while (aI && aI->hasOneUse())
+    {
+        if (hasDependency(aI) && getDependency(aI) == WIAnalysis::RANDOM)
+            return;
+        pI = aI;
+        aI = dyn_cast<const InsertValueInst>(aI->getOperand(0));
+    }
+    if (pI != Inst)
+    {
+        m_depMap.SetAttribute(pI, WIAnalysis::RANDOM);
+        auto it = pI->user_begin();
+        auto e = pI->user_end();
+        for (; it != e; ++it)
+        {
+            m_pChangedNew->push_back(*it);
+        }
+    }
+}
+
 WIAnalysis::WIDependancy WIAnalysisRunner::calculate_dep_simple(const Instruction* I)
 {
     // simply check that all operands are uniform, if so return uniform, else random
@@ -1284,7 +1379,7 @@ WIAnalysis::WIDependancy WIAnalysisRunner::calculate_dep(const CallInst* inst)
         intrinsic_name == llvm_runtimeValue ||
         intrinsic_name == llvm_getMessagePhaseX ||
         intrinsic_name == llvm_getMessagePhaseXV ||
-        intrinsic_name == llvm_surfaceinfo ||
+        intrinsic_name == llvm_readsurfacetypeandformat ||
         intrinsic_name == llvm_simdSize ||
         intrinsic_name == llvm_resinfoptr ||
         intrinsic_name == llvm_sampleinfoptr ||
@@ -1292,6 +1387,7 @@ WIAnalysis::WIDependancy WIAnalysisRunner::calculate_dep(const CallInst* inst)
         intrinsic_name == llvm_ldraw_indexed ||
         intrinsic_name == llvm_cycleCounter ||
         intrinsic_name == llvm_waveShuffleIndex ||
+        intrinsic_name == llvm_waveBroadcast ||
         intrinsic_name == llvm_waveBallot ||
         intrinsic_name == llvm_waveAll ||
         intrinsic_name == llvm_waveClustered ||
@@ -1355,8 +1451,12 @@ WIAnalysis::WIDependancy WIAnalysisRunner::calculate_dep(const CallInst* inst)
         GII_id == GenISAIntrinsic::GenISA_getStageInGridOrigin ||
         GII_id == GenISAIntrinsic::GenISA_getStageInGridSize ||
         GII_id == GenISAIntrinsic::GenISA_getSyncBuffer ||
+        GII_id == GenISAIntrinsic::GenISA_getRtGlobalBufferPtr ||
         GII_id == GenISAIntrinsic::GenISA_GetImplicitBufferPtr ||
-        GII_id == GenISAIntrinsic::GenISA_GetLocalIdBufferPtr)
+        GII_id == GenISAIntrinsic::GenISA_GetLocalIdBufferPtr  ||
+        GII_id == GenISAIntrinsic::GenISA_getAssertBufferPtr  ||
+        GII_id == GenISAIntrinsic::GenISA_staticConstantPatchValue ||
+        GII_id == GenISAIntrinsic::GenISA_bitcastfromstruct)
     {
         switch (GII_id)
         {
@@ -1395,6 +1495,8 @@ WIAnalysis::WIDependancy WIAnalysisRunner::calculate_dep(const CallInst* inst)
         case GenISAIntrinsic::GenISA_getStageInGridOrigin:
         case GenISAIntrinsic::GenISA_getStageInGridSize:
         case GenISAIntrinsic::GenISA_getSyncBuffer:
+        case GenISAIntrinsic::GenISA_getRtGlobalBufferPtr:
+        case GenISAIntrinsic::GenISA_getAssertBufferPtr:
             return ImplicitArgs::getArgDep(GII_id);
         }
 
@@ -1411,22 +1513,6 @@ WIAnalysis::WIDependancy WIAnalysisRunner::calculate_dep(const CallInst* inst)
 
         if (GII_id == GenISAIntrinsic::GenISA_TileYOffset)
         {
-            IGC_ASSERT(m_CGCtx->type == ShaderType::RAYTRACING_SHADER);
-            auto* Ctx = static_cast<RayDispatchShaderContext*>(m_CGCtx);
-            if (auto Mode = Ctx->knownSIMDSize())
-            {
-                auto* TYI = cast<TileYIntrinsic>(inst);
-                uint32_t TileXDim = TYI->getTileXDim();
-                uint32_t SubtileXDim = TYI->getSubtileXDim();
-                const uint32_t Lanes = numLanes(*Mode);
-                // We currently tile along the x-dim first. If the SIMD size
-                // perfectly divides the x-dim, then the y-dim must be uniform.
-                return (TileXDim % Lanes == 0 &&
-                        (SubtileXDim == 0 || SubtileXDim % Lanes == 0)) ?
-                    WIAnalysis::UNIFORM_THREAD :
-                    WIAnalysis::RANDOM;
-            }
-            else
             {
                 return WIAnalysis::RANDOM;
             }
@@ -1534,7 +1620,8 @@ WIAnalysis::WIDependancy WIAnalysisRunner::calculate_dep(const CallInst* inst)
             return WIAnalysis::UNIFORM_THREAD;
         }
 
-        if (intrinsic_name == llvm_waveShuffleIndex)
+        if (intrinsic_name == llvm_waveShuffleIndex ||
+            intrinsic_name == llvm_waveBroadcast)
         {
             Value* op0 = inst->getArgOperand(0);
             Value* op1 = inst->getArgOperand(1);
@@ -1565,6 +1652,11 @@ WIAnalysis::WIDependancy WIAnalysisRunner::calculate_dep(const CallInst* inst)
             return select_conversion[dep0][dep1];
         }
 
+        if (GII_id == GenISAIntrinsic::GenISA_staticConstantPatchValue)
+        {
+            return WIAnalysis::UNIFORM_GLOBAL;
+        }
+
         if (intrinsic_name == llvm_waveBallot || intrinsic_name == llvm_waveAll)
         {
             return WIAnalysis::UNIFORM_THREAD;
@@ -1720,8 +1812,17 @@ WIAnalysis::WIDependancy WIAnalysisRunner::calculate_dep_terminator(
         return WIAnalysis::RANDOM;
         // TODO: Define the dependency requirements of indirectBr
     case Instruction::Switch:
+    {
+        // Same as conditional br
+        const SwitchInst* swInst = cast<SwitchInst>(inst);
+        Value* op = swInst->getCondition();
+        WIAnalysis::WIDependancy dep = getDependency(op);
+        if (WIAnalysis::isDepUniform(dep))
+        {
+            return dep;
+        }
         return WIAnalysis::RANDOM;
-        // TODO: Should this depend only on the condition, like branch?
+    }
     default:
         return WIAnalysis::RANDOM;
     }
