@@ -36,7 +36,7 @@ DECLARE_OPCODE(GenISA_discard, GenISAIntrinsic, llvm_discard, false, false, fals
 DECLARE_OPCODE(GenISA_InitDiscardMask, GenISAIntrinsic, llvm_initdiscardmask, false, false, false, false, false, false, false)
 DECLARE_OPCODE(GenISA_UpdateDiscardMask, GenISAIntrinsic, llvm_updatediscardmask, false, false, false, false, false, false, false)
 DECLARE_OPCODE(GenISA_GetPixelMask, GenISAIntrinsic, llvm_getpixelmask, false, false, false, false, false, false, false)
-DECLARE_OPCODE(GenISA_readsurfaceinfoptr, GenISAIntrinsic, llvm_surfaceinfo, false, false, false, false, false, false, false)
+DECLARE_OPCODE(GenISA_readsurfacetypeandformat, GenISAIntrinsic, llvm_readsurfacetypeandformat, false, false, false, false, false, false, false)
 DECLARE_OPCODE(GenISA_resinfoptr, GenISAIntrinsic, llvm_resinfoptr, false, false, false, false, false, false, false)
 DECLARE_OPCODE(GenISA_sampleinfoptr, GenISAIntrinsic, llvm_sampleinfoptr, false, false, false, false, false, false, false)
 DECLARE_OPCODE(GenISA_PullSampleIndexBarys, GenISAIntrinsic, llvm_pullSampleIndexBarys, false, false, false, false, false, false, false)
@@ -118,6 +118,13 @@ DECLARE_OPCODE(Alloca, Instruction, llvm_alloca, false, false, false, false, fal
 DECLARE_OPCODE(FPExt, Instruction, llvm_fpext, true, false, false, false, false, false, false)
 DECLARE_OPCODE(GenISA_f32tof16_rtz, GenISAIntrinsic, llvm_f32tof16_rtz, true, false, false, false, true, false, false)
 DECLARE_OPCODE(fabs, Intrinsic, llvm_fabs, false, false, true, false, true, false, false)
+#if LLVM_VERSION_MAJOR >= 12
+DECLARE_OPCODE(abs, Intrinsic, llvm_abs, false, false, true, false, true, false, false)
+DECLARE_OPCODE(smin, Intrinsic, llvm_smin, false, false, true, false, true, false, false)
+DECLARE_OPCODE(smax, Intrinsic, llvm_smax, false, false, true, false, true, false, false)
+DECLARE_OPCODE(umin, Intrinsic, llvm_umin, false, false, true, false, true, false, false)
+DECLARE_OPCODE(umax, Intrinsic, llvm_umax, false, false, true, false, true, false, false)
+#endif
 DECLARE_OPCODE(GenISA_fsat, GenISAIntrinsic, llvm_fsat, true, false, true, true, true, false, false)
 DECLARE_OPCODE(canonicalize, Intrinsic, llvm_canonicalize, true, true, true, true, false, false, false)
 DECLARE_OPCODE(GenISA_dp4a_ss, GenISAIntrinsic, llvm_dp4a_ss, false, true, true, true, false, false, false)
@@ -244,6 +251,7 @@ DECLARE_OPCODE(GenISA_WaveClustered, GenISAIntrinsic, llvm_waveClustered, false,
 DECLARE_OPCODE(GenISA_WavePrefix, GenISAIntrinsic, llvm_wavePrefix, false, false, false, false, false, false, false)
 DECLARE_OPCODE(GenISA_QuadPrefix, GenISAIntrinsic, llvm_quadPrefix, false, false, false, false, false, false, false)
 DECLARE_OPCODE(GenISA_WaveShuffleIndex, GenISAIntrinsic, llvm_waveShuffleIndex, false, false, false, false, false, false, false)
+DECLARE_OPCODE(GenISA_WaveBroadcast, GenISAIntrinsic, llvm_waveBroadcast, false, false, false, false, false, false, false)
 
 // Unmasked region
 DECLARE_OPCODE(GenISA_UnmaskedRegionBegin, GenISAIntrinsic, llvm_unmaskedBegin, false, false, false, false, false, false, false)
