@@ -48,6 +48,7 @@ namespace IGC
         firstStateId = IGC_GET_FLAG_VALUE(RetryManagerFirstStateId);
         stateId = firstStateId;
         prevStateId = 500;
+        shaderType = ShaderType::UNKNOWN;
         IGC_ASSERT(stateId < RetryTableSize);
     }
 
@@ -83,7 +84,8 @@ namespace IGC
         // Since we currently can't enable/disable LICM per-function, enabling LICM
         // when retrying for stackcalls seems to give better performance. So always
         // enable when recompiling with stackcalls.
-        return RetryTable[id].allowLICM || !PerFuncRetrySet.empty();
+        return RetryTable[id].allowLICM ||
+            (shaderType == ShaderType::OPENCL_SHADER && !PerFuncRetrySet.empty());
     }
 
     bool RetryManager::AllowAddressArithmeticSinking(Function* F) const
@@ -172,9 +174,10 @@ namespace IGC
         return stateId;
     }
 
-    void RetryManager::Enable()
+    void RetryManager::Enable(ShaderType ty)
     {
         enabled = true;
+        shaderType = ty;
     }
 
     // Disable retry
