@@ -54,6 +54,7 @@ SPDX-License-Identifier: MIT
 #include "Probe/Assertion.h"
 #include <optional>
 #include <Metrics/IGCMetric.h>
+#include "llvmWrapper/IR/Module.h"
 
 /************************************************************************
 This file contains the interface structure and functions to communicate
@@ -746,7 +747,7 @@ namespace IGC
         unsigned GetRetryId() const;
         unsigned GetPerFuncRetryStateId(llvm::Function* F) const;
 
-        void Enable();
+        void Enable(ShaderType ty = ShaderType::UNKNOWN);
         void Disable(bool DisablePerKernel=false);
 
         void SetSpillSize(unsigned int spillSize);
@@ -795,6 +796,9 @@ namespace IGC
         // internal knob to disable retry manager.
         bool enabled;
 
+        // shader type for shader specific opt
+        ShaderType shaderType;
+
         unsigned lastSpillSize = 0;
 
         // cache the compiled kernel during retry
@@ -1103,13 +1107,13 @@ namespace IGC
         void SetSIMDInfo(SIMDInfoBit bit, SIMDMode simd, ShaderDispatchMode mode)
         {
             unsigned int offset = GetSIMDInfoOffset(simd, mode);
-            m_SIMDInfo |= (uint64_t)1 << (bit + offset);
+            m_SIMDInfo |= 1ULL << (bit + offset);
         }
 
         void ClearSIMDInfo(SIMDMode simd, ShaderDispatchMode mode)
         {
             unsigned int offset = GetSIMDInfoOffset(simd, mode);
-            m_SIMDInfo &= ~(0xff << offset);
+            m_SIMDInfo &= ~(0xffULL << offset);
         }
 
         uint64_t GetSIMDInfo() { return m_SIMDInfo; }
