@@ -46,19 +46,116 @@ IGC_INITIALIZE_PASS_DEPENDENCY(MetaDataUtilsWrapper)
 IGC_INITIALIZE_PASS_DEPENDENCY(CodeGenContextWrapper)
 IGC_INITIALIZE_PASS_END(JointMatrixFuncsResolutionPass, PASS_FLAG, PASS_DESC, PASS_CFG_ONLY, PASS_ANALYSIS)
 
-JointMatrixFuncsResolutionPass::JointMatrixFuncsResolutionPass() : FunctionPass(ID)
+// We need module pass, since:
+// 1) we inspect multiple functions to find entry function to get sub group size
+// 2) we maintain map of functions to entry functions across functions we process
+// so the pass is not local to one function.
+JointMatrixFuncsResolutionPass::JointMatrixFuncsResolutionPass() : ModulePass(ID)
 {
     initializeJointMatrixFuncsResolutionPassPass(*PassRegistry::getPassRegistry());
 }
 
-bool JointMatrixFuncsResolutionPass::runOnFunction(Function& F)
+bool JointMatrixFuncsResolutionPass::runOnModule(Module &M)
 {
     m_Ctx = getAnalysis<CodeGenContextWrapper>().getCodeGenContext();
+    m_mdUtils = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
+    FunctionsMap.clear();
+    Changed = false;
+
+    for (auto &F : M)
+    {
+        if (F.isDeclaration())
+            continue;
+
+        if (runOnFunction(F))
+            Changed = true;
+    }
+
+    return Changed;
+}
+
+// Finds entry function for input function. If there are several entry
+// functions, it will return only one (first found).
+// So currently the case, when the same function which is using Joint Matrix
+// is called from several kernels with different sub-group size required,
+// not supported and behavior is undefined.
+Function *JointMatrixFuncsResolutionPass::getEntryFunction(Function *F)
+{
+    if (FunctionsMap.count(F) > 0)
+        return FunctionsMap[F];
+
+    if (isEntryFunc(m_mdUtils, F))
+    {
+        FunctionsMap[F] = F;
+        return F;
+    }
+
+    SmallVector<Function *, 8> toProcess;
+    toProcess.push_back(F);
+    SmallPtrSet<Function *, 8> toSetEntry;
+    toSetEntry.insert(F);
+
+    while (!toProcess.empty())
+    {
+        Function *curFunc = toProcess.pop_back_val();
+        for (auto It = curFunc->use_begin(); It != curFunc->use_end(); It++)
+        {
+            auto user = It->getUser();
+            if (!isa<CallInst>(user))
+                continue;
+
+            auto *CI = cast<CallInst>(user);
+            if (CI->getCalledFunction() == curFunc)
+            {
+                auto caller = CI->getFunction();
+                Function *entryFunc = nullptr;
+                if (FunctionsMap.count(caller) > 0)
+                    entryFunc = FunctionsMap[caller];
+                if (!entryFunc && isEntryFunc(m_mdUtils, caller))
+                    entryFunc = caller;
+                if (entryFunc)
+                {
+                    for (auto *fToSetEntry : toSetEntry)
+                        FunctionsMap[fToSetEntry] = entryFunc;
+                    return entryFunc;
+                }
+                if (toSetEntry.count(caller) == 0) {
+                    toProcess.push_back(caller);
+                    toSetEntry.insert(caller);
+                }
+            }
+        }
+    }
+    FunctionsMap[F] = nullptr;
+    return nullptr;
+}
+
+void JointMatrixFuncsResolutionPass::ResolveSIMDSize(Function *F) {
+    if (m_SIMDSize != 0) return;
+
+    Function *entryFunction = getEntryFunction(F);
+    if (entryFunction) {
+        IGCMD::FunctionInfoMetaDataHandle funcInfoMD = m_mdUtils->getFunctionsInfoItem(entryFunction);
+        IGCMD::SubGroupSizeMetaDataHandle subGroupSize = funcInfoMD->getSubGroupSize();
+        if (subGroupSize->hasValue())
+            m_SIMDSize = subGroupSize->getSIMD_size();
+    }
+
+    // If no entry function found (it means that we could not detect that current function is called
+    // from any kernel) or kernel function doesn't have sub group size requirement set, we anyway
+    // will resolve function, just in case, using default sub group size.
+    // In theory it may cause resolution conflicts if sub group sizes are mixed.
+    if (m_SIMDSize == 0)
+        m_SIMDSize = m_Ctx->platform.hasExecSize16DPAS() ? 16 : 8;
+}
+
+bool JointMatrixFuncsResolutionPass::runOnFunction(Function& F)
+{
     PlaceholderInstructions.clear();
     ResolvedValues.clear();
     ResolvedTypes.clear();
     InstsToErase.clear();
-    Changed = false;
+    m_SIMDSize = 0;
 
     // Use reverse post order traversal to reduce level or recursion
     ReversePostOrderTraversal<Function *> RPOT(&F);
@@ -78,7 +175,7 @@ bool JointMatrixFuncsResolutionPass::runOnFunction(Function& F)
         I->eraseFromParent();
     }
 
-    return Changed;
+    return !ResolvedValues.empty();
 }
 
 static const char *CommonBIPrefix = "__builtin_spirv_";
@@ -375,22 +472,14 @@ std::string JointMatrixFuncsResolutionPass::GetMatrixFuncName(
     name += std::to_string(desc->rows);
     name += "x";
     name += std::to_string(desc->columns);
-    name += "_";
-
-    if (desc->bitWidth == 8) {
-        name += "i8";
-    } else if (desc->bitWidth == 16) {
-        name += "i16";
-    } else if (desc->bitWidth == 32) {
-        name += "i32";
-    } else {
-        IGC_ASSERT_MESSAGE(false, "Unexpected matrix element size.");
-    }
+    name += "_i" + std::to_string(desc->bitWidth);
 
     // We are done creating the mangling for get_coord here()
     if (isGetCoord)
         return name;
 
+    name += "_" + std::to_string(getNumRowsPerWI(desc));
+
     // Continue mangling for load and store
     if (address_space == ADDRESS_SPACE_GLOBAL) {
         name += "_global_";
@@ -424,7 +513,7 @@ static unsigned parseNumber(StringRef name, unsigned *offset) {
 
 /* This function extracts metadata from JointMatrix type names. They use the
  * following convention: intel.joint_matrix_acc_8x8_i32_t */
-static bool parseMatrixTypeNameLegacy(const Type *opaqueType, JointMatrixTypeDescription *outDescription) {
+bool JointMatrixFuncsResolutionPass::parseMatrixTypeNameLegacy(const Type *opaqueType, JointMatrixTypeDescription *outDescription) {
     const PointerType *ptrType = cast<PointerType>(opaqueType);
     StringRef name = IGCLLVM::getNonOpaquePtrEltTy(ptrType)->getStructName();
 
@@ -438,6 +527,10 @@ static bool parseMatrixTypeNameLegacy(const Type *opaqueType, JointMatrixTypeDes
     } else if (name.startswith("intel.joint_matrix_acc_")) {
         outDescription->layout = LayoutRowMajor;
         offset += sizeof "intel.joint_matrix_acc_";
+    } else {
+        std::string msg = "Unexpected Joint Matrix type name: '" + name.str() + "', unknown layout.";
+        m_Ctx->EmitError(msg.c_str(), nullptr);
+        return false;
     }
     offset -= 1; /* Go back to the end of prefix. */
     outDescription->rows = parseNumber(name, &offset);
@@ -450,6 +543,11 @@ static bool parseMatrixTypeNameLegacy(const Type *opaqueType, JointMatrixTypeDes
 
     offset += 1; /* Skip type specifier, [f|i] */
     outDescription->bitWidth = parseNumber(name, &offset);
+    IGC_ASSERT_MESSAGE(outDescription->bitWidth == 8 ||
+                           outDescription->bitWidth == 16 ||
+                           outDescription->bitWidth == 32,
+                       "Unexpected matrix element size.");
+
     return true;
 }
 
@@ -537,6 +635,7 @@ bool JointMatrixFuncsResolutionPass::ParseMatrixTypeName(Type *opaqueType, Joint
     } else {
         std::string msg = "Unexpected Joint Matrix type name: '"
                         + fullName.str() + "', unknown use type.";
+        m_Ctx->EmitError(msg.c_str(), nullptr);
         return false;
     }
     return true;
@@ -552,7 +651,7 @@ static bool isMatrixType(const Type *type)
         return false;
 
     StringRef name = eltType->getStructName();
-    if (name.startswith("intel.joint_matrix"))
+    if (name.startswith("intel.joint_matrix") || name.startswith("spirv.JointMatrixINTEL._"))
         return true;
 
     return false;
@@ -600,17 +699,26 @@ static bool isOrContainsMatrixType(const Type *root)
     return false;
 }
 
-// Custom slicing for tf32 for Matrix A.  In each WI, we have half the number
-// of rows that are present in the SG.
-static unsigned getNumRowsPerWIPackedA(const JointMatrixTypeDescription *desc) {
-    IGC_ASSERT_EXIT_MESSAGE(
-        desc->layout == LayoutPackedA,
-        "Expected getNumRowsPerWIPackedA function call for Matrix type A");
-    bool isTF32 = (desc->isFloating) && (desc->bitWidth == 32);
-    if (isTF32) {
-        return desc->rows % 2 ? desc->rows / 2 + 1 : desc->rows / 2;
+unsigned JointMatrixFuncsResolutionPass::getNumRowsPerWI(const JointMatrixTypeDescription *desc) {
+    IGC_ASSERT_MESSAGE(m_SIMDSize > 0, "Unexpected sub group size.");
+
+    if (desc->layout == LayoutPackedA) {
+        // Custom slicing for tf32 for Matrix A.  In each WI, we have half the number
+        // of rows that are present in the SG.
+        bool isTF32 = (desc->isFloating) && (desc->bitWidth == 32);
+
+        if (isTF32 || (m_Ctx->platform.hasExecSize16DPAS() && m_SIMDSize == 32))
+            return desc->rows % 2 ? desc->rows / 2 + 1 : desc->rows / 2;
+        return desc->rows;
+    } else if (desc->layout == LayoutPackedB) {
+        return (m_Ctx->platform.hasExecSize16DPAS() && m_SIMDSize == 32) ? 4 : 8;
+    } else if (desc->layout == LayoutRowMajor) {
+        if (m_Ctx->platform.hasExecSize16DPAS() && m_SIMDSize == 32)
+            return desc->rows % 2 ? desc->rows / 2 + 1 : desc->rows / 2;
+        return desc->rows;
     }
-    return desc->rows;
+    IGC_ASSERT_MESSAGE(false, "Unexpected matrix layout.");
+    return 1;
 }
 
 Type *JointMatrixFuncsResolutionPass::ResolveType(Type *opaqueType, JointMatrixTypeDescription *outDesc)
@@ -664,32 +772,22 @@ Type *JointMatrixFuncsResolutionPass::ResolveType(Type *opaqueType, JointMatrixT
         return IGCLLVM::FixedVectorType::get(baseType, 16);
     }
 
+    /* Small slice support: */
+    Type *baseType = Type::getInt32Ty(ctx);
     if (desc.layout == LayoutPackedA) {
-        Type *baseType = Type::getInt32Ty(ctx);
         bool isTF32 = (desc.isFloating) && (desc.bitWidth == 32);
         if (!isTF32 && m_Ctx->platform.hasExecSize16DPAS()) {
             baseType = Type::getInt16Ty(ctx);
         }
-        if (desc.rows == 1) {
-            resolvedType = baseType;
-        } else {
-            unsigned numRowsPerWI = getNumRowsPerWIPackedA(&desc);
-            resolvedType =
-                IGCLLVM::FixedVectorType::get(baseType, numRowsPerWI);
-        }
-    } else if (desc.layout == LayoutPackedB) {
-        Type *baseType = Type::getInt32Ty(ctx);
-        resolvedType = IGCLLVM::FixedVectorType::get(baseType, 8);
-    } else if (desc.layout == LayoutRowMajor) {
-        Type *baseType = Type::getInt32Ty(ctx);
-        if (desc.isFloating) {
-            baseType = Type::getFloatTy(ctx);
-        }
-        if (desc.rows == 1) {
-            resolvedType = baseType;
-        } else {
-            resolvedType = IGCLLVM::FixedVectorType::get(baseType, desc.rows);
-        }
+    } else if (desc.layout == LayoutRowMajor && desc.isFloating) {
+        baseType = Type::getFloatTy(ctx);
+    }
+
+    unsigned vectorSize = getNumRowsPerWI(&desc);
+    if (vectorSize == 1) {
+        resolvedType = baseType;
+    } else {
+        resolvedType = IGCLLVM::FixedVectorType::get(baseType, vectorSize);
     }
 
     IGC_ASSERT_EXIT_MESSAGE(resolvedType != nullptr, "Failed to resolve matrix type.");
@@ -1006,22 +1104,21 @@ static unsigned getResolvedVectorElemSize(Type *matrixType) {
     return matrixType->getScalarSizeInBits();
 }
 
-static int getSliceSize(const JointMatrixTypeDescription *desc, Type *matTy) {
-    unsigned contribTypeWidth = getResolvedVectorElemSize(matTy);
+int JointMatrixFuncsResolutionPass::getSliceSize(const JointMatrixTypeDescription *desc, Type *matTy) {
+    if (desc->layout == LayoutRowMajor)
+        return getNumRowsPerWI(desc);
 
-    if (desc->layout == LayoutRowMajor) {
-        return desc->rows;
-    }
-    if (desc->bitWidth != 0) {
-        if (desc->layout == LayoutPackedA) {
-            unsigned numRowsPerWI = getNumRowsPerWIPackedA(desc);
-            return numRowsPerWI * (contribTypeWidth / desc->bitWidth);
-        }
-        if (desc->layout == LayoutPackedB) {
-            return 8 * (contribTypeWidth / desc->bitWidth);
+    if (desc->layout == LayoutPackedA || desc->layout == LayoutPackedB) {
+        if (desc->bitWidth == 0) {
+            IGC_ASSERT_MESSAGE(false, "Unexpected matrix element bit width.");
+            return 1;
         }
+
+        unsigned contribTypeWidth = getResolvedVectorElemSize(matTy);
+        return getNumRowsPerWI(desc) * (contribTypeWidth / desc->bitWidth);
     }
-    IGC_ASSERT_MESSAGE(true, "Unexpected matrix layout.");
+
+    IGC_ASSERT_MESSAGE(false, "Unexpected matrix layout.");
     return 1;
 }
 
@@ -1094,6 +1191,15 @@ Value *JointMatrixFuncsResolutionPass::ResolveFill(CallInst *CI) {
         fillValue = builder.CreateLoad(vectorElementType, fillValue);
     }
 
+    // For TF32 type, the slice has a type of i32, however, the value we are
+    // filling with has a type of float.  So we need a bitcast.
+    bool isTF32 = (desc.isFloating) && (desc.bitWidth == 32);
+    if (isTF32) {
+        fillValue = builder.CreateBitCast(
+            fillValue, Type::getIntNTy(builder.getContext(),
+                                       getResolvedVectorElemSize(matTy)));
+    }
+
     Value *slice = fillValue;
 
     if (IGCLLVM::FixedVectorType *ty =
@@ -1154,9 +1260,8 @@ Instruction *JointMatrixFuncsResolutionPass::ResolveGetCoord(CallInst *CI) {
     return newCall;
 }
 
-template <class BuilderT>
-static Value *createSliceExtract
-      (BuilderT *builder, Value *matrix, Value *index, const JointMatrixTypeDescription *desc, Type *matTy) {
+Value *JointMatrixFuncsResolutionPass::createSliceExtract
+      (IRBuilder<> *builder, Value *matrix, Value *index, const JointMatrixTypeDescription *desc, Type *matTy) {
     const int sliceSize = getSliceSize(desc, matTy);
     const int vectorSize = getResolvedVectorSize(matrix->getType());
     /* Unpacking: */
@@ -1210,7 +1315,7 @@ Value *JointMatrixFuncsResolutionPass::ResolveSliceInsert(CallInst *CI) {
         offset = builder.CreateTruncOrBitCast(offset, Type::getIntNTy(builder.getContext(), vecElemSize));
 
         /* clear element bits: */
-        uint64_t maskValue = (1 << desc.bitWidth) - 1;
+        uint64_t maskValue = (1ULL << desc.bitWidth) - 1;
         Value *mask = builder.CreateShl(ConstantInt::get(element->getType(), maskValue), offset);
         mask = builder.CreateNot(mask);
         element = builder.CreateAnd(element, mask);
@@ -1258,7 +1363,7 @@ Value *JointMatrixFuncsResolutionPass::ResolveSliceExtract(CallInst *CI) {
         Value *offset = builder.CreateURem(index, ConstantInt::get(index->getType(), packFactor));
         offset = builder.CreateMul(offset, ConstantInt::get(offset->getType(), desc.bitWidth));
         element = builder.CreateAShr(element, offset);
-        uint64_t mask = (1 << desc.bitWidth) - 1;
+        uint64_t mask = (1ULL << desc.bitWidth) - 1;
         element = builder.CreateAnd(element, mask);
 
         element = builder.CreateTruncOrBitCast(element, Type::getIntNTy(builder.getContext(), desc.bitWidth));
@@ -1441,7 +1546,7 @@ Type *JointMatrixFuncsResolutionPass::ResolveStructType(Type *oldType)
     if (ResolvedTypes.count(oldType) > 0)
         return ResolvedTypes[oldType];
 
-    StructType *structType = dyn_cast<StructType>(oldType);
+    StructType *structType = cast<StructType>(oldType);
     SmallString<28> name;
     StructType *newType = StructType::create(oldType->getContext(),
                                              (structType->getName() +
@@ -1543,10 +1648,12 @@ void JointMatrixFuncsResolutionPass::visitCallInst(CallInst& CI)
      * supported also basic block terminators should be used as
      * transformation starting point */
     if (funcName.startswith(CommonBIPrefix)) {
+        ResolveSIMDSize(CI.getParent()->getParent());
         ResolveCall(&CI);
         return;
     }
     if (funcName.startswith("_Z") && funcName.contains("__spirv_JointMatrix")) {
+        ResolveSIMDSize(CI.getParent()->getParent());
         ResolveCall(&CI);
         return;
     }
@@ -1559,7 +1666,7 @@ void JointMatrixFuncsResolutionPass::visitAllocaInst(AllocaInst &I)
 
     if (!isOrContainsMatrixType(I.getAllocatedType()))
         return;
-
+    ResolveSIMDSize(I.getParent()->getParent());
     ResolveGeneric(&I);
 }
 
@@ -1570,7 +1677,7 @@ void JointMatrixFuncsResolutionPass::visitGetElementPtrInst(GetElementPtrInst &I
 
     if (!isOrContainsMatrixType(I.getSourceElementType()))
         return;
-
+    ResolveSIMDSize(I.getParent()->getParent());
     ResolveGeneric(&I);
 }
 
@@ -1582,6 +1689,7 @@ void JointMatrixFuncsResolutionPass::visitStoreInst(StoreInst &I)
     Value *val = I.getValueOperand();
     if (isOrContainsMatrixType(val->getType()))
     {
+        ResolveSIMDSize(I.getParent()->getParent());
         ResolveGeneric(&I);
         return;
     }
@@ -1603,6 +1711,7 @@ void JointMatrixFuncsResolutionPass::visitStoreInst(StoreInst &I)
         !PTI->getDestTy()->isIntegerTy(64) || BC == nullptr)
         return;
 
+    ResolveSIMDSize(I.getParent()->getParent());
     InsertPlaceholder(&I);
     Value *PTIOperand = PTI->getOperand(0);
     Type *newBCElementType = ResolveTypes(
@@ -1660,6 +1769,7 @@ void JointMatrixFuncsResolutionPass::visitBitCastInst(BitCastInst &I)
     if (!isOrContainsMatrixType(I.getSrcTy()) && !isOrContainsMatrixType(I.getDestTy()))
         return;
 
+    ResolveSIMDSize(I.getParent()->getParent());
     ResolveGeneric(&I);
 }
 
@@ -1685,5 +1795,6 @@ void JointMatrixFuncsResolutionPass::visitPtrToIntInst(PtrToIntInst &I)
     if (!isOrContainsMatrixType(I.getSrcTy()))
         return;
 
+    ResolveSIMDSize(I.getParent()->getParent());
     ResolveGeneric(&I);
 }
