@@ -73,6 +73,8 @@ namespace {
         /// LSC Fence intrinsics call method
         Instruction* CreateLSCFenceIntrinsicCallInst();
 
+        Instruction* CreateLSCFenceEvictToMemory();
+
         /// LSC Atomic intrinsics call method
         Instruction* CreateLSCAtomicIntrinsicCallInst(bool isLocalMem);
 
@@ -169,6 +171,7 @@ namespace {
         static const StringRef PREFIX_LSC_LOAD_status;
 
         static const StringRef PREFIX_LSC_FENCE;
+        static const StringRef PREFIX_LSC_FENCE_EVICT_TO_MEMORY;
         static const StringRef PREFIX_LSC_ATOMIC;
         static const StringRef PREFIX_LSC_PREFETCH;
     };
@@ -186,6 +189,7 @@ const StringRef LSCFuncsResolution::PREFIX_LSC_LOAD_BLOCK_global = "__builtin_IB
 const StringRef LSCFuncsResolution::PREFIX_LSC_LOAD_status = "__builtin_IB_lsc_load_status_global_";
 
 const StringRef LSCFuncsResolution::PREFIX_LSC_FENCE  = "__builtin_IB_lsc_fence_";
+const StringRef LSCFuncsResolution::PREFIX_LSC_FENCE_EVICT_TO_MEMORY = "__builtin_IB_lsc_fence_evict_to_memory";
 const StringRef LSCFuncsResolution::PREFIX_LSC_ATOMIC = "__builtin_IB_lsc_atomic_";
 const StringRef LSCFuncsResolution::PREFIX_LSC_PREFETCH = "__builtin_IB_lsc_prefetch_global_";
 
@@ -287,7 +291,11 @@ void LSCFuncsResolution::visitCallInst(CallInst &CI)
         lscCall = CreateLSCAtomicIntrinsicCallInst(isLocalMem);
     //////////////
     // misc stuff
-    } else if (FN.startswith(LSCFuncsResolution::PREFIX_LSC_FENCE)) {
+    } else if (FN.startswith(LSCFuncsResolution::PREFIX_LSC_FENCE_EVICT_TO_MEMORY)) {
+        // LSC fence
+        lscCall = CreateLSCFenceEvictToMemory();
+    }
+    else if (FN.startswith(LSCFuncsResolution::PREFIX_LSC_FENCE)) {
         // LSC fence
         lscCall = CreateLSCFenceIntrinsicCallInst();
     } else {
@@ -440,6 +448,44 @@ Instruction* LSCFuncsResolution::CreateLSCFenceIntrinsicCallInst() {
     return lscCall;
 }
 
+// Resolve __builtin_IB_lsc_fence_evict_to_memory() builtin.
+// For XE platforms it is represented by the sequence of
+// evict followed by lush_l3 calls.
+Instruction* LSCFuncsResolution::CreateLSCFenceEvictToMemory()
+{
+    auto context = getAnalysis<CodeGenContextWrapper>().getCodeGenContext();
+
+    if (hasError())
+    {
+        return nullptr;
+    }
+
+    Value* args[3]
+    {
+        getConstantInt32(LSC_UGM), // immediate sfid
+        getConstantInt32(LSC_SCOPE_GPU),  // immediate scope of the fence
+        (context->platform.getPlatformInfo().eRenderCoreFamily == IGFX_XE_HPC_CORE) ?
+            getConstantInt32(LSC_FENCE_OP_NONE) :
+            getConstantInt32(LSC_FENCE_OP_EVICT)   // immediate flush type
+    };
+
+    Function* lscFunc = GenISAIntrinsic::getDeclaration(
+        m_pCurrInstFunc->getParent(), GenISAIntrinsic::GenISA_LSCFence, None);
+    Instruction* lscCall = CallInst::Create(lscFunc, args, "", m_pCurrInst);
+
+    if (context->platform.getPlatformInfo().eRenderCoreFamily == IGFX_XE_HPG_CORE)
+    {
+        args[2] = getConstantInt32(LSC_FENCE_OP_FLUSHL3);
+
+        lscCall = CallInst::Create(lscFunc, args, "", m_pCurrInst);
+    }
+
+    // It does not really matter which lscCall is returned, as
+    // that pointer is used to call ReplaceAllUsesWith,
+    // but these two instructions do not have any users.
+    return lscCall;
+}
+
 Instruction* LSCFuncsResolution::CreateLSCAtomicIntrinsicCallInst(
     bool isLocalMem)
 {
@@ -497,7 +543,7 @@ Instruction* LSCFuncsResolution::CreateLSCAtomicIntrinsicCallInst(
         atomArg2,                      // value [cmpxchg] or zero if unused
         getConstantInt32(atomicOp),    // atomic op
         isLocalMem ?                   // cache options (default for local)
-            getConstantInt32(LSC_L1DEF_L3DEF) : getCacheControlOpts(ccOpndIx)
+            getConstantInt32(LSC_L1DEF_L3DEF) : getCacheControlOpts(ccOpndIx, true)
     };
 
     GenISAIntrinsic::ID id =
@@ -807,10 +853,26 @@ Constant *LSCFuncsResolution::getImmediateElementOffset(
     }
 }
 
-Constant *LSCFuncsResolution::getCacheControlOpts(int i, bool)
+Constant *LSCFuncsResolution::getCacheControlOpts(int i, bool isAtomic)
 {
-    // TODO: error if atomic uses any sort of caching on L1
-    return getImmediateEnum(i, LSC_L1DEF_L3DEF, LSC_L1IAR_WB_L3C_WB);
+    Constant *c = getImmediateEnum(i, LSC_L1DEF_L3DEF, LSC_L1IAR_WB_L3C_WB);
+
+    if (isAtomic)
+    {
+        ConstantInt* ci = cast<ConstantInt>(c);
+        switch (ci->getZExtValue())
+        {
+        case LSC_L1DEF_L3DEF:
+        case LSC_L1UC_L3UC:
+        case LSC_L1UC_L3C_WB:
+            break;
+        default:
+            reportError("atomic must not use caching on L1");
+            c = getConstantInt32(LSC_L1DEF_L3DEF);
+        }
+    }
+
+    return c;
 }
 
 void LSCFuncsResolution::reportError(const char *what) {
