@@ -59,17 +59,17 @@ const int NamedBarriersResolution::GetMaxNamedBarriers()
 NamedBarriersResolution::NamedBarriersResolution() : ModulePass(ID)
 {
     m_CountNamedBarriers = 0;
-    m_GFX_GEN = IGFX_BROADWELL;
+    m_GFX_CORE = IGFX_GEN8_CORE;
     m_NamedBarrierType = nullptr;
     m_NamedBarrierID = nullptr;
     m_NamedBarrierArray = nullptr;
     initializeNamedBarriersResolutionPass(*PassRegistry::getPassRegistry());
 }
 
-NamedBarriersResolution::NamedBarriersResolution(PRODUCT_FAMILY GFX_GEN) : ModulePass(ID)
+NamedBarriersResolution::NamedBarriersResolution(GFXCORE_FAMILY GFX_CORE) : ModulePass(ID)
 {
     m_CountNamedBarriers = 0;
-    m_GFX_GEN = GFX_GEN;
+    m_GFX_CORE = GFX_CORE;
     m_NamedBarrierType = nullptr;
     m_NamedBarrierID = nullptr;
     m_NamedBarrierArray = nullptr;
@@ -242,7 +242,7 @@ void NamedBarriersResolution::HandleNamedBarrierSyncHW(CallInst& NBarrierSyncCal
     Value* trueValue = IRB.getInt1(true);
     Value* falseValue = IRB.getInt1(false);
 
-    ConstantInt* memFenceType = dyn_cast<ConstantInt>(NBarrierSyncCall.getArgOperand(1));
+    ConstantInt* memFenceType = cast<ConstantInt>(NBarrierSyncCall.getArgOperand(1));
     // LOCAL = 1
     // GLOBAL = 2
     Value* isGlobal = ( (int)memFenceType->getValue().getSExtValue() & 2 ) == 2 ? trueValue : falseValue;
@@ -257,6 +257,7 @@ void NamedBarriersResolution::HandleNamedBarrierSyncHW(CallInst& NBarrierSyncCal
             falseValue, // bool flushIcache
             isGlobal,   // bool isGlobal
             falseValue, // bool invalidateL1
+            falseValue, // bool evictL1
         },
         "",
         &(NBarrierSyncCall));
@@ -288,7 +289,7 @@ int NamedBarriersResolution::AlignNBCnt2BarrierNumber(uint NBCnt)
         6   B24
         7   B32
     */
-    if      (NBCnt > 32) { IGC_ASSERT_EXIT_MESSAGE(0, "NamedBarriersResolution : Incorrect named barrier count"); }
+    if      (NBCnt > 32) { IGC_ASSERT_UNREACHABLE(); } // NamedBarriersResolution : Incorrect named barrier count
     else if (NBCnt > 24) { return 32; }
     else if (NBCnt > 16) { return 24; }
     else if (NBCnt > 8)  { return 16; }
@@ -321,7 +322,7 @@ void NamedBarriersResolution::HandleNamedBarrierInitSW(CallInst& NBarrierInitCal
     // Check if we already have setup global variabels
     if (!IsNamedBarriersAdded())
     {
-        initGlobalVariables(module, NBarrierInitCall.getType()->getPointerElementType());
+        initGlobalVariables(module, IGCLLVM::getNonOpaquePtrEltTy(NBarrierInitCall.getType()));
     }
 
     auto newName = "__builtin_spirv_OpNamedBarrierInitialize_i32_p3__namedBarrier_p3i32";
@@ -331,7 +332,7 @@ void NamedBarriersResolution::HandleNamedBarrierInitSW(CallInst& NBarrierInitCal
         m_NamedBarrierType->getPointerTo(SPIRAS_Local),
         Type::getInt32PtrTy(context, SPIRAS_Local)
     };
-    Type *BaseTy = cast<PointerType>(m_NamedBarrierArray->getType())->getPointerElementType();
+    Type *BaseTy = IGCLLVM::getNonOpaquePtrEltTy(m_NamedBarrierArray->getType());
     auto pointerNBarrier = GetElementPtrInst::Create(BaseTy, m_NamedBarrierArray, { getInt64(module, 0), getInt32(module, 0) }, "", &(NBarrierInitCall));
     auto bitcastPointerNBarrier = BitCastInst::CreatePointerBitCastOrAddrSpaceCast(pointerNBarrier, m_NamedBarrierType->getPointerTo(SPIRAS_Local), "", &(NBarrierInitCall));
     SmallVector<Value*, 3> ArgsVal
@@ -417,6 +418,6 @@ void NamedBarriersResolution::visitCallInst(CallInst& CI)
 
 bool NamedBarriersResolution::NamedBarrierHWSupport()
 {
-    bool hwSupport = m_GFX_GEN == IGFX_PVC;
+    bool hwSupport = m_GFX_CORE == IGFX_XE_HPC_CORE;
     return hwSupport;
 }
\ No newline at end of file
