@@ -67,7 +67,7 @@ bool PrivateMemoryUsageAnalysis::runOnModule(Module& M)
     // If there are stack called functions in the module, add PRIVATE_BASE to all kernels to be safe.
     // PRIVATE_BASE is needed for kernel to get the stack base offset.
     // Callee does not require this arg, since all stack access will be done using the stack-pointer
-    if (hasStackCall || pCtx->m_enableFunctionPointer)
+    if (hasStackCall || pCtx->m_enableFunctionPointer || IGC_IS_FLAG_ENABLED(ForceAddingStackcallKernelPrerequisites))
     {
         for (Module::iterator I = M.begin(), E = M.end(); I != E; ++I)
         {
