@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2021 Intel Corporation
+Copyright (C) 2017-2023 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -9,6 +9,7 @@ SPDX-License-Identifier: MIT
 #include "AdaptorCommon/ImplicitArgs.hpp"
 #include "AdaptorCommon/AddImplicitArgs.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/ProgramScopeConstants/ProgramScopeConstantAnalysis.hpp"
+#include "Compiler/Optimizer/OpenCLPasses/OpenCLPrintf/OpenCLPrintfAnalysis.hpp"
 #include "Compiler/IGCPassSupport.h"
 #include "common/LLVMWarningsPush.hpp"
 #include <llvm/IR/Module.h>
@@ -128,47 +129,58 @@ bool ProgramScopeConstantAnalysis::runOnModule(Module& M)
         }
 
         DataVector* inlineProgramScopeBuffer = nullptr;
-        if (AS == ADDRESS_SPACE_GLOBAL)
-        {
-            if (!hasInlineGlobalBuffer)
-            {
-                InlineProgramScopeBuffer ilpsb;
-                ilpsb.alignment = 0;
-                ilpsb.allocSize = 0;
-                m_pModuleMd->inlineGlobalBuffers.push_back(ilpsb);
-                hasInlineGlobalBuffer = true;
-            }
-            inlineProgramScopeBuffer = &m_pModuleMd->inlineGlobalBuffers.back().Buffer;
+        auto initInlineGlobalBuffer = [&]() {
+            InlineProgramScopeBuffer ilpsb;
+            ilpsb.alignment = 0;
+            ilpsb.allocSize = 0;
+            m_pModuleMd->inlineGlobalBuffers.push_back(ilpsb);
+            hasInlineGlobalBuffer = true;
+        };
+        auto initInlineConstantBuffer = [&]() {
+            // General constants
+            InlineProgramScopeBuffer ilpsb;
+            ilpsb.alignment = 0;
+            ilpsb.allocSize = 0;
+            m_pModuleMd->inlineConstantBuffers.push_back(ilpsb);
 
+            // String literals
+            InlineProgramScopeBuffer ilpsbString;
+            ilpsbString.alignment = 0;
+            ilpsbString.allocSize = 0;
+            m_pModuleMd->inlineConstantBuffers.push_back(ilpsbString);
+            hasInlineConstantBuffer = true;
+        };
+
+        // When ZeBin is enabled, constant variables that are string literals
+        // used only by printf will be stored in the second constant buffer.
+        ConstantDataSequential* cds = dyn_cast<ConstantDataSequential>(initializer);
+        bool isZebinPrintfStringConst = Ctx->enableZEBinary() &&
+            cds && (cds->isCString() || cds->isString()) &&
+            OpenCLPrintfAnalysis::isPrintfOnlyStringConstant(globalVar);
+        // Here we follow SPV_EXT_relaxed_printf_string_address_space to relax
+        // the address space requirement of printf strings and accept
+        // non-constant address space printf strings. However, we expect it is
+        // the only exception and FE should produce a IGC input that satisfies
+        // any other OpenCL printf restrictions.
+        if (isZebinPrintfStringConst)
+        {
+            if (!hasInlineConstantBuffer)
+                initInlineConstantBuffer();
+            m_pModuleMd->stringConstants.insert(globalVar);
+            inlineProgramScopeBuffer = &m_pModuleMd->inlineConstantBuffers[1].Buffer;
         }
         else
         {
-            if (!hasInlineConstantBuffer)
+            if (AS == ADDRESS_SPACE_GLOBAL)
             {
-                // General constants
-                InlineProgramScopeBuffer ilpsb;
-                ilpsb.alignment = 0;
-                ilpsb.allocSize = 0;
-                m_pModuleMd->inlineConstantBuffers.push_back(ilpsb);
-
-                // String literals
-                InlineProgramScopeBuffer ilpsbString;
-                ilpsbString.alignment = 0;
-                ilpsbString.allocSize = 0;
-                m_pModuleMd->inlineConstantBuffers.push_back(ilpsbString);
-                hasInlineConstantBuffer = true;
-            }
-
-            // When ZeBin is enabled, constant variables that are string literals
-            // will be stored in the second const buffer
-            ConstantDataSequential* cds = dyn_cast<ConstantDataSequential>(initializer);
-            bool isStringConst = cds && (cds->isCString() || cds->isString());
-            if (IGC_IS_FLAG_ENABLED(EnableZEBinary) && isStringConst)
-            {
-                inlineProgramScopeBuffer = &m_pModuleMd->inlineConstantBuffers[1].Buffer;
+                if (!hasInlineGlobalBuffer)
+                    initInlineGlobalBuffer();
+                inlineProgramScopeBuffer = &m_pModuleMd->inlineGlobalBuffers.back().Buffer;
             }
             else
             {
+                if (!hasInlineConstantBuffer)
+                    initInlineConstantBuffer();
                 inlineProgramScopeBuffer = &m_pModuleMd->inlineConstantBuffers[0].Buffer;
             }
         }
@@ -223,14 +235,14 @@ bool ProgramScopeConstantAnalysis::runOnModule(Module& M)
     for (auto globalVar : zeroInitializedGlobals)
     {
         unsigned AS = cast<PointerType>(globalVar->getType())->getAddressSpace();
-        unsigned &offset = (AS == ADDRESS_SPACE_GLOBAL) ? m_pModuleMd->inlineGlobalBuffers.back().allocSize : m_pModuleMd->inlineConstantBuffers[0].allocSize;
+        size_t &offset = (AS == ADDRESS_SPACE_GLOBAL) ? m_pModuleMd->inlineGlobalBuffers.back().allocSize : m_pModuleMd->inlineConstantBuffers[0].allocSize;
 #if LLVM_VERSION_MAJOR < 11
         offset = iSTD::Align(offset, m_DL->getPreferredAlignment(globalVar));
 #else
         offset = iSTD::Align(offset, (unsigned)m_DL->getPreferredAlign(globalVar).value());
 #endif
         inlineProgramScopeOffsets[globalVar] = offset;
-        offset += (unsigned)(m_DL->getTypeAllocSize(globalVar->getType()->getPointerElementType()));
+        offset += (unsigned)(m_DL->getTypeAllocSize(IGCLLVM::getNonOpaquePtrEltTy(globalVar->getType())));
     }
 
     if (inlineProgramScopeOffsets.size())
@@ -245,28 +257,28 @@ bool ProgramScopeConstantAnalysis::runOnModule(Module& M)
         IGC::appendToUsed(M, globalArray);
     }
 
-    // Check if zebin is enabled
-    bool zebinEnable = IGC_IS_FLAG_ENABLED(EnableZEBinary);
+    // Kernels and Subroutines:
+    //  Add the implicit arg to the function argument list if a constant buffer is created and there
+    //  is no stack calls
+    //
+    // Stackcalls:
+    //  Stackcall ABI does not allow implicit args, so rely on relocation for global variable access
 
-    // patch-token-path:
-    //     Just add the implicit argument to each function if a constant
-    //     buffer has been created.  This will technically burn a patch
-    //     token on kernels that don't actually use the buffer but it saves
-    //     us having to walk the def-use chain (we can't just check if a
-    //     constant is used in the kernel; for example, a global buffer
-    //     may contain pointers that in turn point into the constant
-    //     address space).
-    // zebinary path:
-    //     Don't add the implicit arguments and rely solely on relocations
-    //     for global variable reference since the implicit arguments were
-    //     removed from zebinary.
-    if (!zebinEnable && hasInlineConstantBuffer)
+    // Workaround: When there is stringConstants in the module, do not insert
+    // implicit arguments to prevent const vars getting promoted
+    // at statelessToStateful pass. In zebin path, stateful promotion
+    // of const vars can't work well with printf strings.
+    bool skipConstAndGlobalBaseArgs =
+             Ctx->enableZEBinary() && !m_pModuleMd->stringConstants.empty();
+
+    if (!skipConstAndGlobalBaseArgs && hasInlineConstantBuffer)
     {
         for (auto& pFunc : M)
         {
             if (pFunc.isDeclaration()) continue;
+
             // Skip functions called from function marked with stackcall attribute
-            if (AddImplicitArgs::hasStackCallInCG(&pFunc, *Ctx)) continue;
+            if (AddImplicitArgs::hasStackCallInCG(&pFunc)) continue;
 
             // Always add for kernels and subroutines
             SmallVector<ImplicitArg::ArgType, 1> implicitArgs;
@@ -275,13 +287,13 @@ bool ProgramScopeConstantAnalysis::runOnModule(Module& M)
         }
     }
 
-    if (!zebinEnable && hasInlineGlobalBuffer)
+    if (!skipConstAndGlobalBaseArgs && hasInlineGlobalBuffer)
     {
         for (auto& pFunc : M)
         {
             if (pFunc.isDeclaration()) continue;
             // Skip functions called from function marked with stackcall attribute
-            if (AddImplicitArgs::hasStackCallInCG(&pFunc, *Ctx)) continue;
+            if (AddImplicitArgs::hasStackCallInCG(&pFunc)) continue;
 
             // Always add for kernels and subroutines
             SmallVector<ImplicitArg::ArgType, 1> implicitArgs;
@@ -404,10 +416,20 @@ void ProgramScopeConstantAnalysis::addData(Constant* initializer,
     DataVector& inlineProgramScopeBuffer,
     PointerOffsetInfoList& pointerOffsetInfoList,
     BufferOffsetMap& inlineProgramScopeOffsets,
-    unsigned addressSpace)
+    unsigned addressSpace,
+    bool forceAlignmentOne)
 {
     // Initial alignment padding before insert the current constant into the buffer.
-    alignBuffer(inlineProgramScopeBuffer, m_DL->getABITypeAlignment(initializer->getType()));
+    alignment_t typeAlignment = forceAlignmentOne ? 1 : m_DL->getABITypeAlignment(initializer->getType());
+    alignBuffer(inlineProgramScopeBuffer, typeAlignment);
+
+    // If the initializer is packed struct make sure, that every variable inside
+    // will have also align 1
+    if (StructType* s = dyn_cast<StructType>(initializer->getType()))
+    {
+        if (cast<StructType>(s)->isPacked())
+            forceAlignmentOne = true;
+    }
 
     // We need to do extra work with pointers here: we don't know their actual addresses
     // at compile time so we find the offset from the base of the buffer they point to
@@ -427,7 +449,7 @@ void ProgramScopeConstantAnalysis::addData(Constant* initializer,
             // We can only patch global and constant pointers.
             if (pointedToAddrSpace == ADDRESS_SPACE_GLOBAL || pointedToAddrSpace == ADDRESS_SPACE_CONSTANT)
             {
-                if (IGC_IS_FLAG_ENABLED(EnableZEBinary))
+                if (getAnalysis<CodeGenContextWrapper>().getCodeGenContext()->enableZEBinary())
                 {
                     // For zebin, instead of relying on the old patching logic, we can let RT directly patch the
                     // physical address of the previously defined global into the current buffer that uses it.
@@ -475,7 +497,7 @@ void ProgramScopeConstantAnalysis::addData(Constant* initializer,
         {
             inlineProgramScopeBuffer.insert(inlineProgramScopeBuffer.end(), pointerSize, 0);
         }
-        else if (isa<FunctionType>(ptrType->getPointerElementType()))
+        else if (isa<FunctionType>(IGCLLVM::getNonOpaquePtrEltTy(ptrType)))
         {
             // Save patch info for function pointer to be patched later by runtime
             // The initializer value must be a function pointer and has the "referenced-indirectly" attribute
@@ -593,7 +615,7 @@ void ProgramScopeConstantAnalysis::addData(Constant* initializer,
             Constant* C = initializer->getAggregateElement(i);
             IGC_ASSERT_MESSAGE(C, "getAggregateElement returned null, unsupported constant");
             // Since the type may not be primitive, extra alignment is required.
-            addData(C, inlineProgramScopeBuffer, pointerOffsetInfoList, inlineProgramScopeOffsets, addressSpace);
+            addData(C, inlineProgramScopeBuffer, pointerOffsetInfoList, inlineProgramScopeOffsets, addressSpace, forceAlignmentOne);
         }
     }
     // And, finally, we have to handle base types - ints and floats.
@@ -624,5 +646,5 @@ void ProgramScopeConstantAnalysis::addData(Constant* initializer,
 
     // final padding.  This gets used by the vec3 types that will insert zero padding at the
     // end after inserting the actual vector contents (this is due to sizeof(vec3) == 4 * sizeof(scalarType)).
-    alignBuffer(inlineProgramScopeBuffer, m_DL->getABITypeAlignment(initializer->getType()));
+    alignBuffer(inlineProgramScopeBuffer, typeAlignment);
 }
