@@ -169,6 +169,10 @@ void WIFuncResolution::visitCallInst(CallInst& CI)
     {
         wiRes = getSyncBufferPtr(CI);
     }
+    else if (funcName.equals(WIFuncsAnalysis::GET_ASSERT_BUFFER))
+    {
+        wiRes = getAssertBufferPtr(CI);
+    }
     else
     {
         // Non WI function, do nothing
@@ -253,6 +257,7 @@ static_assert(sizeof(packed::implicit_args) == sizeof(implicit_args), "Implicit
 //    uint32_t group_count_z;
 //    uint32_t padding0;
 //    uint64_t rt_global_buffer_ptr;
+//    uint64_t assert_buffer_ptr;
 //};
 
 // For SIMD8:
@@ -579,6 +584,19 @@ Value* WIFuncResolution::getSyncBufferPtr(CallInst& CI)
     return syncBuffer;
 }
 
+Value* WIFuncResolution::getAssertBufferPtr(CallInst& CI)
+{
+    // Receives:
+    // call i8 addrspace(1)* @__builtin_IB_get_assert_buffer()
+
+    // Creates:
+    // i8 addrspace(1)* %assertBuffer
+    auto F = CI.getParent()->getParent();
+    Value* assertBuffer = m_implicitArgs.getImplicitArgValue(*F, ImplicitArg::ASSERT_BUFFER_POINTER, m_pMdUtils);
+
+    return assertBuffer;
+}
+
 
 // Register pass to igc-opt
 #define PASS_FLAG2 "igc-lower-implicit-arg-intrinsic"
@@ -900,6 +918,13 @@ void LowerImplicitArgIntrinsics::visitCallInst(CallInst& CI)
             V = Result;
             break;
         }
+        case GenISAIntrinsic::GenISA_getAssertBufferPtr: {
+            unsigned int Offset = offsetof(implicit_args, assert_buffer_ptr);
+            auto Result = BuildLoadInst(CI, Offset, Builder.getInt64Ty());
+            Result = Builder.CreateIntToPtr(Result, CI.getType());
+            V = Result;
+            break;
+        }
         default:
             break;
         }
