@@ -23,6 +23,7 @@ SPDX-License-Identifier: MIT
 #include "secure_string.h"
 #include "AdaptorCommon/customApi.hpp"
 
+#include <mutex>
 #include <sstream>
 #include <stdlib.h>
 #include <string>
@@ -44,8 +45,6 @@ SPDX-License-Identifier: MIT
 // IGC is expected to put this code in single file, without unncessary llvm (and other dependencies).
 // Then FCL will just include this single file to avoid code duplication and maintainability issues.
 
-#include <mutex>
-
 #if defined(_WIN32 )|| defined( _WIN64 )
 #include <direct.h>
 #include <process.h>
@@ -1538,21 +1537,26 @@ namespace TC
         }
 
         IOCLFEBinaryResult *pResultPtr = NULL;
+        int res = 0;
+        {
 #ifdef _WIN32
-        int res = m_CCModule.pCompile(
+            static std::mutex cclangMtx;
+            std::lock_guard<std::mutex> lck(cclangMtx);
+            res = m_CCModule.pCompile(
 #else
-        int res = Compile(
+            res = Compile(
 #endif
-            pInputArgs->pszProgramSource,
-            (const char**)pInputArgs->inputHeaders.data(),
-            (unsigned int)pInputArgs->inputHeaders.size(),
-            (const char**)pInputArgs->inputHeadersNames.data(),
-            NULL,
-            0,
-            options.c_str(),
-            optionsEx.c_str(),
-            pInputArgs->oclVersion.c_str(),
-            &pResultPtr);
+                pInputArgs->pszProgramSource,
+                (const char**)pInputArgs->inputHeaders.data(),
+                (unsigned int)pInputArgs->inputHeaders.size(),
+                (const char**)pInputArgs->inputHeadersNames.data(),
+                NULL,
+                0,
+                options.c_str(),
+                optionsEx.c_str(),
+                pInputArgs->oclVersion.c_str(),
+                &pResultPtr);
+        }
         if (0 != BuildOptionsAreValid(options, exceptString)) res = -43;
 
         Utils::FillOutputArgs(pResultPtr, pOutputArgs, exceptString);
@@ -1816,7 +1820,9 @@ namespace TC
                     // which is simply not worth it. Without it there's no good and reliable way to get
                     // the kernel names for the hash generation, so the warning is to be emitted instead.
                 {
-                    std::string errorString = pOutputArgs->pErrorString;
+                    std::string errorString("");
+                    if (pOutputArgs->pErrorString)
+                        errorString = pOutputArgs->pErrorString;
                     errorString.append("warning: EnableKernelNamesBasedHash flag doesn't affect .cl dump's hash\n");
                     SetErrorString(errorString.c_str(), pOutputArgs);
                 }
