@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2022 Intel Corporation
+Copyright (C) 2022-2023 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -118,6 +118,24 @@ bool isOverloadedArg(unsigned IntrinID, unsigned ArgNum);
 /// in intrinsic is overloaded
 bool isOverloadedRet(unsigned IntrinID);
 
+std::string getInternalName(ID id, llvm::ArrayRef<llvm::Type *> Tys);
+
+bool isInternalMemoryIntrinsic(ID id);
+
+inline bool isInternalMemoryIntrinsic(const llvm::Value *V) {
+  return isInternalMemoryIntrinsic(getInternalIntrinsicID(V));
+}
+
+inline bool isInternalMemoryIntrinsic(const llvm::Function *F) {
+  return isInternalMemoryIntrinsic(getInternalIntrinsicID(F));
+}
+
+bool isMemoryBlockIntrinsic(const llvm::Instruction *I);
+
+unsigned getMemoryVectorSizePerLane(const llvm::Instruction *I);
+unsigned getMemorySimdWidth(const llvm::Instruction *I);
+unsigned getMemoryRegisterElementSize(const llvm::Instruction *I);
+
 } // namespace vc::InternalIntrinsic
 
 #endif
