@@ -67,10 +67,10 @@ struct GenXBackendOptions {
   // Enable strict debug info validation
   bool DebuggabilityValidateDWARF = false;
 
-  // Special mode of operation for emulation BiF precompilation.
-  // It is expected that this mode is set only by specialized tools and should
-  // not be touched during standard compilation flow.
-  bool BiFEmulationCompilation = false;
+  // Special mode of operation for BiF precompilation. It is expected that this
+  // mode is set only by specialized tools and should not be touched during
+  // standard compilation flow.
+  bool BiFCompilation = false;
 
   // Enable/disable regalloc dump.
   bool DumpRegAlloc = false;
@@ -90,6 +90,7 @@ struct GenXBackendOptions {
 
   // Whether to enable finalizer dumps.
   bool EnableAsmDumps = false;
+  bool EnableIsaDumps = false;
   // Whether to enable dumps of kernel debug information
   bool EnableDebugInfoDumps = false;
   std::string DebugInfoDumpsNameOverride;
@@ -124,6 +125,9 @@ struct GenXBackendOptions {
   // use default value.
   unsigned GRFSize = 0;
 
+  // Use compiler heuristics to determine number of GRF.
+  bool AutoLargeGRF = false;
+
   // Use bindless mode for buffers.
   bool UseBindlessBuffers = false;
 
@@ -169,6 +173,9 @@ struct GenXBackendOptions {
   // Subgroup size used for cross-module calls/returns
   unsigned InteropSubgroupSize = 16;
 
+  // Run auxiliary checker/fixup for GV access clobbering cases.
+  bool CheckGVClobbering = false;
+
   // Compile until vISA stage only.
   bool EmitVisaOnly = false;
 
@@ -189,13 +196,7 @@ struct GenXBackendOptions {
   GenXBackendOptions(InitFromLLVMOpts) { enforceLLVMOptions(); }
 };
 
-enum BiFKind {
-  OCLGeneric,
-  VCPrintf,
-  VCEmulation,
-  VCSPIRVBuiltins,
-  Size
-};
+enum BiFKind { OCLGeneric, VCPrintf, VCBuiltins, VCSPIRVBuiltins, Size };
 
 class GenXBackendData {
   // The owner of OpenCL generic BiF module.
@@ -238,10 +239,8 @@ public:
   // Return whether regalloc results should be printed.
   bool enableRegAllocDump() const { return Options.DumpRegAlloc; }
 
-  // Return whether emulation BiF compilation mode is enabled.
-  bool isBiFEmulationCompilation() const {
-    return Options.BiFEmulationCompilation;
-  }
+  // Return whether BiF compilation mode is enabled.
+  bool isBiFCompilation() const { return Options.BiFCompilation; }
 
   // Return maximum available space in bytes for stack purposes.
   unsigned getStackSurfaceMaxSize() const {
@@ -298,6 +297,7 @@ public:
   }
 
   bool asmDumpsEnabled() const { return Options.EnableAsmDumps; }
+  bool isaDumpsEnabled() const { return Options.EnableIsaDumps; }
   bool dbgInfoDumpsEnabled() const { return Options.EnableDebugInfoDumps; }
   const std::string &dbgInfoDumpsNameOverride() const {
     return Options.DebugInfoDumpsNameOverride;
@@ -339,6 +339,8 @@ public:
 
   unsigned getGRFSize() const { return Options.GRFSize; }
 
+  bool isAutoLargeGRFMode() const { return Options.AutoLargeGRF; }
+
   // Return pointer to WA_TABLE. Can be null.
   const WA_TABLE *getWATable() const {
     return Options.WATable;
@@ -374,6 +376,8 @@ public:
     return Options.InteropSubgroupSize;
   }
 
+  bool checkGVClobbering() const { return Options.CheckGVClobbering; }
+
   bool isHashMovsEnabled() const { return Options.EnableHashMovs; }
   bool isHashMovsAtPrologueEnabled() const {
     return Options.EnableHashMovsAtPrologue;
