@@ -228,6 +228,7 @@ static GenXBackendOptions createBackendOptions(const vc::CompileOptions &Opts) {
 
   BackendOpts.DisableFinalizerMsg = Opts.DisableFinalizerMsg;
   BackendOpts.EnableAsmDumps = Opts.DumpAsm;
+  BackendOpts.EnableIsaDumps = Opts.DumpIsa;
   BackendOpts.EnableDebugInfoDumps = Opts.DumpDebugInfo;
   BackendOpts.Dumper = Opts.Dumper.get();
   BackendOpts.ShaderOverrider = Opts.ShaderOverrider.get();
@@ -245,6 +246,7 @@ static GenXBackendOptions createBackendOptions(const vc::CompileOptions &Opts) {
   BackendOpts.WATable = Opts.WATable;
   if (Opts.GRFSize)
     BackendOpts.GRFSize = Opts.GRFSize.getValue();
+  BackendOpts.AutoLargeGRF = Opts.EnableAutoLargeGRF;
   BackendOpts.UseBindlessBuffers = Opts.UseBindlessBuffers;
   if (Opts.SaveStackCallLinkage)
     BackendOpts.SaveStackCallLinkage = true;
@@ -261,6 +263,8 @@ static GenXBackendOptions createBackendOptions(const vc::CompileOptions &Opts) {
   if (Opts.InteropSubgroupSize)
     BackendOpts.InteropSubgroupSize = Opts.InteropSubgroupSize;
 
+  BackendOpts.CheckGVClobbering = Opts.CheckGVClobbering;
+
   BackendOpts.Binary = Opts.Binary;
 
   BackendOpts.DisableLiveRangesCoalescing =
@@ -288,8 +292,8 @@ static GenXBackendData createBackendData(const vc::ExternalData &Data,
   GenXBackendData BackendData;
   BackendData.BiFModule[BiFKind::OCLGeneric] =
       IGCLLVM::makeMemoryBufferRef(*Data.OCLGenericBIFModule);
-  BackendData.BiFModule[BiFKind::VCEmulation] =
-      IGCLLVM::makeMemoryBufferRef(*Data.VCEmulationBIFModule);
+  BackendData.BiFModule[BiFKind::VCBuiltins] =
+      IGCLLVM::makeMemoryBufferRef(*Data.VCBuiltinsBIFModule);
   BackendData.BiFModule[BiFKind::VCSPIRVBuiltins] =
       IGCLLVM::makeMemoryBufferRef(*Data.VCSPIRVBuiltinsBIFModule);
   if (PointerSizeInBits == 64)
@@ -380,17 +384,9 @@ static void optimizeIR(const vc::CompileOptions &Opts,
   PerModulePasses.run(M);
 }
 
-static void dumpFinalOutput(const vc::CompileOptions &Opts, const Module &M,
-                            StringRef IsaBinary) {
-  if (Opts.DumpIR && Opts.Dumper)
-    Opts.Dumper->dumpModule(M, "final.ll");
-  if (Opts.DumpIsa && Opts.Dumper)
-    Opts.Dumper->dumpBinary({IsaBinary.data(), IsaBinary.size()}, "final.isa");
-}
-
 static void populateCodeGenPassManager(const vc::CompileOptions &Opts,
                                        const vc::ExternalData &ExtData,
-                                       TargetMachine &TM, raw_pwrite_stream &OS,
+                                       TargetMachine &TM,
                                        legacy::PassManager &PM) {
   TargetLibraryInfoImpl TLII{TM.getTargetTriple()};
   PM.add(new TargetLibraryInfoWrapperPass(TLII));
@@ -407,60 +403,25 @@ static void populateCodeGenPassManager(const vc::CompileOptions &Opts,
 
   auto FileType = IGCLLVM::TargetMachine::CodeGenFileType::CGFT_AssemblyFile;
 
+  llvm::raw_null_ostream NOS;
   bool AddPasses =
-      TM.addPassesToEmitFile(PM, OS, nullptr, FileType, DisableIrVerifier);
+      TM.addPassesToEmitFile(PM, NOS, nullptr, FileType, DisableIrVerifier);
   IGC_ASSERT_MESSAGE(!AddPasses, "Bad filetype for vc-codegen");
 }
 
-static vc::ocl::CompileOutput runOclCodeGen(const vc::CompileOptions &Opts,
-                                            const vc::ExternalData &ExtData,
-                                            TargetMachine &TM, Module &M) {
-  vc::PassManager PM;
-
-  SmallString<32> IsaBinary;
-  raw_svector_ostream OS(IsaBinary);
-  raw_null_ostream NullOS;
-  if (Opts.DumpIsa)
-    populateCodeGenPassManager(Opts, ExtData, TM, OS, PM);
-  else
-    populateCodeGenPassManager(Opts, ExtData, TM, NullOS, PM);
-
-  GenXOCLRuntimeInfo::CompiledModuleT CompiledModule;
-  PM.add(createGenXOCLInfoExtractorPass(CompiledModule));
-
-  PM.run(M);
-  dumpFinalOutput(Opts, M, IsaBinary);
-
-  return CompiledModule;
-}
-
-static vc::cm::CompileOutput runCmCodeGen(const vc::CompileOptions &Opts,
-                                          const vc::ExternalData &ExtData,
-                                          TargetMachine &TM, Module &M) {
-  vc::PassManager PM;
-
-  SmallString<32> IsaBinary;
-  raw_svector_ostream OS(IsaBinary);
-  populateCodeGenPassManager(Opts, ExtData, TM, OS, PM);
-  PM.run(M);
-  dumpFinalOutput(Opts, M, IsaBinary);
-  vc::cm::CompileOutput Output;
-  Output.IsaBinary.assign(IsaBinary.begin(), IsaBinary.end());
-  return Output;
-}
-
 static vc::CompileOutput runCodeGen(const vc::CompileOptions &Opts,
                                     const vc::ExternalData &ExtData,
                                     TargetMachine &TM, Module &M) {
-  switch (Opts.Binary) {
-  case vc::BinaryKind::CM:
-    return runCmCodeGen(Opts, ExtData, TM, M);
-  case vc::BinaryKind::OpenCL:
-  case vc::BinaryKind::ZE:
-    return runOclCodeGen(Opts, ExtData, TM, M);
-  default:
-      IGC_ASSERT_UNREACHABLE(); // Unknown binary format
-  }
+  vc::PassManager PM;
+
+  populateCodeGenPassManager(Opts, ExtData, TM, PM);
+
+  vc::CompileOutput CompiledModule;
+  PM.add(createGenXOCLInfoExtractorPass(CompiledModule));
+
+  PM.run(M);
+
+  return CompiledModule;
 }
 
 // Parse global llvm cl options.
@@ -587,6 +548,9 @@ Expected<vc::CompileOutput> vc::Compile(ArrayRef<char> Input,
 
   vc::CompileOutput Output = runCodeGen(Opts, ExtData, TM, M);
 
+  if (Opts.DumpIR && Opts.Dumper)
+    Opts.Dumper->dumpModule(M, "final.ll");
+
   printLLVMStats(Opts);
   printLLVMTimers(Opts);
   return Output;
@@ -695,7 +659,7 @@ deriveOptimizationLevel(opt::Arg *A, OptSpecifier PrimaryOpt) {
     StringRef Val = A->getValue();
     return parseOptimizationLevelString(Val);
   } else {
-    IGC_ASSERT(A->getOption().matches(OPT_opt_disable_ze));
+    IGC_ASSERT(A->getOption().matches(OPT_opt_disable_common));
     return vc::OptimizerLevel::None;
   }
 }
@@ -720,14 +684,16 @@ static Error fillApiOptions(const opt::ArgList &ApiOptions,
     Opts.UsePlain2DImages = true;
   if (ApiOptions.hasArg(OPT_vc_enable_preemption))
     Opts.EnablePreemption = true;
-  if (ApiOptions.hasArg(OPT_library_compilation_ze))
+  if (ApiOptions.hasArg(OPT_library_compilation_common))
     Opts.SaveStackCallLinkage = true;
   if (ApiOptions.hasArg(OPT_vc_disable_non_overlapping_region_opt))
     Opts.ForceDisableNonOverlappingRegionOpt = true;
   if (ApiOptions.hasArg(OPT_vc_disable_indvars_opt))
     Opts.ForceDisableIndvarsOpt = true;
+  if (ApiOptions.hasArg(OPT_enable_auto_large_GRF_mode_common))
+    Opts.EnableAutoLargeGRF = true;
 
-  if (opt::Arg *A = ApiOptions.getLastArg(OPT_register_file_size)) {
+  if (opt::Arg *A = ApiOptions.getLastArg(OPT_exp_register_file_size_common)) {
     StringRef V = A->getValue();
     auto MaybeGRFSize = StringSwitch<Optional<unsigned>>(V)
                             .Case("128", 128)
@@ -739,12 +705,12 @@ static Error fillApiOptions(const opt::ArgList &ApiOptions,
   }
 
   if (opt::Arg *A =
-          ApiOptions.getLastArg(OPT_enable_zebin_ze, OPT_disable_zebin_ze))
+          ApiOptions.getLastArg(OPT_enable_zebin_common, OPT_disable_zebin_common))
     switch (A->getOption().getID()) {
-    case OPT_enable_zebin_ze:
+    case OPT_enable_zebin_common:
       Opts.Binary = vc::BinaryKind::ZE;
       break;
-    case OPT_disable_zebin_ze:
+    case OPT_disable_zebin_common:
       Opts.Binary = vc::BinaryKind::OpenCL;
       break;
     default:
@@ -765,7 +731,7 @@ static Error fillApiOptions(const opt::ArgList &ApiOptions,
   }
 
   if (opt::Arg *A =
-          ApiOptions.getLastArg(OPT_vc_optimize, OPT_opt_disable_ze)) {
+          ApiOptions.getLastArg(OPT_vc_optimize, OPT_opt_disable_common)) {
     auto MaybeLevel = deriveOptimizationLevel(A, OPT_vc_optimize);
     if (!MaybeLevel)
       return makeOptionError(*A, ApiOptions, /*IsInternal=*/false);
@@ -777,7 +743,7 @@ static Error fillApiOptions(const opt::ArgList &ApiOptions,
   }
 
   if (opt::Arg *A =
-          ApiOptions.getLastArg(OPT_vc_codegen_optimize, OPT_opt_disable_ze)) {
+          ApiOptions.getLastArg(OPT_vc_codegen_optimize, OPT_opt_disable_common)) {
     auto MaybeLevel = deriveOptimizationLevel(A, OPT_vc_codegen_optimize);
     if (!MaybeLevel)
       return makeOptionError(*A, ApiOptions, /*IsInternal=*/false);
@@ -814,9 +780,9 @@ static Error fillInternalOptions(const opt::ArgList &InternalOptions,
   if (InternalOptions.hasArg(OPT_freset_llvm_stats))
     Opts.ResetLLVMStats = true;
   Opts.StatsFile = InternalOptions.getLastArgValue(OPT_stats_file).str();
-  if (InternalOptions.hasArg(OPT_intel_use_bindless_buffers_ze))
+  if (InternalOptions.hasArg(OPT_use_bindless_buffers_common))
     Opts.UseBindlessBuffers = true;
-  if (InternalOptions.hasArg(OPT_emit_zebin_visa_sections))
+  if (InternalOptions.hasArg(OPT_emit_zebin_visa_sections_common))
     Opts.EmitZebinVisaSections = true;
   if (InternalOptions.hasArg(OPT_fdisable_debuggable_kernels))
     Opts.EmitDebuggableKernels = false;
@@ -842,12 +808,12 @@ static Error fillInternalOptions(const opt::ArgList &InternalOptions,
   }
 
   if (opt::Arg *A = InternalOptions.getLastArg(
-          OPT_binary_format, OPT_allow_zebin_ze, OPT_disable_zebin_ze)) {
+          OPT_binary_format, OPT_allow_zebin_common, OPT_disable_zebin_common)) {
     auto OptID = A->getOption().getID();
 
-    if (OptID == OPT_allow_zebin_ze)
+    if (OptID == OPT_allow_zebin_common)
       Opts.Binary = vc::BinaryKind::ZE;
-    else if (OptID == OPT_disable_zebin_ze)
+    else if (OptID == OPT_disable_zebin_common)
       Opts.Binary = vc::BinaryKind::OpenCL;
     else {
       StringRef Val = A->getValue();
@@ -915,12 +881,12 @@ static std::string composeLLVMArgs(const opt::ArgList &ApiArgs,
   }
 
   // Add gtpin options if any.
-  if (ApiArgs.hasArg(IGC::options::api::OPT_gtpin_rera))
+  if (ApiArgs.hasArg(IGC::options::api::OPT_gtpin_rera_common))
     Result += " -finalizer-opts='-GTPinReRA'";
-  if (ApiArgs.hasArg(IGC::options::api::OPT_gtpin_grf_info))
+  if (ApiArgs.hasArg(IGC::options::api::OPT_gtpin_grf_info_common))
     Result += " -finalizer-opts='-getfreegrfinfo -rerapostschedule'";
   if (opt::Arg *A =
-          ApiArgs.getLastArg(IGC::options::api::OPT_gtpin_scratch_area_size)) {
+          ApiArgs.getLastArg(IGC::options::api::OPT_gtpin_scratch_area_size_common)) {
     Result += " -finalizer-opts='-GTPinScratchAreaSize ";
     Result += A->getValue();
     Result += "'";
