@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2022 Intel Corporation
+Copyright (C) 2017-2023 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -855,8 +855,10 @@ CategoryAndAlignment GenXCategory::getCategoryAndAlignmentForDef(Value *V) const
       IGC_ASSERT(IntrinsicID != GenXIntrinsic::genx_convert);
       if (IntrinsicID == GenXIntrinsic::genx_convert_addr)
         return vc::RegCategory::Address;
-      if (GenXIntrinsic::isAnyNonTrivialIntrinsic(IntrinsicID) && !GenXIntrinsic::isRdRegion(IntrinsicID)
-          && !GenXIntrinsic::isWrRegion(IntrinsicID) && !GenXIntrinsic::isAbs(IntrinsicID)) {
+      if (vc::isAnyNonTrivialIntrinsic(IntrinsicID) &&
+          !GenXIntrinsic::isRdRegion(IntrinsicID) &&
+          !GenXIntrinsic::isWrRegion(IntrinsicID) &&
+          !GenXIntrinsic::isAbs(IntrinsicID)) {
         // For any normal intrinsic, look up the argument class.
         GenXIntrinsicInfo II(IntrinsicID);
         auto AI = II.getRetInfo();
@@ -947,13 +949,12 @@ CategoryAndAlignment GenXCategory::getCategoryAndAlignmentForUse(
     if (CI->isInlineAsm())
       Category = getCategoryForInlasmConstraintedOp(CI, U->getOperandNo(),
                                                     false /*IsOutput*/);
-    else if (IGCLLVM::isIndirectCall(*CI))
+    else if (CI->isIndirectCall())
       Category = vc::RegCategory::General;
     else {
       Function *Callee = CI->getCalledFunction();
-      unsigned IntrinID = GenXIntrinsic::not_any_intrinsic;
-      if (Callee)
-        IntrinID = vc::getAnyIntrinsicID(Callee);
+      IGC_ASSERT(nullptr != Callee);
+      unsigned IntrinID = vc::getAnyIntrinsicID(Callee);
       // We should not see genx_convert, as it is inserted into a value after
       // using this function to determine its category.
       IGC_ASSERT(IntrinID != GenXIntrinsic::genx_convert);
