@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2021 Intel Corporation
+Copyright (C) 2017-2023 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -22,8 +22,8 @@ SPDX-License-Identifier: MIT
 /// coalesces the result of the copy into the result of the phi node or
 /// two address op).
 ///
-/// There are three different kinds of coalescing. Copy coalescing is done first,
-/// then the other two are done together.
+/// There are three different kinds of coalescing. Copy coalescing is done
+/// first, then the other two are done together.
 ///
 /// 1. Copy coalescing.
 ///
@@ -327,10 +327,14 @@ namespace {
     void visitPHINode(PHINode &Phi);
     void visitCallInst(CallInst &CI);
     void visitGenXIntrinsicInst(GenXIntrinsicInst &II);
+    void visitInternalIntrinsicInst(InternalIntrinsicInst &II);
     void visitCastInst(CastInst &CI);
     void visitExtractValueInst(ExtractValueInst &EVI);
     void visitInsertValueInst(InsertValueInst &IVI);
+
   private:
+    void processTwoAddrIntrinsic(CallInst &CI);
+
     unsigned getPriority(Type *Ty, BasicBlock *BB) const;
     unsigned getPriority(SimpleValue Val) const;
     // Various permutations of the function to record a coalescing candidate.
@@ -564,20 +568,20 @@ void GenXCoalescing::visitCallInst(CallInst &CI) {
 }
 
 /***********************************************************************
- * visitGenXIntrinsicInst : process an intrinsic with a two address operand
+ * processTwoAddrIntrinsic: process an intrinsic with a two address operand
  * (including the case of operand 0 in wrregion). That operand has to be in
  * the same register as the result.
- * NOTE: *.predef.reg intrinsics should not participate in coalescing
- * since they don't have any LR
  */
-void GenXCoalescing::visitGenXIntrinsicInst(GenXIntrinsicInst &II) {
-  if (GenXIntrinsic::isReadWritePredefReg(&II))
-    return;
-  auto OperandNum = getTwoAddressOperandNum(&II);
+void GenXCoalescing::processTwoAddrIntrinsic(CallInst &CI) {
+  auto IID = vc::getAnyIntrinsicID(&CI);
+  IGC_ASSERT(vc::isAnyNonTrivialIntrinsic(IID));
+
+  auto OperandNum = getTwoAddressOperandNum(&CI);
   if (!OperandNum)
     return;
-  if (Baling->isBaled(&II) ||
-      GenXIntrinsic::isReadWritePredefReg(II.getOperand(0))) {
+
+  if (Baling->isBaled(&CI) ||
+      GenXIntrinsic::isReadWritePredefReg(CI.getOperand(0))) {
     // The intrinsic is baled into a wrregion. The two address
     // operand must also have a rdregion baled in whose input is
     // the "old value" input of the wrregion, and the coalescing
@@ -586,8 +590,21 @@ void GenXCoalescing::visitGenXIntrinsicInst(GenXIntrinsicInst &II) {
     // anything here.
     return;
   }
+
   // Normal unbaled twoaddr operand.
-  recordNormalCandidate(&II, *OperandNum);
+  recordNormalCandidate(&CI, *OperandNum);
+}
+
+void GenXCoalescing::visitInternalIntrinsicInst(InternalIntrinsicInst &II) {
+  processTwoAddrIntrinsic(II);
+}
+
+void GenXCoalescing::visitGenXIntrinsicInst(GenXIntrinsicInst &II) {
+  // *.predef.reg intrinsics should not participate in coalescing since they
+  // don't have any LR
+  if (GenXIntrinsic::isReadWritePredefReg(&II))
+    return;
+  processTwoAddrIntrinsic(II);
 }
 
 /***********************************************************************
@@ -2111,4 +2128,3 @@ void GenXCoalescing::replaceAllUsesWith(Instruction *OldInst,
                ToCopy.end());
   OldInst->replaceAllUsesWith(NewInst);
 }
-
