@@ -12,6 +12,7 @@ SPDX-License-Identifier: MIT
 #include "GenXSubtarget.h"
 #include "llvm/ADT/SmallVector.h"
 #include "llvm/IR/Constant.h"
+#include "llvm/IR/Constants.h"
 #include "llvm/IR/Dominators.h"
 #include "llvm/IR/Instructions.h"
 
@@ -94,6 +95,12 @@ private:
   Instruction *loadNonSimple(Instruction *InsertBefore);
   Instruction *loadSplatConstant(Instruction *InsertPt);
   Instruction *loadNonPackedIntConst(Instruction *InsertPt);
+
+  Instruction *loadPackedInt(Instruction *InsertBefore);
+  Instruction *loadCyclicConstruct(Instruction *InsertBefore,
+                                   const SmallVectorImpl<Constant *> &Elements,
+                                   unsigned UndefBits, unsigned RemainingBits,
+                                   Instruction *Result);
 };
 
 // Some instructions force their operands to be constants.
@@ -111,7 +118,11 @@ inline bool opMustBeConstant(Instruction *I, unsigned OpNum) {
 bool areConstantsEqual(const Constant *C1, const Constant *C2);
 
 // Remove all genx.constant* intrinsics that have non-constant source
-bool cleanupConstantLoads(Function *F);
+bool cleanupConstantLoads(Function *F, std::vector<CallInst *> &ConstList);
+
+// Replace one of paired constants to add-instruction
+bool checkAddPattern(Function *F, std::vector<CallInst *> &ConstList,
+    const GenXSubtarget *ST);
 
 // Load a constant using the llvm.genx.constant intrinsic.
 inline Instruction *
