@@ -636,7 +636,7 @@ ModuleToVisaTransformInfo::ModuleToVisaTransformInfo(
 namespace {
 
 class GenObjectWrapper {
-  FINALIZER_INFO *JitInfo = nullptr;
+  vISA::FINALIZER_INFO *JitInfo = nullptr;
   std::unique_ptr<IGC::VISADebugInfo> VISADebugInfo;
   // TODO: remove this once DbgDecoder is refactored
   unsigned GenDbgInfoDataSize = 0;
@@ -676,7 +676,7 @@ public:
     return *VISADebugInfo;
   }
 
-  const FINALIZER_INFO &getJitInfo() const {
+  const vISA::FINALIZER_INFO &getJitInfo() const {
     IGC_ASSERT(!hasErrors() && JitInfo);
     return *JitInfo;
   };
@@ -749,7 +749,7 @@ class CompiledVisaWrapper {
   }
 
 public:
-  const FINALIZER_INFO &getJitInfo() const { return GOW.getJitInfo(); };
+  const vISA::FINALIZER_INFO &getJitInfo() const { return GOW.getJitInfo(); };
 
   const FinalizedDI &getFinalizerDI() const {
     IGC_ASSERT(ErrMsg.empty() && VisaKernelDI);
@@ -892,7 +892,7 @@ public:
     return ST.getGRFByteSize();
   }
   unsigned getNumGRFs() const override {
-    return CompiledVisa.getJitInfo().numGRFTotal;
+    return CompiledVisa.getJitInfo().stats.numGRFTotal;
   }
   unsigned getPointerSize() const override {
     return F.getParent()->getDataLayout().getPointerSize();
@@ -1057,6 +1057,8 @@ public:
   int getFPReg() const override { return -1; }
   uint64_t getFPOffset() const override { return 16; }
 
+  bool usesSlot1ScratchSpill() const override { return false; }
+
   const GenXVisaRegAlloc::Reg *getRegisterForValue(const Value *V) const {
     return RA.getRegForValueOrNull(const_cast<Value *>(V));
   }
