@@ -15,6 +15,7 @@ SPDX-License-Identifier: MIT
 #include "GenXTargetMachine.h"
 #include "GenXUtil.h"
 #include "OCLRuntimeInfoPrinter.h"
+#include "GenXGlobalUniform.h"
 
 #include "vc/Utils/GenX/GlobalVariable.h"
 #include "vc/Utils/GenX/InternalMetadata.h"
@@ -104,15 +105,29 @@ private:
     return [Attr](StringRef Token) { return Token == Attr; };
   }
 
-  ArgKindType getOCLArgKind(ArrayRef<StringRef> Tokens, unsigned ArgNo) const;
+  ArgKindType getOCLArgKind(ArrayRef<StringRef> Tokens,
+                            const Argument &Arg) const;
   ArgAccessKindType getOCLArgAccessKind(ArrayRef<StringRef> Tokens,
                                         ArgKindType Kind) const;
   std::pair<ArgKindType, ArgAccessKindType>
-  translateArgDesc(unsigned ArgNo) const;
+  translateArgDesc(const Argument &Arg) const;
   unsigned getArgSizeInBytes(const Argument &Arg) const;
 };
 } // namespace llvm
 
+static alignment_t
+getAlignment(const Argument &Arg,
+             GenXOCLRuntimeInfo::KernelArgInfo::KindType Kind) {
+  using ArgKindType = GenXOCLRuntimeInfo::KernelArgInfo::KindType;
+  if (Kind != ArgKindType::SLM)
+    return 0;
+
+  Type *TypeToAlign = Arg.getType();
+  TypeToAlign = IGCLLVM::getNonOpaquePtrEltTy(TypeToAlign);
+  return Arg.getParent()->getParent()->getDataLayout().getABITypeAlignment(
+      TypeToAlign);
+}
+
 KernelArgBuilder::ArgAccessKindType
 KernelArgBuilder::getOCLArgAccessKind(ArrayRef<StringRef> Tokens,
                                       ArgKindType Kind) const {
@@ -130,6 +145,7 @@ KernelArgBuilder::getOCLArgAccessKind(ArrayRef<StringRef> Tokens,
       return ArgAccessKindType::ReadOnly;
     if (any_of(Tokens, getStrPred(OCLAttributes::WriteOnly)))
       return ArgAccessKindType::WriteOnly;
+  case ArgKindType::SLM:
     return ArgAccessKindType::ReadWrite;
   default:
     return ArgAccessKindType::None;
@@ -138,14 +154,18 @@ KernelArgBuilder::getOCLArgAccessKind(ArrayRef<StringRef> Tokens,
 
 KernelArgBuilder::ArgKindType
 KernelArgBuilder::getOCLArgKind(ArrayRef<StringRef> Tokens,
-                                unsigned ArgNo) const {
+                                const Argument &Arg) const {
+  unsigned ArgNo = Arg.getArgNo();
   unsigned RawKind = KM.getArgKind(ArgNo);
+  const auto *ArgTy = Arg.getType();
 
   // Implicit arguments.
   if (vc::isLocalSizeKind(RawKind))
     return ArgKindType::LocalSize;
   if (vc::isGroupCountKind(RawKind))
     return ArgKindType::GroupCount;
+  if (vc::isAssertBufferKind(RawKind))
+    return ArgKindType::AssertBuffer;
   if (vc::isPrintBufferKind(RawKind))
     return ArgKindType::PrintBuffer;
   if (vc::isPrivateBaseKind(RawKind))
@@ -165,6 +185,9 @@ KernelArgBuilder::getOCLArgKind(ArrayRef<StringRef> Tokens,
     // Bindless buffers have general category but buffer annotation.
     if (any_of(Tokens, getStrPred(OCLAttributes::Buffer)))
       return ArgKindType::BindlessBuffer;
+    if (ArgTy->isPointerTy())
+      if (vc::getAddrSpace(ArgTy) == vc::AddrSpace::Local)
+        return ArgKindType::SLM;
     return ArgKindType::General;
   case vc::RegCategory::Surface:
     if (any_of(Tokens, getStrPred(OCLAttributes::Image1d)))
@@ -196,7 +219,8 @@ KernelArgBuilder::getOCLArgKind(ArrayRef<StringRef> Tokens,
 
 // Retrieve Kind and AccessKind from given ArgTypeDesc in metadata.
 std::pair<KernelArgBuilder::ArgKindType, KernelArgBuilder::ArgAccessKindType>
-KernelArgBuilder::translateArgDesc(unsigned ArgNo) const {
+KernelArgBuilder::translateArgDesc(const Argument &Arg) const {
+  unsigned ArgNo = Arg.getArgNo();
   std::string Translated{KM.getArgTypeDesc(ArgNo)};
   // Transform each separator to space.
   std::transform(Translated.begin(), Translated.end(), Translated.begin(),
@@ -213,7 +237,7 @@ KernelArgBuilder::translateArgDesc(unsigned ArgNo) const {
   std::sort(Tokens.begin(), Tokens.end());
   Tokens.erase(std::unique(Tokens.begin(), Tokens.end()), Tokens.end());
 
-  const ArgKindType Kind = getOCLArgKind(Tokens, ArgNo);
+  const ArgKindType Kind = getOCLArgKind(Tokens, Arg);
   const ArgAccessKindType AccessKind = getOCLArgAccessKind(Tokens, Kind);
   return {Kind, AccessKind};
 }
@@ -224,14 +248,14 @@ unsigned KernelArgBuilder::getArgSizeInBytes(const Argument &Arg) const {
     return DL.getPointerTypeSize(ArgTy);
   if (KM.isBufferType(Arg.getArgNo()))
     return DL.getPointerSize();
-  return ArgTy->getPrimitiveSizeInBits() / genx::ByteBits;
+  return DL.getTypeSizeInBits(ArgTy) / genx::ByteBits;
 }
 
 GenXOCLRuntimeInfo::KernelArgInfo
 KernelArgBuilder::translateArgument(const Argument &Arg) const {
   GenXOCLRuntimeInfo::KernelArgInfo Info;
-  const unsigned ArgNo = Arg.getArgNo();
-  std::tie(Info.Kind, Info.AccessKind) = translateArgDesc(ArgNo);
+  unsigned ArgNo = Arg.getArgNo();
+  std::tie(Info.Kind, Info.AccessKind) = translateArgDesc(Arg);
   Info.Offset = KM.getArgOffset(ArgNo);
   Info.SizeInBytes = getArgSizeInBytes(Arg);
   Info.BTI = KM.getBTI(ArgNo);
@@ -241,6 +265,7 @@ KernelArgBuilder::translateArgument(const Argument &Arg) const {
   // Linearization arguments have a non-zero offset in the original explicit
   // byval arg.
   Info.OffsetInArg = KM.getOffsetInArg(ArgNo);
+  Info.Alignment = (unsigned)getAlignment(Arg, Info.Kind);
 
   return Info;
 }
@@ -252,7 +277,8 @@ KernelArgBuilder::translateArgument(const Argument &Arg) const {
 //===----------------------------------------------------------------------===//
 // Just perform linear instructions scan to find usage stats.
 void GenXOCLRuntimeInfo::KernelInfo::setInstructionUsageProperties(
-    const FunctionGroup &FG, const GenXBackendConfig &BC) {
+    const FunctionGroup &FG, GenXOCLRuntimeInfo &RI,
+    const GenXBackendConfig &BC) {
   for (Function *F : FG) {
     for (BasicBlock &BB : *F) {
       for (Instruction &I : BB) {
@@ -273,10 +299,15 @@ void GenXOCLRuntimeInfo::KernelInfo::setInstructionUsageProperties(
         case GenXIntrinsic::genx_sample_unorm:
           UsesSample = true;
           break;
-        case GenXIntrinsic::genx_dpas:
         case GenXIntrinsic::genx_dpas2:
-        case GenXIntrinsic::genx_dpasw:
         case GenXIntrinsic::genx_dpas_nosrc0:
+        case GenXIntrinsic::genx_dpas:
+          if (!DisableEUFusion) {
+            const auto &GUA = RI.getAnalysis<GenXGlobalUniformAnalysis>(*F);
+            if (!GUA.isUniform(BB))
+              DisableEUFusion = true;
+          }
+        case GenXIntrinsic::genx_dpasw:
         case GenXIntrinsic::genx_dpasw_nosrc0:
           UsesDPAS = true;
           break;
@@ -349,9 +380,12 @@ GenXOCLRuntimeInfo::KernelInfo::KernelInfo(const GenXSubtarget &ST)
                                                    ST.getGRFByteSize()} {}
 
 GenXOCLRuntimeInfo::KernelInfo::KernelInfo(const FunctionGroup &FG,
+                                           GenXOCLRuntimeInfo &RI,
                                            const GenXSubtarget &ST,
                                            const GenXBackendConfig &BC) {
-  setInstructionUsageProperties(FG, BC);
+  DisableEUFusion = BC.isDisableEUFusion();
+
+  setInstructionUsageProperties(FG, RI, BC);
 
   GRFSizeInBytes = ST.getGRFByteSize();
 
@@ -359,7 +393,6 @@ GenXOCLRuntimeInfo::KernelInfo::KernelInfo(const FunctionGroup &FG,
       vc::getStackAmount(FG.getHead(), BC.getStatelessPrivateMemSize());
 
   SupportsDebugging = BC.emitDebuggableKernels();
-  DisableEUFusion = BC.isDisableEUFusion();
 
   vc::KernelMetadata KM{FG.getHead()};
   IGC_ASSERT_MESSAGE(KM.isKernel(), "Expected kernel as head of function group");
@@ -408,6 +441,7 @@ struct ModuleDataT {
   RawSectionInfo Global;
   RawSectionInfo ConstString;
   GenXOCLRuntimeInfo::SymbolSeq ExternalGlobals;
+  GenXOCLRuntimeInfo::SymbolSeq ZeroInitGlobals;
 
   ModuleDataT() = default;
   ModuleDataT(const Module &M);
@@ -618,6 +652,7 @@ ModuleDataT::ModuleDataT(const Module &M) {
       make_filter_range(M.globals(), [](const GlobalVariable &GV) {
         return vc::isRealGlobalVariable(GV);
       });
+  SmallVector<const GlobalVariable*, 4> ZeroInitialized;
   for (auto &GV : RealGlobals) {
     if (GV.isConstant()) {
       if (GV.hasAttribute(vc::PrintfStringVariable))
@@ -632,13 +667,27 @@ ModuleDataT::ModuleDataT(const Module &M) {
     } else if (GV.hasInitializer()) {
       IGC_ASSERT_MESSAGE(!GV.hasAttribute(vc::PrintfStringVariable),
                          "non-const global variable cannot be a printf string");
-      appendGlobalVariableData(Global, GV, DL);
+      auto *Init = GV.getInitializer();
+      if (Init->isNullValue())
+        ZeroInitialized.push_back(&GV);
+      else
+        appendGlobalVariableData(Global, GV, DL);
     } else {
       // External global variables
       auto Name = GV.getName().str();
       ExternalGlobals.emplace_back(vISA::S_UNDEF, 0, 0, Name.c_str());
     }
   }
+
+  uint32_t Offset = Global.Data.getFullSize();
+  for (auto *GV : ZeroInitialized) {
+    Offset = alignTo(Offset, getAlignment(*GV));
+    uint32_t Size =
+        vc::getTypeSize(GV->getInitializer()->getType(), &DL).inBytes();
+    ZeroInitGlobals.emplace_back(vISA::S_GLOBAL_VAR, Offset, Size,
+                                 GV->getName().str().c_str());
+    Offset += Size;
+  }
 }
 
 static GenXOCLRuntimeInfo::ModuleInfoT getModuleInfo(const Module &M) {
@@ -655,6 +704,8 @@ static GenXOCLRuntimeInfo::ModuleInfoT getModuleInfo(const Module &M) {
       ModuleData.ConstString.Data.begin(), ModuleData.ConstString.Data.end(),
       std::back_inserter(ModuleInfo.ConstString.Symbols));
 
+  llvm::copy(ModuleData.ZeroInitGlobals,
+             std::back_inserter(ModuleInfo.Global.Symbols));
   llvm::copy(ModuleData.ExternalGlobals,
              std::back_inserter(ModuleInfo.Global.Symbols));
 
@@ -672,7 +723,12 @@ static GenXOCLRuntimeInfo::ModuleInfoT getModuleInfo(const Module &M) {
   ModuleInfo.Global.Data.Buffer =
       std::move(ModuleData.Global.Data).emitConsolidatedData();
   ModuleInfo.Global.Data.Alignment = 0;
-  ModuleInfo.Global.Data.AdditionalZeroedSpace = 0;
+  if (!ModuleData.ZeroInitGlobals.empty()) {
+    auto &Last = ModuleData.ZeroInitGlobals.back();
+    ModuleInfo.Global.Data.AdditionalZeroedSpace =
+        Last.s_offset + Last.s_size - ModuleInfo.Global.Data.Buffer.size();
+  } else
+    ModuleInfo.Global.Data.AdditionalZeroedSpace = 0;
 
   ModuleInfo.ConstString.Data.Buffer =
       std::move(ModuleData.ConstString.Data).emitConsolidatedData();
@@ -685,6 +741,7 @@ static GenXOCLRuntimeInfo::ModuleInfoT getModuleInfo(const Module &M) {
 namespace {
 
 class RuntimeInfoCollector final {
+  GenXOCLRuntimeInfo &RI;
   const FunctionGroupAnalysis &FGA;
   const GenXBackendConfig &BC;
   VISABuilder &VB;
@@ -698,11 +755,13 @@ public:
   using CompiledModuleT = GenXOCLRuntimeInfo::CompiledModuleT;
 
 public:
-  RuntimeInfoCollector(const FunctionGroupAnalysis &InFGA,
+  RuntimeInfoCollector(GenXOCLRuntimeInfo &InRI,
+                       const FunctionGroupAnalysis &InFGA,
                        const GenXBackendConfig &InBC, VISABuilder &InVB,
                        const GenXSubtarget &InST, const Module &InM,
                        const GenXDebugInfo &InDbg)
-      : FGA{InFGA}, BC{InBC}, VB{InVB}, ST{InST}, M{InM}, DBG{InDbg} {}
+      : RI{InRI}, FGA{InFGA}, BC{InBC}, VB{InVB}, ST{InST}, M{InM}, DBG{InDbg} {
+  }
 
   CompiledModuleT run();
 
@@ -754,7 +813,7 @@ RuntimeInfoCollector::collectFunctionGroupInfo(const FunctionGroup &FG) const {
   using CompiledKernel = GenXOCLRuntimeInfo::CompiledKernel;
 
   // Compiler info.
-  KernelInfo Info{FG, ST, BC};
+  KernelInfo Info{FG, RI, ST, BC};
 
   const Function *KernelFunction = FG.getHead();
   const std::string KernelName = KernelFunction->getName().str();
@@ -868,6 +927,7 @@ void GenXOCLRuntimeInfo::getAnalysisUsage(AnalysisUsage &AU) const {
   AU.addRequired<GenXModule>();
   AU.addRequired<GenXDebugInfo>();
   AU.addRequired<TargetPassConfig>();
+  AU.addRequired<GenXGlobalUniformAnalysis>();
   AU.setPreservesAll();
 }
 
@@ -884,7 +944,7 @@ bool GenXOCLRuntimeInfo::runOnModule(Module &M) {
   VISABuilder &VB =
       *((GM.HasInlineAsm() || !BC.getVISALTOStrings().empty()) ? GM.GetVISAAsmReader() : GM.GetCisaBuilder());
 
-  CompiledModule = RuntimeInfoCollector{FGA, BC, VB, ST, M, DBG}.run();
+  CompiledModule = RuntimeInfoCollector{*this, FGA, BC, VB, ST, M, DBG}.run();
   return false;
 }
 
@@ -899,5 +959,6 @@ INITIALIZE_PASS_DEPENDENCY(GenXBackendConfig);
 INITIALIZE_PASS_DEPENDENCY(GenXModule);
 INITIALIZE_PASS_DEPENDENCY(GenXDebugInfo);
 INITIALIZE_PASS_DEPENDENCY(TargetPassConfig);
+INITIALIZE_PASS_DEPENDENCY(GenXGlobalUniformAnalysis);
 INITIALIZE_PASS_END(GenXOCLRuntimeInfo, "GenXOCLRuntimeInfo",
                     "GenXOCLRuntimeInfo", false, true)
