@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2021 Intel Corporation
+Copyright (C) 2017-2022 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -301,7 +301,7 @@ Region genx::makeRegionFromBaleInfo(const Instruction *Inst, const BaleInfo &BI,
  */
 unsigned genx::getLegalRegionSizeForTarget(const GenXSubtarget &ST,
                                            const Region &R, unsigned Idx,
-                                           bool Allow2D,
+                                           bool Allow2D, bool UseRealIdx,
                                            unsigned InputNumElements,
                                            AlignmentInfo *AI) {
   Alignment Align;
@@ -310,8 +310,8 @@ unsigned genx::getLegalRegionSizeForTarget(const GenXSubtarget &ST,
     if (AI)
       Align = AI->get(R.Indirect);
   }
-  return getLegalRegionSizeForTarget(ST, R, Idx, Allow2D, InputNumElements,
-                                     Align);
+  return getLegalRegionSizeForTarget(ST, R, Idx, Allow2D, UseRealIdx,
+                                     InputNumElements, Align);
 }
 
 /***********************************************************************
@@ -333,7 +333,7 @@ unsigned genx::getLegalRegionSizeForTarget(const GenXSubtarget &ST,
  */
 unsigned genx::getLegalRegionSizeForTarget(const GenXSubtarget &ST,
                                            const Region &R, unsigned Idx,
-                                           bool Allow2D,
+                                           bool Allow2D, bool UseRealIdx,
                                            unsigned InputNumElements,
                                            Alignment Align) {
   // Determine the max valid width.
@@ -380,7 +380,9 @@ unsigned genx::getLegalRegionSizeForTarget(const GenXSubtarget &ST,
       // crossing rules.
       unsigned ElementsPerGRF = GRFByteSize / R.ElementBytes;
       unsigned ElementsToBoundary = 1;
-      unsigned RealIdx = Idx / R.Width * R.VStride + Idx % R.Width * R.Stride;
+      unsigned RealIdx =
+          UseRealIdx ? Idx / R.Width * R.VStride + Idx % R.Width * R.Stride
+                     : Idx;
       if (!R.Indirect) {
         unsigned OffsetElements = R.getOffsetInElements();
         // For a direct operand, just use the constant offset of the
@@ -389,7 +391,8 @@ unsigned genx::getLegalRegionSizeForTarget(const GenXSubtarget &ST,
         // boundary.
         unsigned NumGRF = 2;
         // For PVC it's legal to read only from one GFR in byte source
-        if (ST.isPVC() && R.ElementBytes == genx::ByteBytes)
+        if (!ST.hasMultiIndirectByteRegioning() &&
+            R.ElementBytes == genx::ByteBytes)
           NumGRF = 1;
         ElementsToBoundary = (NumGRF * ElementsPerGRF) -
                              ((RealIdx + OffsetElements) % ElementsPerGRF);
@@ -527,11 +530,11 @@ unsigned genx::getLegalRegionSizeForTarget(const GenXSubtarget &ST,
 
     }
   }
-  if (AdjustValidWidthForTarget)
-    if (ST.isPVC() && R.is2D()) {
-      while ((ValidWidth * R.ElementBytes) >= 64)
-        ValidWidth /= 2;
-    }
+  if (AdjustValidWidthForTarget && !ST.hasMultiIndirectByteRegioning() &&
+      R.is2D()) {
+    while ((ValidWidth * R.ElementBytes) >= ST.getGRFByteSize())
+      ValidWidth /= 2;
+  }
 
   // Some targets do not have multi indirect byte regioning and in general case
   // transformation from multi indirect region to indirect is possible for
@@ -539,6 +542,17 @@ unsigned genx::getLegalRegionSizeForTarget(const GenXSubtarget &ST,
   if (R.isMultiIndirect() && R.ElementBytes == genx::ByteBytes &&
       !ST.hasMultiIndirectByteRegioning())
     ValidWidth = 1;
+
+  // Each source region with indirect addressing can potentially cross GRF
+  // boundary with a row. 1D regions are handled in CISABuilder but 2Ds have to
+  // be splitted into 1Ds. To force the splits reported valid width is reduced
+  // to the biggest power of 2 less than region width. Multi indirect
+  // regions which could cross GRF boundary are already reported with valid
+  // width of 1 so they don't have to be handled here
+  if (R.Indirect && !R.isMultiIndirect() && R.is2D()) {
+    ValidWidth = std::min(ValidWidth, 1U << genx::log2(R.Width - Idx % R.Width));
+  }
+
   return ValidWidth;
 }
 
