@@ -142,6 +142,34 @@ bool GenXVisaRegAlloc::runOnFunctionGroup(FunctionGroup &FGArg)
   PredefinedRegs.push_back(&RegStorage.back());
   RegStorage.emplace_back(
       vc::RegCategory::General, PreDefined_Vars::PREDEFINED_SR0,
+      IGCLLVM::FixedVectorType::get(Type::getInt32Ty(FGArg.getContext()), 4));
+  PredefinedRegs.push_back(&RegStorage.back());
+  RegStorage.emplace_back(
+      vc::RegCategory::General, PreDefined_Vars::PREDEFINED_CR0,
+      IGCLLVM::FixedVectorType::get(Type::getInt32Ty(FGArg.getContext()), 4));
+  PredefinedRegs.push_back(&RegStorage.back());
+  RegStorage.emplace_back(
+      vc::RegCategory::General, PreDefined_Vars::PREDEFINED_CE0,
+      IGCLLVM::FixedVectorType::get(Type::getInt32Ty(FGArg.getContext()), 1));
+  PredefinedRegs.push_back(&RegStorage.back());
+  RegStorage.emplace_back(
+      vc::RegCategory::General, PreDefined_Vars::PREDEFINED_DBG,
+      IGCLLVM::FixedVectorType::get(Type::getInt32Ty(FGArg.getContext()), 1));
+  PredefinedRegs.push_back(&RegStorage.back());
+  RegStorage.emplace_back(
+      vc::RegCategory::General, PreDefined_Vars::PREDEFINED_COLOR,
+      IGCLLVM::FixedVectorType::get(Type::getInt32Ty(FGArg.getContext()), 1));
+  PredefinedRegs.push_back(&RegStorage.back());
+  RegStorage.emplace_back(
+      vc::RegCategory::General, PreDefined_Vars::PREDEFINED_IMPL_ARG_BUF_PTR,
+      IGCLLVM::FixedVectorType::get(Type::getInt32Ty(FGArg.getContext()), 1));
+  PredefinedRegs.push_back(&RegStorage.back());
+  RegStorage.emplace_back(
+      vc::RegCategory::General, PreDefined_Vars::PREDEFINED_LOCAL_ID_BUF_PTR,
+      IGCLLVM::FixedVectorType::get(Type::getInt32Ty(FGArg.getContext()), 1));
+  PredefinedRegs.push_back(&RegStorage.back());
+  RegStorage.emplace_back(
+      vc::RegCategory::General, PreDefined_Vars::PREDEFINED_MSG0,
       IGCLLVM::FixedVectorType::get(Type::getInt32Ty(FGArg.getContext()), 1));
   PredefinedRegs.push_back(&RegStorage.back());
 
@@ -444,7 +472,7 @@ static LiveRange *coalesceConstState(Instruction &ToCoalesce,
   IGC_ASSERT_MESSAGE(LR->Category == vc::RegCategory::Surface ||
                          LR->Category == vc::RegCategory::Sampler,
                      "wrong argument: ToCoalesce should have surface category");
-  auto IID = GenXIntrinsic::getGenXIntrinsicID(&ToCoalesce);
+  auto IID = vc::getAnyIntrinsicID(&ToCoalesce);
   if (IID != GenXIntrinsic::genx_convert &&
       IID != GenXIntrinsic::genx_constanti)
     return CommonLR;
@@ -543,7 +571,7 @@ void GenXVisaRegAlloc::extraCoalescing()
         //
         auto skipTwoAddrCoalesce = [](Instruction *Inst) {
           unsigned IntrinsicID = vc::getAnyIntrinsicID(Inst);
-          if (IntrinsicID == GenXIntrinsic::not_any_intrinsic)
+          if (!vc::isAnyNonTrivialIntrinsic(IntrinsicID))
             return false;
           GenXIntrinsicInfo Info(IntrinsicID);
           for (auto AI : Info.getInstDesc()) {
@@ -580,7 +608,7 @@ void GenXVisaRegAlloc::extraCoalescing()
             continue;
           if (Operand->getType() != Inst->getType())
             continue;
-          if (isRestrictedByVisa(GenXIntrinsic::getGenXIntrinsicID(Inst), oi))
+          if (isRestrictedByVisa(vc::getAnyIntrinsicID(Inst), oi))
             continue;
           auto OperandLR = Liveness->getLiveRangeOrNull(Operand);
           if (!OperandLR || OperandLR->Category != vc::RegCategory::General)
