@@ -442,6 +442,9 @@ namespace IGC
         bool DisableEUFusion                            = false;
         bool DisableFDivToFMulInvOpt                    = false;
         bool initializePhiSampleSourceWA                = false;
+        bool WaDisableSubspanUseNoMaskForCB             = false;
+        
+        unsigned FastestS1Options                       = 0;  // FCEXP_NO_EXPRIMENT. Can't access the enum here for some reason.
     };
 
     enum class ThreadIDLayout
@@ -694,6 +697,9 @@ namespace IGC
         //when true, compiler enables MatchMad optimization for VS
         bool allowMatchMadOptimizationforVS = false;
 
+        //when true, compiler disables MatchMad optimization for CS
+        bool disableMatchMadOptimizationForCS = false;
+
         bool disableMemOptforNegativeOffsetLoads = false;
 
         bool enableThreeWayLoadSpiltOpt = false;
