@@ -819,7 +819,7 @@ static void setImpliedIGCKeys()
     IGC_SET_IMPLIED_REGKEY(ShaderDumpEnableAll, 1, ShaderDumpEnable, true);
     IGC_SET_IMPLIED_REGKEY(ShaderDumpEnableAll, 1, EnableVISASlowpath, true);
     IGC_SET_IMPLIED_REGKEY(ShaderDumpEnableAll, 1, EnableVISADumpCommonISA, true);
-
+    IGC_SET_IMPLIED_REGKEY_ANY(GenerateOptionsFile, ShaderDumpEnable, true);
     IGC_SET_IMPLIED_REGKEY(ShaderDumpEnable, 1, DumpLLVMIR, true);
     IGC_SET_IMPLIED_REGKEY(ShaderDumpEnable, 1, EnableCosDump, true);
     IGC_SET_IMPLIED_REGKEY(ShaderDumpEnable, 1, DumpOCLProgramInfo, true);
@@ -846,6 +846,8 @@ static void setImpliedIGCKeys()
     IGC_SET_IMPLIED_REGKEY(ForceOCLSIMDWidth, 16, EnableOCLSIMD16, true);
     IGC_SET_IMPLIED_REGKEY(ForceOCLSIMDWidth,  8, EnableOCLSIMD32, false);
     IGC_SET_IMPLIED_REGKEY(ForceOCLSIMDWidth,  8, EnableOCLSIMD16, false);
+
+    IGC_SET_IMPLIED_REGKEY(StackOverflowDetection, 1, ForceAddingStackcallKernelPrerequisites, true);
 }
 
 void setImpliedRegkey(SRegKeyVariableMetaData& name,
