@@ -74,8 +74,7 @@ public:
                                    const char *functionName) override;
   VISA_BUILDER_API int AddPayloadSection(VISAFunction *&function,
                                          const char *functionName) override;
-  VISA_BUILDER_API int Compile(const char *isaFileNameint,
-                               std::ostream *os = nullptr,
+  VISA_BUILDER_API int Compile(const char *isaasmFileName,
                                bool emit_visa_only = false) override;
 
   VISA_BUILDER_API void SetOption(vISAOptions option, bool val) override {
@@ -488,8 +487,9 @@ public:
 
   bool CISA_create_ifcall_instruction(VISA_opnd *pred_opnd,
                                       VISA_EMask_Ctrl emask, unsigned exec_size,
-                                      VISA_opnd *funcAddr, unsigned arg_size,
-                                      unsigned return_size, int lineNum);
+                                      bool isUniform, VISA_opnd *funcAddr,
+                                      unsigned arg_size, unsigned return_size,
+                                      int lineNum);
 
   bool CISA_create_faddr_instruction(const char *sym_name, VISA_opnd *dst,
                                      int lineNum);
