@@ -87,14 +87,16 @@ class BinInst;
 class G4_FCALL {
   uint16_t argSize;
   uint16_t retSize;
+  bool uniform = false;
 
 public:
   G4_FCALL() = delete;
-  G4_FCALL(uint16_t argVarSz, uint16_t retVarSz)
-      : argSize(argVarSz), retSize(retVarSz) {}
+  G4_FCALL(uint16_t argVarSz, uint16_t retVarSz, bool isUniform)
+      : argSize(argVarSz), retSize(retVarSz), uniform(isUniform) {}
 
   uint16_t getArgSize() const { return argSize; }
   uint16_t getRetSize() const { return retSize; }
+  bool isUniform() const { return uniform; }
 };
 
 // Forward references for classes used by G4_INST.
@@ -151,6 +153,7 @@ protected:
   bool doPostRA : 1; // for NoMaskWA
   bool canBeAcc : 1; // The inst can be ACC, including the inst's dst
                      // and the use operands in the DU chain.
+  bool doNotDelete : 1;
   G4_ExecSize execSize;
 
   // make it private so only the IR_Builder can create new instructions
@@ -282,6 +285,13 @@ public:
 
   void setPredicate(G4_Predicate *p);
   G4_Predicate *getPredicate() const { return predicate; }
+  const G4_VarBase *getPredicateBase() const {
+    return predicate ? predicate->getBase() : nullptr;
+  }
+  G4_VarBase *getPredicateBase() {
+    return const_cast<G4_VarBase *>(((const G4_INST *)this)
+        ->getPredicateBase());
+  }
 
   void setSaturate(G4_Sat s) { sat = (s == g4::SAT); }
   void setSaturate(bool z) { sat = z; }
@@ -815,7 +825,10 @@ public:
 
   bool canInstBeAcc(GlobalOpndHashTable *ght);
 
-  bool canInstBeAcc() const { return canBeAcc; };
+  bool canInstBeAcc() const { return canBeAcc; }
+
+  virtual bool isDoNotDelete() const { return doNotDelete; }
+  void markDoNotDelete() { doNotDelete = true; }
 
   bool canSrcBeFlagForPropagation(Gen4_Operand_Number opndNum) const;
 
@@ -1387,6 +1400,12 @@ public:
     return G4_Intrinsics[(int)intrinsicId].flags &
            (1ull << IntrinsicFlags::HasSideEffects);
   }
+
+  bool isDoNotDelete() const override {
+    // Return true if hasSideEffects is true. Otherwise, check if doNotDelete
+    // is marked or not.
+    return hasSideEffects() ? true : doNotDelete;
+  }
 };
 // place for holding all physical register operands
 //
@@ -1535,7 +1554,7 @@ inline bool G4_INST::isRelocationMov() const {
 
 inline const char *G4_INST::getLabelStr() const {
   vISA_ASSERT(srcs[0] && srcs[0]->isLabel(), ERROR_UNKNOWN);
-  return srcs[0]->asLabel()->getLabel();
+  return srcs[0]->asLabel()->getLabelName();
 }
 
 inline bool G4_InstCF::isUniformGoto(unsigned KernelSimdSize) const {
@@ -1554,12 +1573,12 @@ inline bool G4_InstCF::isIndirectJmp() const {
 
 inline const char *G4_InstCF::getJipLabelStr() const {
   vISA_ASSERT(jip != NULL && jip->isLabel(), ERROR_UNKNOWN);
-  return jip->asLabel()->getLabel();
+  return jip->asLabel()->getLabelName();
 }
 
 inline const char *G4_InstCF::getUipLabelStr() const {
   vISA_ASSERT(uip != NULL && uip->isLabel(), ERROR_UNKNOWN);
-  return uip->asLabel()->getLabel();
+  return uip->asLabel()->getLabelName();
 }
 
 inline bool G4_InstCF::isIndirectCall() const {
