@@ -475,7 +475,7 @@ G4_Kernel::G4_Kernel(const PlatformInfo &pInfo, INST_LIST_NODE_ALLOCATOR &alloc,
   } else {
     gtPinInfo = nullptr;
   }
-
+  regSharingHeuristics = m_options->getOption(vISA_RegSharingHeuristics);
   // NoMask WA
   m_EUFusionNoMaskWAInfo = nullptr;
 
@@ -623,10 +623,8 @@ bool G4_Kernel::updateKernelToLargerGRF() {
   if (numRegTotal == grfMode.getMaxGRF())
     return false;
 
-  numRegTotal = grfMode.getLargerGRF();
-
   // Scale number of GRFs, Acc, SWSB tokens.
-  setKernelParameters();
+  setKernelParameters(grfMode.getLargerGRF());
   fg.builder->rebuildPhyRegPool(getNumRegTotal());
   return true;
 }
@@ -638,10 +636,8 @@ bool G4_Kernel::updateKernelToSmallerGRF() {
   if (numRegTotal == grfMode.getMinGRF())
     return false;
 
-  numRegTotal = grfMode.getSmallerGRF();
-
   // Scale number of GRFs, Acc, SWSB tokens.
-  setKernelParameters();
+  setKernelParameters(grfMode.getSmallerGRF());
   fg.builder->rebuildPhyRegPool(getNumRegTotal());
   return true;
 }
@@ -656,15 +652,31 @@ void G4_Kernel::updateKernelByRegPressure(unsigned regPressure) {
   if (newGRF == numRegTotal)
     return;
 
-  numRegTotal = newGRF;
-
   // Scale number of threads, Acc, SWSB tokens.
-  setKernelParameters();
+  setKernelParameters(newGRF);
 
   // Update physical register pool
   fg.builder->rebuildPhyRegPool(getNumRegTotal());
 }
 
+//
+// Updates kernel's related structures based on NumGRF attribute
+//
+bool G4_Kernel::updateKernelFromNumGRFAttr() {
+  unsigned attrNumGRF =
+      m_kernelAttrs->getInt32KernelAttr(Attributes::ATTR_NumGRF);
+  if (!grfMode.isValidNumGRFs(attrNumGRF))
+    return false;
+  if (numRegTotal == attrNumGRF)
+    return true;
+
+  regSharingHeuristics = false;
+  // Scale number of GRFs, Acc, SWSB tokens.
+  setKernelParameters(attrNumGRF);
+  fg.builder->rebuildPhyRegPool(getNumRegTotal());
+  return true;
+}
+
 //
 // Evaluate AddrExp/AddrExpList to Imm
 //
@@ -831,12 +843,15 @@ unsigned StackCallABI::getNumCalleeSaveRegs() const {
 uint32_t StackCallABI::numReservedABIGRF() const {
   if (version == StackCallABIVersion::VER_1)
     return 3;
-  else {
-    // for ABI version > 1,
+  else if (version == StackCallABIVersion::VER_2) {
     if (kernel->getOption(vISA_PreserveR0InR0))
       return 2;
     return 3;
   }
+  else {
+    // for ABI version > 2
+    return 1;
+  }
 }
 
 uint32_t StackCallABI::getFPSPGRF() const {
@@ -850,11 +865,14 @@ uint32_t StackCallABI::getFPSPGRF() const {
 
 uint32_t StackCallABI::getSpillHeaderGRF() const {
   // For ABI V1 return r126.
-  // For ABI V2, V3 return r126.
+  // For ABI V2 return r126.
+  // For ABI V3 return r127.
   if (version == StackCallABIVersion::VER_1)
     return getStackCallStartReg() + SpillHeaderGRF;
-  else
+  else if (version == StackCallABIVersion::VER_2)
     return (kernel->getNumRegTotal() - 1) - SpillHeaderGRF;
+  else
+    return kernel->stackCall.getFPSPGRF();
 }
 
 uint32_t StackCallABI::getThreadHeaderGRF() const {
@@ -931,7 +949,7 @@ unsigned G4_Kernel::getLargestInputRegister() {
   return regNum;
 }
 
-void G4_Kernel::setKernelParameters() {
+void G4_Kernel::setKernelParameters(unsigned newGRF) {
   unsigned overrideGRFNum = 0, overrideNumThreads = 0, overrideNumSWSB = 0,
            overrideNumAcc = 0;
 
@@ -939,27 +957,36 @@ void G4_Kernel::setKernelParameters() {
   overrideNumThreads = m_options->getuInt32Option(vISA_HWThreadNumberPerEU);
   overrideNumSWSB = m_options->getuInt32Option(vISA_SWSBTokenNum);
   overrideNumAcc = m_options->getuInt32Option(vISA_numGeneralAcc);
-  regSharingHeuristics = m_options->getOption(vISA_RegSharingHeuristics);
 
   //
   // Number of threads/GRF can currently be set by:
-  // 1.- IGC flag (reg key)
-  // 2.- Compiler option entered by user for
+  // 1.- Per kernel attribute
+  // 2.- IGC flag (reg key)
+  // 3.- Compiler option entered by user for
   //      2.1 entire module
   //      2.2 kernel function
-  // 3.- Compiler heuristics
+  // 4.- Compiler heuristics
+  //
+  // 1 is set via kernel attribute. 2 and 3 via vISA option.
+  // If none of them are set, compiler selects the best option (4).
   //
 
-  if (overrideNumThreads > 0) {
+  if (newGRF > 0) {
+    // per kernel attribute or GRF change during compilation
+    grfMode.setModeByNumGRFs(newGRF);
+    overrideGRFNum = 0;
+  } else if (overrideNumThreads > 0) {
+    // Forcing a specific number of threads
     grfMode.setModeByNumThreads(overrideNumThreads);
     overrideGRFNum = 0;
+    regSharingHeuristics = false;
   } else if (overrideGRFNum != grfMode.getDefaultGRF()) {
+    // Forcing a specific number of GRFs
     grfMode.setModeByNumGRFs(overrideGRFNum);
-  } else if (regSharingHeuristics) {
-    // GRFMode is set to default mode when kernel is created
-    // During compilation GRFMode may change by updating numRegTotal
-    if (numRegTotal > 0)
-      grfMode.setModeByNumGRFs(numRegTotal);
+    regSharingHeuristics = false;
+  } else {
+    // Use default value
+    grfMode.setDefaultGRF();
     overrideGRFNum = 0;
   }
 
@@ -1363,7 +1390,10 @@ void G4_Kernel::dumpG4Internal(const std::string &file) {
 }
 
 void G4_Kernel::dumpG4InternalTo(std::ostream &os) {
-  os << ".kernel " << name << "\n";
+  if (name)
+    os << ".kernel " << name << "\n";
+  else
+    os << ".kernel\n";
 
   for (const G4_Declare *d : Declares) {
     static const int MIN_DECL = 34; // skip the built-in decls
