@@ -92,7 +92,7 @@ public:
   void setGTPinInitFromL0(bool val) { gtpinInitFromL0 = val; }
   bool isGTPinInitFromL0() const { return gtpinInitFromL0; }
 
-  void addIndirRef(G4_Declare *addr, G4_Declare *target) {
+  void addIndirRef(const G4_Declare *addr, G4_Declare *target) {
     indirRefs[addr].push_back(target);
   }
 
@@ -109,7 +109,7 @@ private:
   gtpin::igc::igc_init_t *gtpin_init = nullptr;
 
   // Store Addr Var -> Array of targets
-  std::unordered_map<G4_Declare *, std::vector<G4_Declare *>> indirRefs;
+  std::unordered_map<const G4_Declare *, std::vector<G4_Declare *>> indirRefs;
 }; // class gtPinData
 
 class G4_BB;
@@ -140,6 +140,12 @@ public:
       }
     }
   }
+  bool isValidNumGRFs(unsigned grfs) const {
+    auto iter =
+        std::find_if(configs.begin(), configs.end(),
+                     [grfs](const Config &c) { return c.numGRF == grfs; });
+    return iter != configs.end();
+  }
 
   unsigned findModeByRegPressure(unsigned maxRP, unsigned largestInputReg);
 
@@ -149,6 +155,7 @@ public:
     return configs[configs.size() - 1].numGRF;
   }
   unsigned getDefaultGRF() const { return configs[defaultMode].numGRF; }
+  void setDefaultGRF() { currentMode = defaultMode; }
   unsigned getLargerGRF() const {
     return currentMode + 1 < configs.size() ? configs[currentMode + 1].numGRF
                                             : configs[currentMode].numGRF;
@@ -277,7 +284,7 @@ private:
   // Class contains sub-regs for chosen ABI version.
   // Values are initialized by ctor. For eg,
   // As per ABI v1, v2 BE_FP is in sub-reg 3 whereas
-  // as per ABI v3, BE_FP is in sub-reg 6.
+  // as per ABI v3, BE_FP is in sub-reg 0.
   struct SubRegs_Stackcall {
     unsigned int Ret_IP = 0;
     unsigned int Ret_EM = 0;
@@ -323,10 +330,10 @@ private:
   static constexpr unsigned int SubRegs_Stackcall_v1_v2_FE_SP = 3; // :uq
 
   // For VISA ABI v3
-  static constexpr unsigned int SubRegs_Stackcall_v3_Ret_IP = 0; // :uq
-  static constexpr unsigned int SubRegs_Stackcall_v3_Ret_EM = 2; // :ud
-  static constexpr unsigned int SubRegs_Stackcall_v3_BE_SP = 4; // :ud
-  static constexpr unsigned int SubRegs_Stackcall_v3_BE_FP = 6; // :ud
+  static constexpr unsigned int SubRegs_Stackcall_v3_BE_FP = 0;  // :ud
+  static constexpr unsigned int SubRegs_Stackcall_v3_BE_SP = 2;  // :ud
+  static constexpr unsigned int SubRegs_Stackcall_v3_Ret_IP = 4; // :ud
+  static constexpr unsigned int SubRegs_Stackcall_v3_Ret_EM = 6; // :ud
   static constexpr unsigned int SubRegs_Stackcall_v3_FE_FP = 4; // :uq
   static constexpr unsigned int SubRegs_Stackcall_v3_FE_SP = 5; // :uq
 
@@ -358,7 +365,7 @@ private:
   static constexpr unsigned int FrameDescriptorOfsets_v3_FE_SP =
       SubRegs_Stackcall_v3_FE_SP * 8;
   static constexpr unsigned int FrameDescriptorOfsets_v3_Ret_IP =
-      SubRegs_Stackcall_v3_Ret_IP * 8;
+      SubRegs_Stackcall_v3_Ret_IP * 4;
   static constexpr unsigned int FrameDescriptorOfsets_v3_Ret_EM =
       SubRegs_Stackcall_v3_Ret_EM * 4;
   static constexpr unsigned int FrameDescriptorOfsets_v3_BE_FP =
@@ -456,8 +463,6 @@ private:
 
   bool m_hasIndirectCall = false;
 
-  GRFMode grfMode;
-
   // map key is filename string with complete path.
   // if first elem of pair is false, the file wasn't found.
   // the second elem of pair stores the actual source line stream
@@ -524,6 +529,7 @@ public:
   unsigned char minor_version;
 
   StackCallABI stackCall;
+  GRFMode grfMode;
 
   G4_Kernel(const PlatformInfo &pInfo, INST_LIST_NODE_ALLOCATOR &alloc,
             Mem_Manager &m, Options *options, Attributes *anAttr,
@@ -632,6 +638,7 @@ public:
   bool updateKernelToLargerGRF();
   bool updateKernelToSmallerGRF();
   void updateKernelByRegPressure(unsigned regPressure);
+  bool updateKernelFromNumGRFAttr();
 
   void evalAddrExp();
 
@@ -762,7 +769,7 @@ private:
   G4_BB *getNextBB(G4_BB *bb) const;
   unsigned getBinOffsetOfBB(G4_BB *bb) const;
 
-  void setKernelParameters();
+  void setKernelParameters(unsigned newGRF = 0);
 
   void dumpDotFileInternal(const std::string &baseName);
   void dumpG4Internal(const std::string &baseName);
