@@ -19,6 +19,7 @@ SPDX-License-Identifier: MIT
 // clang-format off
 #include "common/LLVMWarningsPush.hpp"
 #include "llvm/Support/Allocator.h"
+#include "llvm/ADT/DenseSet.h"
 #include "common/LLVMWarningsPop.hpp"
 // clang-format on
 
@@ -87,11 +88,18 @@ public:
   BankConflictPass(GlobalRA &g, bool global) : gra(g), forGlobal(global) {}
 };
 
-// The forbidden kind for the forbidden bit of each register files.
-// Note that:
-// a) There is no forbidden regsiter for address and flag regsiters.
-// We keep them just in case.
-// b) All the forbidden kinds from EOT to RESERVEGRF are for GRF
+// A LiveRange's Register Class (RC) is expressed as a bitset of
+// forbidden physical registers that it cannot be assigned to.
+// -- Address and Flag live ranges currently all belong to same RC
+// (FBD_ADDR/FBD_FLAG) as there are no restrictions on their assignment.
+// -- The rest of RCs are for GRF live ranges. By default such live ranges have
+// FBD_RESERVEDGRF, which reserves a number of GRFs such as r0/r1/RN-1 based on
+// kernel type and user options.
+// -- The other GRF RCs (e.g., FBD_EOT) reserves additional GRFs on top of
+// FDB_RESERVEDGRF base on their usage in the kernel.
+// -- RCs must be pre-defined, dynamic creation of new RCs is not supported. If
+// your live range belongs to more than one RC, you should add a new RC and
+// initialize it in generateForbiddenTemplates()
 enum class forbiddenKind {
   FBD_ADDR = 0,
   FBD_FLAG = 1,
@@ -101,9 +109,32 @@ enum class forbiddenKind {
   FBD_EOTLASTGRF,
   FBD_CALLERSAVE,
   FBD_CALLEESAVE,
-  FBD_NUM,
+  FBD_UNASSIGNED, // This should only ever exist in between rounds of
+                  // incremental RA.
 };
 
+constexpr bool isGRFRegClass(forbiddenKind kind) {
+  return kind == forbiddenKind::FBD_RESERVEDGRF ||
+         kind == forbiddenKind::FBD_EOT || kind == forbiddenKind::FBD_LASTGRF ||
+         kind == forbiddenKind::FBD_EOTLASTGRF ||
+         kind == forbiddenKind::FBD_CALLERSAVE ||
+         kind == forbiddenKind::FBD_CALLEESAVE;
+}
+
+// Return true if k1 is a sub RC of k2, i.e., all of k2's forbidden registers
+// are also forbidden in k1.
+constexpr bool isSubRegClass(forbiddenKind k1, forbiddenKind k2) {
+  if (k1 == k2)
+    return true;
+  if (k2 == forbiddenKind::FBD_UNASSIGNED)
+    return true;
+  if (k2 == forbiddenKind::FBD_RESERVEDGRF && isGRFRegClass(k1))
+    return true;
+  if (k1 == forbiddenKind::FBD_EOTLASTGRF)
+    return k2 == forbiddenKind::FBD_EOT || k2 == forbiddenKind::FBD_LASTGRF;
+  return false;
+}
+
 enum class AugmentationMasks {
   Undetermined = 0,
   Default16Bit = 1,
@@ -117,7 +148,7 @@ class LiveRange final {
   G4_RegVar *const var;
   G4_Declare *const dcl;
   const G4_RegFileKind regKind;
-  forbiddenKind forbiddenType;
+  forbiddenKind forbiddenType = forbiddenKind::FBD_UNASSIGNED;
   BitSet *forbidden = nullptr;
   bool spilled = false;
   bool isUnconstrained = false;
@@ -234,7 +265,7 @@ public:
 public:
   void setForbidden(forbiddenKind f);
   void markForbidden(vISA::Mem_Manager &GCMem, int reg, int numReg);
-  BitSet *getForbidden();
+  const BitSet *getForbidden();
   int getNumForbidden();
   G4_RegVar *getVar() const { return var; }
   G4_Declare *getDcl() const { return dcl; }
@@ -261,7 +292,7 @@ public:
 
   void resetForbidden() {
     forbidden = nullptr;
-    forbiddenType = forbiddenKind::FBD_NUM;
+    forbiddenType = forbiddenKind::FBD_UNASSIGNED;
   }
 
 private:
@@ -305,7 +336,7 @@ namespace vISA {
 class Augmentation {
 private:
   // pair of default mask, non-default mask
-  using MaskDeclares = std::pair<llvm_SBitVector, llvm_SBitVector>;
+  using MaskDeclares = std::pair<llvm::DenseSet<unsigned int>, llvm::DenseSet<unsigned int>>;
   G4_Kernel &kernel;
   Interference &intf;
   GlobalRA &gra;
@@ -321,6 +352,7 @@ private:
   // all call sites of func.
   std::unordered_map<FuncInfo *, MaskDeclares> overlapDclsWithFunc;
   std::unordered_map<G4_Declare *, MaskDeclares> retDeclares;
+  bool useGenericAugAlign = false;
 
   bool updateDstMaskForGather(G4_INST *inst, std::vector<unsigned char> &mask);
   bool updateDstMaskForGatherRaw(G4_INST *inst,
@@ -359,24 +391,24 @@ private:
   bool isCompatible(const G4_Declare *testDcl,
                     const G4_Declare *biggerDcl) const;
   void buildInterferenceIncompatibleMask();
-  void buildInteferenceForCallSiteOrRetDeclare(G4_Declare *newDcl,
+  void buildInteferenceForCallSiteOrRetDeclare(std::vector<G4_Declare*>& dcls,
                                                MaskDeclares *mask);
   void buildInteferenceForCallsite(FuncInfo *func);
   void buildInteferenceForRetDeclares();
   void buildSummaryForCallees();
   void expireIntervals(unsigned startIdx);
-  void buildSIMDIntfDcl(G4_Declare *newDcl, bool isCall);
-  void buildSIMDIntfAll(G4_Declare *newDcl);
+  void buildSIMDIntfDcl(G4_Declare *newDcl);
+  void storeOverlapWithCallRet(G4_Declare *newDcl,
+                               const std::vector<bool> &globalVars);
   void handleSIMDIntf(G4_Declare *firstDcl, G4_Declare *secondDcl, bool isCall);
   bool weakEdgeNeeded(AugmentationMasks, AugmentationMasks);
-
-  void addSIMDIntfDclForCallSite(G4_BB *callBB);
-
-  void addSIMDIntfForRetDclares(G4_Declare *newDcl);
+  void addSIMDIntfDclForCallSite(G4_BB *callBB,
+                                 const std::vector<bool> &globalVars);
+  void addSIMDIntfForRetDclares(G4_Declare *newDcl,
+                                const std::vector<bool> &globalVars);
 
 public:
-  Augmentation(G4_Kernel &k, Interference &i, const LivenessAnalysis &l,
-               const LiveRangeVec &ranges, GlobalRA &g);
+  Augmentation(Interference &i, const LivenessAnalysis &l, GlobalRA &g);
   ~Augmentation();
 
   void augmentIntfGraph();
@@ -386,35 +418,76 @@ public:
   }
 };
 
+// This class stores base matrices used for interference building for graph coloring.
+struct InterferenceMatrixStorage {
+  // This member is a half triangle representation of interference graph implemented
+  // as a sparse bitvector. Interference construction uses this member. When
+  // incremental RA is enabled, this member is updated incrementally.
+  std::vector<llvm_SBitVector> sparseMatrix;
+  // This member is constructed after SIMT and SIMD interference are computed.
+  // It's a full triangle representation of interference matrix with trivial
+  // traversal. This member is reconstructed in each graph color iteration.
+  std::vector<std::vector<unsigned>> sparseIntf;
+};
+
 // This class contains implementation of various methods to implement
 // incremental intf computation. Instance of this class is created
 // once and stored in GlobalRA. This class should therefore not
 // hold pointer to GraphColor/Interference or such other short-living
 // instances.
 class IncrementalRA {
+  friend Interference;
+
+  const llvm_SBitVector &getSparseMatrix(unsigned int id) {
+    return sparseMatrix[id];
+  }
+
 private:
   GlobalRA &gra;
   G4_Kernel &kernel;
   LiveRangeVec lrs;
+  std::vector<llvm_SBitVector>& sparseMatrix;
+  std::vector<std::vector<unsigned>>& sparseIntf;
   G4_RegFileKind selectedRF = G4_RegFileKind::G4_UndefinedRF;
   unsigned int level = 0;
   std::unordered_set<G4_Declare *> needIntfUpdate;
+  std::unordered_set<G4_Declare *> evenAlignCache;
   unsigned int maxDclId = 0;
   // Map of root G4_Declare* -> id assigned to its G4_RegVar
   // This allows us to reuse ids from previous iteration.
   std::unordered_map<G4_Declare *, unsigned int> varIdx;
   unsigned int maxVarIdx = 0;
 
+  // Only BBs present in this set have interference computation
+  // done for them. It's assumed other BBs have no change in
+  // liveness so no change in interference is expected.
+  std::unordered_set<const G4_BB *> updateIntfForBB;
+
+  bool updateIntfForBBValid = false;
+
   // Reset state to mark start of new type of GRA (eg, from flag to GRF)
   void reset();
 
+  // During incremental update we need to remove edges between each
+  // incremental intf candidate and their neighbor.
+  void resetEdges();
+
+  // Compute BBs needing incremental update in current iteration.
+  // List of BBs needing update is stored in updateIntfForBB set.
+  void collectBBs(const LivenessAnalysis *liveness);
+
+  // Special variables such as r0 that are live-out have their
+  // interference marked already. So these should never be treated as
+  // incremental RA candidates.
+  void eraseLiveOutsFromIncrementalUpdate();
+
 public:
   llvm::SpecificBumpPtrAllocator<LiveRange> mem;
 
   IncrementalRA(GlobalRA &g);
 
-  bool isEnabled() { return level > 0; }
-  bool isEnabledWithVerification() { return level == 2; }
+  bool isEnabled() const { return level > 0; }
+  bool isEnabledWithVerification() const { return level == 2; }
 
   static bool isEnabled(G4_Kernel &kernel) {
     // 0 - disabled
@@ -428,10 +501,20 @@ public:
   }
 
   void registerNextIter(G4_RegFileKind rf,
-                        const LivenessAnalysis *liveness = nullptr);
+                        const LivenessAnalysis *liveness,
+                        const Interference *intf);
   // After computing interference incrementally, GraphColor needs to clear
   // candidate list to prepare for new incremental RA temps.
-  void clearCandidates() { needIntfUpdate.clear(); }
+  void clearCandidates() {
+    needIntfUpdate.clear();
+    updateIntfForBB.clear();
+    updateIntfForBBValid = false;
+    for (auto* evenAlignCandidate : evenAlignCache) {
+      if (!evenAlignCandidate->getAliasDeclare())
+        needIntfUpdate.insert(evenAlignCandidate);
+    }
+    evenAlignCache.clear();
+  }
 
   LiveRangeVec &getLRs() { return lrs; }
 
@@ -485,9 +568,33 @@ public:
   void reduceMaxDclId(unsigned int reduceBy) {
     if (!level)
       return;
-    maxDclId -= reduceBy;
+    if (maxDclId > 0) {
+      vISA_ASSERT(maxDclId >= reduceBy, "error removing partial dcls");
+      maxDclId -= reduceBy;
+    }
   }
 
+  void resetPartialDcls();
+
+  bool intfNeededForBB(const G4_BB *bb) const {
+    // If incremental RA is not enabled then all BBs need intf update.
+    if (!isEnabled() || !updateIntfForBBValid)
+      return true;
+    // If incremental RA is enabled then check whether we need to run
+    // intf for BB.
+    return updateIntfForBB.count(bb) > 0;
+  }
+
+  bool hasAnyCandidates() const { return needIntfUpdate.size() > 0; }
+
+  bool intfNeededForVar(G4_Declare *dcl) const {
+    if (needIntfUpdate.size() == 0)
+      return true;
+    return needIntfUpdate.count(dcl) > 0;
+  }
+
+  void evenAlignUpdate(G4_Declare *dcl) { evenAlignCache.insert(dcl); }
+
 private:
   // For verification only
   std::vector<llvm_SBitVector> def_in;
@@ -536,14 +643,14 @@ class Interference {
   Augmentation aug;
   IncrementalRA &incRA;
 
-  std::vector<std::vector<unsigned>> sparseIntf;
+  std::vector<std::vector<unsigned>>& sparseIntf;
 
   // sparse interference matrix.
   // we don't directly update sparseIntf to ensure uniqueness
   // like dense matrix, interference is not symmetric (that is, if v1 and v2
   // interfere and v1 < v2, we insert (v1, v2) but not (v2, v1)) for better
   // cache behavior
-  std::vector<llvm_SBitVector> sparseMatrix;
+  std::vector<llvm_SBitVector>& sparseMatrix;
 
   unsigned int denseMatrixLimit = 0;
 
@@ -557,16 +664,6 @@ class Interference {
 
   G4_Declare *getGRFDclForHRA(int GRFNum) const;
 
-  bool useDenseMatrix() const {
-    // The size check is added to prevent offset overflow in
-    // generateSparseIntfGraph() and help avoid out-of-memory
-    // issue in dense matrix allocation.
-    unsigned long long size = static_cast<unsigned long long>(rowSize) *
-                              static_cast<unsigned long long>(maxId);
-    unsigned long long max = std::numeric_limits<unsigned int>::max();
-    return (maxId < denseMatrixLimit) && (size < max);
-  }
-
   // Only upper-half matrix is now used in intf graph.
   inline void safeSetInterference(unsigned v1, unsigned v2) {
     // Assume v1 < v2
@@ -649,8 +746,7 @@ class Interference {
   void setupLRs(G4_BB *bb);
 
 public:
-  Interference(const LivenessAnalysis *l, const LiveRangeVec &lr, unsigned n,
-               unsigned ns, unsigned nm, GlobalRA &g);
+  Interference(const LivenessAnalysis *l, GlobalRA &g);
 
   ~Interference() {
     if (useDenseMatrix()) {
@@ -658,6 +754,16 @@ public:
     }
   }
 
+  bool useDenseMatrix() const {
+    // The size check is added to prevent offset overflow in
+    // generateSparseIntfGraph() and help avoid out-of-memory
+    // issue in dense matrix allocation.
+    unsigned long long size = static_cast<unsigned long long>(rowSize) *
+                              static_cast<unsigned long long>(maxId);
+    unsigned long long max = std::numeric_limits<unsigned int>::max();
+    return (maxId < denseMatrixLimit) && (size < max);
+  }
+
   const std::vector<G4_Declare *> *
   getCompatibleSparseIntf(G4_Declare *d) const {
     if (compatibleSparseIntf.size() > 0) {
@@ -726,11 +832,8 @@ public:
 class GraphColor {
   GlobalRA &gra;
 
-  // This is not necessarily the same as the number of available physical GRFs,
-  // as failSafeRA will reserve some GRF.
-  // FIXME: failSafeRA should use RegisterClass/forbidden GRF to model this
-  // instead of directly changing the number of GRFs.
-  unsigned totalGRFRegCount;
+  // Same as GRF count in G4_Kernel.
+  const unsigned totalGRFRegCount;
   const unsigned numVar;
   // The original code has no comments whatsoever (sigh), but best as I can tell
   // this vector is used to track the active values held by each of A0's
@@ -779,6 +882,8 @@ class GraphColor {
   unsigned numColor = 0;
 
   bool failSafeIter = false;
+  // Reserved GRF count for fail-safe RA
+  unsigned reserveSpillGRFCount = 0;
 
   unsigned edgeWeightGRF(const LiveRange *lr1, const LiveRange *lr2);
   unsigned edgeWeightARF(const LiveRange *lr1, const LiveRange *lr2);
@@ -835,8 +940,7 @@ public:
   unsigned int getNumVars() const { return numVar; }
   float getSpillRatio() const { return (float)spilledLRs.size() / numVar; }
   void markFailSafeIter(bool f) { failSafeIter = f; }
-  void setTotalGRFRegCount(unsigned c) { totalGRFRegCount = c; }
-  unsigned getTotalGRFRegCount() { return totalGRFRegCount; }
+  void setReserveSpillGRFCount(unsigned c) { reserveSpillGRFCount = c; }
 };
 
 struct BundleConflict {
@@ -864,6 +968,7 @@ struct RAVarInfo {
   std::vector<BundleConflict> bundleConflicts;
   G4_SubReg_Align subAlign = G4_SubReg_Align::Any;
   bool isEvenAlign = false;
+  AugmentationMasks augMask = AugmentationMasks::Undetermined;
 };
 
 class VerifyAugmentation {
@@ -930,7 +1035,7 @@ class ForbiddenRegs {
 public:
   ForbiddenRegs(IR_Builder &b) : builder(b) {
     // Initialize forbidden bits
-    forbiddenVec.resize((size_t)forbiddenKind::FBD_NUM);
+    forbiddenVec.resize((size_t)forbiddenKind::FBD_UNASSIGNED);
     forbiddenVec[(size_t)forbiddenKind::FBD_ADDR].resize(
         getForbiddenVectorSize(G4_ADDRESS));
     forbiddenVec[(size_t)forbiddenKind::FBD_FLAG].resize(
@@ -1011,7 +1116,6 @@ private:
   void saveRestoreA0(G4_BB *);
   static const RAVarInfo defaultValues;
   std::vector<RAVarInfo> vars;
-  std::vector<AugmentationMasks> varMasks;
   std::vector<G4_Declare *> UndeclaredVars;
 
   // fake declares for each GRF reg, used by HRA
@@ -1132,13 +1236,14 @@ public:
   PointsToAnalysis &pointsToAnalysis;
   FCALL_RET_MAP fcallRetMap;
 
-  bool useLscForSpillFill = false;
-  bool useLscForScatterSpill = false;
-  bool useLscForNonStackCallSpillFill = false;
+  const bool useLscForSpillFill;
+  const bool useLscForScatterSpill;
+  const bool useLscForNonStackCallSpillFill;
   bool useFastRA = false;
   bool useHybridRAwithSpill = false;
   bool useLocalRA = false;
 
+  InterferenceMatrixStorage intfStorage;
   IncrementalRA incRA;
 
 
@@ -1334,18 +1439,11 @@ public:
   }
 
   AugmentationMasks getAugmentationMask(const G4_Declare *dcl) const {
-    auto dclid = dcl->getDeclId();
-    if (dclid >= varMasks.size()) {
-      return AugmentationMasks::Undetermined;
-    }
-    return varMasks[dclid];
+    return getVar(dcl).augMask;
   }
 
   void setAugmentationMask(const G4_Declare *dcl, AugmentationMasks m) {
-    auto dclid = dcl->getDeclId();
-    if (dclid >= varMasks.size())
-      varMasks.resize(dclid + 1);
-    varMasks[dclid] = m;
+    allocVar(dcl).augMask = m;
     if (dcl->getIsSplittedDcl()) {
       for (const G4_Declare *subDcl : getSubDclList(dcl)) {
         setAugmentationMask(subDcl, m);
@@ -1452,9 +1550,13 @@ public:
 
   GlobalRA(G4_Kernel &k, PhyRegPool &r, PointsToAnalysis &p2a)
       : kernel(k), builder(*k.fg.builder), regPool(r), pointsToAnalysis(p2a),
-        fbdRegs(*k.fg.builder), incRA(*this) {
+        fbdRegs(*k.fg.builder), incRA(*this),
+        useLscForSpillFill(k.fg.builder->supportsLSC()),
+        useLscForNonStackCallSpillFill(
+            k.fg.builder->useLscForNonStackSpillFill()),
+        useLscForScatterSpill(k.fg.builder->supportsLSC() &&
+                              k.fg.builder->getOption(vISA_scatterSpill)) {
     vars.resize(k.Declares.size());
-    varMasks.resize(k.Declares.size());
 
     if (kernel.getOptions()->getOption(vISA_VerifyAugmentation)) {
       verifyAugmentation = std::make_unique<VerifyAugmentation>();
