@@ -801,6 +801,8 @@ int LinearScanRA::linearScanRA() {
       kernel.fg.getHasStackCalls() || kernel.fg.getIsStackCallFunc();
   int globalScratchOffset =
       kernel.getInt32KernelAttr(Attributes::ATTR_SpillMemOffset);
+  // FXIME: This does not match GCRA's definition (it has an additional
+  // builder.suppportsLSC())
   bool useScratchMsgForSpill =
       !hasStackCall && (globalScratchOffset < (int)(SCRATCH_MSG_LIMIT * 0.6));
   bool enableSpillSpaceCompression =
@@ -922,10 +924,9 @@ int LinearScanRA::linearScanRA() {
         }
       }
 
-      SpillManagerGRF spillGRF(gra, nextSpillOffset, l.getNumSelectedVar(), &l,
-                               &spillLRs, enableSpillSpaceCompression,
-                               useScratchMsgForSpill,
-                               builder.avoidDstSrcOverlap(), {});
+      SpillManagerGRF spillGRF(gra, nextSpillOffset, &l, &spillLRs,
+                               enableSpillSpaceCompression,
+                               useScratchMsgForSpill);
 
       spillGRF.spillLiveRanges(&kernel);
       nextSpillOffset = spillGRF.getNextOffset();
@@ -1119,6 +1120,9 @@ void LinearScanRA::setDstReferences(
     lr = CreateLocalLiveRange(dcl);
   }
 
+  if (!lr)
+    return;
+
   if (dcl->isOutput() && !lr->isGRFRegAssigned()) {
     lr->setLastRef(curInst, lastInstLexID * 2 + 1);
     if (!lr->isPushedToIntervalList()) {
@@ -1128,8 +1132,7 @@ void LinearScanRA::setDstReferences(
     return;
   }
 
-  if (lr == nullptr ||
-      (dcl->getRegFile() == G4_INPUT &&
+  if ((dcl->getRegFile() == G4_INPUT &&
        dcl != kernel.fg.builder->getStackCallArg() &&
        dcl != kernel.fg.builder->getStackCallRet()) ||
       (lr->isGRFRegAssigned() && (!dcl->getRegVar()->isGreg()))) // ARF
@@ -1208,6 +1211,9 @@ void LinearScanRA::setSrcReferences(
     lr = CreateLocalLiveRange(dcl);
   }
 
+  if (!lr)
+    return;
+
   if (dcl->isOutput() && !lr->isGRFRegAssigned()) {
     lr->setLastRef(curInst, lastInstLexID * 2 + 1);
     if (!lr->isPushedToIntervalList()) {
@@ -1217,8 +1223,7 @@ void LinearScanRA::setSrcReferences(
     return;
   }
 
-  if (lr == nullptr ||
-      (dcl->getRegFile() == G4_INPUT &&
+  if ((dcl->getRegFile() == G4_INPUT &&
        dcl != kernel.fg.builder->getStackCallRet() &&
        dcl != kernel.fg.builder->getStackCallArg()) ||
       (lr->isGRFRegAssigned() && (!dcl->getRegVar()->isGreg()))) // ARF
