@@ -33,14 +33,17 @@ static const unsigned LARGE_BLOCK_SIZE = 20000;
 static const unsigned LARGE_BLOCK_SIZE_RPE = 32000;
 static const unsigned PRESSURE_REDUCTION_MIN_BENEFIT = 5; // percentage
 static const unsigned PRESSURE_REDUCTION_THRESHOLD = 110;
+static const unsigned PRESSURE_LATENCY_HIDING_THRESHOLD = 104;
 static const unsigned PRESSURE_HIGH_THRESHOLD = 128;
 static const unsigned PRESSURE_REDUCTION_THRESHOLD_SIMD32 = 120;
+static const unsigned EXTRA_REGISTERS_FOR_RA = 10; // percentage
 
 namespace {
 
 // Forward declaration.
 class preNode;
 struct RegisterPressure;
+template<G4_RegFileKind Kind> class AregUsage;
 
 using preNodeAlloc = llvm::SpecificBumpPtrAllocator<preNode>;
 
@@ -198,6 +201,7 @@ private:
   friend class SethiUllmanACCQueue;
   friend class SethiUllmanQueue;
   friend class LatencyQueue;
+  template <G4_RegFileKind Kind> friend class AregUsage;
 };
 
 // The dependency graph for a basic block.
@@ -363,12 +367,16 @@ struct RegisterPressure {
   }
 
   void recompute(G4_BB *BB) { rpe->runBB(BB); }
+  void recompute() { rpe->run(); }
 
   // Return the register pressure in GRF for an instruction.
   unsigned getPressure(G4_INST *Inst) const {
     return rpe->getRegisterPressure(Inst);
   }
 
+  // Return the max register pressure
+  unsigned getMaxRP() const { return rpe->getMaxRP(); }
+
   // Return the max pressure in GRFs for this block.
   unsigned getPressure(G4_BB *bb, std::vector<G4_INST *> *Insts = nullptr) {
     unsigned Max = 0;
@@ -490,9 +498,9 @@ public:
 
   // MaxPressure is the BB reg-pressure before and after scheduling
   // ReassignID of PreNodes when this is not 1st-round scheduling
-  // KernelRP is the measure max reg-pressure of this kernel before scheduling
+  // UpperBoundGRF is the measure max reg-pressure of this kernel before scheduling
   bool scheduleBlockForLatency(unsigned &MaxPressure, bool ReassignID,
-                               unsigned KernelRP);
+                               unsigned UpperBoundGRF);
 
 private:
   void SethiUllmanScheduling();
@@ -547,29 +555,28 @@ static unsigned getLatencyHidingThreshold(G4_Kernel &kernel, unsigned NumGrfs) {
   unsigned RPThreshold =
       kernel.getOptions()->getuInt32Option(vISA_preRA_ScheduleRPThreshold);
   if (RPThreshold == 0) {
-    RPThreshold = 104;
+    RPThreshold = PRESSURE_LATENCY_HIDING_THRESHOLD;
   }
   return unsigned(RPThreshold * (std::max(NumGrfs, 128u) - 32u) / 96u);
 }
 
-preRA_Scheduler::preRA_Scheduler(G4_Kernel &k, RPE *rpe)
-    : kernel(k), rpe(rpe), m_options(kernel.getOptions()) {}
+preRA_Scheduler::preRA_Scheduler(G4_Kernel &k) : kernel(k) {}
 
 preRA_Scheduler::~preRA_Scheduler() {}
 
-bool preRA_Scheduler::run() {
+bool preRA_Scheduler::run(unsigned &KernelPressure) {
   if (kernel.getInt32KernelAttr(Attributes::ATTR_Target) != VISA_3D) {
     // Do not run pre-RA scheduler for CM unless user forces it.
-    if (!m_options->getOption(vISA_preRA_ScheduleForce))
+    if (!kernel.getOption(vISA_preRA_ScheduleForce))
       return false;
   }
 
   unsigned Threshold = getRPReductionThreshold(kernel);
-  unsigned SchedCtrl = m_options->getuInt32Option(vISA_preRA_ScheduleCtrl);
+  unsigned SchedCtrl = kernel.getuInt32Option(vISA_preRA_ScheduleCtrl);
 
   auto LT = LatencyTable::createLatencyTable(*kernel.fg.builder);
   SchedConfig config(SchedCtrl);
-  RegisterPressure rp(kernel, rpe);
+  RegisterPressure rp(kernel, nullptr);
   // skip extreme test cases that scheduling does not good
   // if (kernel.fg.getNumBB() >= 10000 && rp.rpe->getMaxRP() >= 800)
   //   return false;
@@ -582,16 +589,16 @@ bool preRA_Scheduler::run() {
       continue;
     }
 
-    if (kernel.getOptions()->getuInt32Option(vISA_ScheduleStartBBID) &&
+    if (kernel.getuInt32Option(vISA_ScheduleStartBBID) &&
         (bb->getId() <
-         kernel.getOptions()->getuInt32Option(vISA_ScheduleStartBBID))) {
+         kernel.getuInt32Option(vISA_ScheduleStartBBID))) {
       SCHED_DUMP(std::cerr << "Skip BB" << bb->getId() << "\n");
       continue;
     }
 
-    if (kernel.getOptions()->getuInt32Option(vISA_ScheduleEndBBID) &&
+    if (kernel.getuInt32Option(vISA_ScheduleEndBBID) &&
         (bb->getId() >
-         kernel.getOptions()->getuInt32Option(vISA_ScheduleEndBBID))) {
+         kernel.getuInt32Option(vISA_ScheduleEndBBID))) {
       SCHED_DUMP(std::cerr << "Skip BB" << bb->getId() << "\n");
       continue;
     }
@@ -609,120 +616,120 @@ bool preRA_Scheduler::run() {
     Changed |= S.scheduleBlockForPressure(MaxPressure, Threshold);
     Changed |= S.scheduleBlockForLatency(MaxPressure, Changed, 0);
   }
-  if (kernel.getOptions()->getOption(vISA_PreSchedGRFPressure)) {
-    rp.rpe->run();
-    kernel.fg.builder->getJitInfo()->stats.maxGRFPressure = rp.rpe->getMaxRP();
-  }
+
+  if (Changed)
+    rp.recompute();
+  KernelPressure = rp.getMaxRP();
+
   return Changed;
 }
 
-preRA_RegSharing::preRA_RegSharing(G4_Kernel &k, RPE *rpe)
-    : kernel(k), rpe(rpe) {}
+preRA_RegSharing::preRA_RegSharing(G4_Kernel &k) : kernel(k) {}
 
 preRA_RegSharing::~preRA_RegSharing() {}
 
-bool preRA_RegSharing::run() {
-  // General algorithm:
-  //  1. Schedule for pressure
-  //      - If RP is low (e.g. < 64, based on platform), set maximum number of
-  //      threads
-  //  2. Estimate number of threads [4 .. 12] based on initial RP
-  //  3. Schedule for latency (obtain ILP, stalls, throughput)
-  //  4. Compute cost of schedule
-  //  5. Based on schedule cost:
-  //      - Return ok (keep best schedule)
-  //      - Goto 3
+bool preRA_RegSharing::run(unsigned &KernelPressure) {
 
   if (kernel.getInt32KernelAttr(Attributes::ATTR_Target) != VISA_3D) {
     // Do not run pre-RA scheduler for CM unless user forces it.
-    if (!kernel.getOptions()->getOption(vISA_preRA_ScheduleForce))
+    if (!kernel.getOption(vISA_preRA_ScheduleForce))
       return false;
   }
 
-  bool changed = false;
+  bool Changed = false;
 
-  unsigned SchedCtrl =
-      kernel.getOptions()->getuInt32Option(vISA_preRA_ScheduleCtrl);
+  unsigned SchedCtrl = kernel.getuInt32Option(vISA_preRA_ScheduleCtrl);
   SchedConfig config(SchedCtrl);
-
-  RegisterPressure rp(kernel, rpe);
-
-  std::unordered_map<G4_BB *, unsigned int> rpBB;
-  unsigned KernelPressure = 0;
-
-  // Obtain register pressure estimate of every BB
-  for (auto bb : kernel.fg) {
-    if (bb->size() < SMALL_BLOCK_SIZE || bb->size() > LARGE_BLOCK_SIZE_RPE) {
-      SCHED_DUMP(std::cerr << "Skip block with instructions " << bb->size()
-                           << "\n");
-      continue;
-    }
-
-    unsigned pressure = rp.getPressure(bb);
-    rpBB[bb] = pressure;
-
-    if (pressure > KernelPressure) {
-      KernelPressure = pressure;
-    }
-  }
-
-  // Obs: Heuristic considering PVC with 2 GRF modes as of 03/2020
-  // If maximum register pressure is higher than default GRF mode,
-  // assign the smallest number of threads to this kernel.
-  if (!kernel.getOptions()->getuInt32Option(vISA_HWThreadNumberPerEU) &&
-      (KernelPressure >
-       kernel.getScaledGRFSize(PRESSURE_HIGH_THRESHOLD) -
-           kernel.getOptions()->getuInt32Option(vISA_ReservedGRFNum))) {
-    // Update number of threads, GRF, Acc and SWSB
-    kernel.updateKernelToLargerGRF();
-  }
-
-  // skip extreme test cases that scheduling does not good
-  // if (kernel.fg.getNumBB() >= 10000 && KernelPressure >= 800)
-  //   return false;
-
-  unsigned Threshold = getRPReductionThreshold(kernel);
+  RegisterPressure rp(kernel, nullptr);
+  KernelPressure = rp.getMaxRP();
+  unsigned RPReductionThreshold = getRPReductionThreshold(kernel);
   auto LT = LatencyTable::createLatencyTable(*kernel.fg.builder);
 
+  // Schedule for reg pressure reduction if needed
   for (auto bb : kernel.fg) {
+    // Skip BBs:
     if (bb->size() < SMALL_BLOCK_SIZE || bb->size() > LARGE_BLOCK_SIZE) {
       SCHED_DUMP(std::cerr << "Skip block with instructions " << bb->size()
                            << "\n");
       continue;
     }
 
-    if (kernel.getOptions()->getuInt32Option(vISA_ScheduleStartBBID) &&
-        (bb->getId() <
-         kernel.getOptions()->getuInt32Option(vISA_ScheduleStartBBID))) {
+    if (kernel.getuInt32Option(vISA_ScheduleStartBBID) &&
+        (bb->getId() < kernel.getuInt32Option(vISA_ScheduleStartBBID))) {
       SCHED_DUMP(std::cerr << "Skip BB" << bb->getId() << "\n");
       continue;
     }
 
-    if (kernel.getOptions()->getuInt32Option(vISA_ScheduleEndBBID) &&
-        (bb->getId() >
-         kernel.getOptions()->getuInt32Option(vISA_ScheduleEndBBID))) {
+    if (kernel.getuInt32Option(vISA_ScheduleEndBBID) &&
+        (bb->getId() > kernel.getuInt32Option(vISA_ScheduleEndBBID))) {
       SCHED_DUMP(std::cerr << "Skip BB" << bb->getId() << "\n");
       continue;
     }
 
-    unsigned MaxPressure = rpBB.find(bb) == rpBB.end() ? 0 : rpBB[bb];
-    if (MaxPressure <= Threshold && !config.UseLatency) {
-      SCHED_DUMP(std::cerr << "Skip block with rp " << MaxPressure << "\n");
-      continue;
-    }
-
-    SCHED_DUMP(rp.dump(bb, "Before scheduling, "));
+    // Schedule:
+    SCHED_DUMP(rp.dump(bb, "Before scheduling for pressure reduction, "));
     preDDD ddd(kernel, bb);
     BB_Scheduler S(kernel, ddd, rp, config, *LT);
+    unsigned BBRP = rp.getPressure(bb);
+    Changed |= S.scheduleBlockForPressure(BBRP, RPReductionThreshold);
+  }
 
-    changed |= S.scheduleBlockForPressure(MaxPressure, Threshold);
-      changed |= S.scheduleBlockForLatency(MaxPressure, changed, 0);
+  if (Changed) {
+    // Re-compute register pressure estimation
+    rp.recompute();
+    KernelPressure = rp.getMaxRP();
   }
-  if (kernel.getOptions()->getOption(vISA_PreSchedGRFPressure)) {
-    rp.rpe->run();
-    kernel.fg.builder->getJitInfo()->stats.maxGRFPressure = rp.rpe->getMaxRP();
+
+  // Adjust GRF based on register pressure
+  unsigned oldGRFNum = kernel.getNumRegTotal();
+  kernel.updateKernelByRegPressure(KernelPressure);
+  bool GRFdecreased = kernel.getNumRegTotal() < oldGRFNum;
+  Changed = false;
+
+  // Schedule for latency hiding if needed
+  for (auto bb : kernel.fg) {
+    // Skip BBs:
+    if (bb->size() < SMALL_BLOCK_SIZE || bb->size() > LARGE_BLOCK_SIZE) {
+      SCHED_DUMP(std::cerr << "Skip block with instructions " << bb->size()
+                           << "\n");
+      continue;
+    }
+
+    if (kernel.getuInt32Option(vISA_ScheduleStartBBID) &&
+        (bb->getId() < kernel.getuInt32Option(vISA_ScheduleStartBBID))) {
+      SCHED_DUMP(std::cerr << "Skip BB" << bb->getId() << "\n");
+      continue;
+    }
+
+    if (kernel.getuInt32Option(vISA_ScheduleEndBBID) &&
+        (bb->getId() > kernel.getuInt32Option(vISA_ScheduleEndBBID))) {
+      SCHED_DUMP(std::cerr << "Skip BB" << bb->getId() << "\n");
+      continue;
+    }
+
+    // Schedule:
+    SCHED_DUMP(rp.dump(bb, "Before scheduling for latency hiding, "));
+    preDDD ddd(kernel, bb);
+    BB_Scheduler S(kernel, ddd, rp, config, *LT);
+    unsigned BBRP = rp.getPressure(bb);
+
+    unsigned UpperBoundGRF = 0;
+    if (GRFdecreased && KernelPressure < kernel.grfMode.getMaxGRF())
+      UpperBoundGRF = kernel.grfMode.getLargerGRF();
+    Changed |= S.scheduleBlockForLatency(BBRP, Changed, UpperBoundGRF);
   }
-  return changed;
+
+  if (Changed) {
+    rp.recompute();
+    KernelPressure = rp.getMaxRP();
+  }
+  // In RA extra registers might be needed to satisfy
+  // some restrictions, e.g. alignment, SIMD size, etc.
+  // So extra registers are provided.
+  unsigned ExtraRegs = (unsigned)(kernel.getNumRegTotal() * EXTRA_REGISTERS_FOR_RA / 100.0f);
+  kernel.updateKernelByRegPressure(KernelPressure + ExtraRegs);
+
+  return Changed;
 }
 
 bool BB_Scheduler::verifyScheduling() {
@@ -1205,6 +1212,107 @@ void BB_Scheduler::SethiUllmanScheduling() {
 
 namespace {
 
+template<G4_RegFileKind Kind>
+class AregUsage {
+  struct Write {
+    preNode *N = nullptr;
+    unsigned NumRegsUsed = 0;
+  };
+  // The list is to use to track active scheduled nodes that write Aregs.
+  std::list<Write> Writes;
+  // Numbers of physical architecture registers
+  const unsigned PhyRegNum;
+  // The total Aregs used by active Areg writes.
+  unsigned NumRegsUsed = 0;
+
+public:
+  AregUsage(unsigned N) : PhyRegNum(N) {}
+
+  bool isPressureHigh() const { return NumRegsUsed >= PhyRegNum; }
+
+  unsigned numRegsUsed() const { return NumRegsUsed; }
+
+  unsigned numPhysicalRegs() const { return PhyRegNum; }
+
+  void update(preNode *N);
+
+  unsigned numActiveRegWritten(const G4_INST *Inst) const;
+
+  unsigned numActiveRegRead(const G4_INST *Inst) const;
+};
+
+template<>
+void AregUsage<G4_FLAG>::update(preNode *N) {
+  vASSERT(N && N->isScheduled);
+  // Only update flag usage when the scheduled node uses flag through condition
+  // modifier and predicate. Currently assume the register allocator could do
+  // the optimal assignment and simply add the numbers of flag registers used
+  // by active flag writes to represent the total active flag registers.
+  // TODO: Consider tracking flag uses through src and dst as well.
+  G4_INST *NInst = N->getInst();
+  if (!NInst || !NInst->usesFlag())
+    return;
+
+  auto allFlagUsesScheduled = [](preNode *Wr) -> bool {
+    return std::all_of(Wr->Succs.begin(), Wr->Succs.end(), [Wr](preEdge &E) {
+      preNode *Succ = E.getNode();
+      G4_INST *SuccInst = Succ->getInst();
+      return !SuccInst ||
+          SuccInst->getPredicateBase() != Wr->getInst()->getCondModBase() ||
+          Succ->isScheduled;
+    });
+  };
+
+  // Erase a flag write if
+  // 1. the new node that has the same cond mod, or
+  // 2. all of its successors that use the flag are scheduled
+  bool WriteFlag = NInst->getCondMod() && NInst->opcode() != G4_sel;
+  for (auto IT = Writes.begin(), IE = Writes.end(); IT != IE; ) {
+    Write &Wr = *IT;
+    if (WriteFlag &&
+        NInst->getCondModBase() == Wr.N->getInst()->getCondModBase()) {
+      NumRegsUsed -= Wr.NumRegsUsed;
+      IT = Writes.erase(IT);
+    } else if (allFlagUsesScheduled(Wr.N)) {
+      NumRegsUsed -= Wr.NumRegsUsed;
+      IT = Writes.erase(IT);
+    } else
+      ++IT;
+  }
+  // Append the new flag write to the end.
+  if (WriteFlag) {
+    unsigned NumRegsNeeded =
+        NInst->getCondMod()->getTopDcl()->getNumRegNeeded();
+    vASSERT(NumRegsNeeded == 1 || NumRegsNeeded == 2);
+    NumRegsUsed += NumRegsNeeded;
+    Writes.push_back({N, NumRegsNeeded});
+  }
+}
+
+template<>
+unsigned AregUsage<G4_FLAG>::numActiveRegWritten(const G4_INST *Inst) const {
+  if (!Inst || !Inst->getCondMod() || Inst->opcode() == G4_sel)
+    return 0;
+
+  for (const Write &Wr : Writes) {
+    if (Inst->getCondModBase() == Wr.N->getInst()->getCondModBase())
+      return Wr.NumRegsUsed;
+  }
+  return 0;
+}
+
+template<>
+unsigned AregUsage<G4_FLAG>::numActiveRegRead(const G4_INST *Inst) const {
+  if (!Inst || !Inst->getPredicate())
+    return 0;
+
+  for (const Write &Wr : Writes) {
+    if (Inst->getPredicateBase() == Wr.N->getInst()->getCondModBase())
+      return Wr.NumRegsUsed;
+  }
+  return 0;
+}
+
 // Queue for scheduling to hide latency.
 class LatencyQueue : public QueueBase {
   // Represent the type of SchedCandidate picked from ready-list. The types are
@@ -1212,8 +1320,7 @@ class LatencyQueue : public QueueBase {
   enum CandReason : uint8_t {
     NoCand,
     PseudoKill,
-    UsesActiveFlag,
-    UsesNoFlag,
+    AregPhyLimit,
     NodeGroup,
     SendNoReturn,
     NodePriority,
@@ -1226,8 +1333,7 @@ class LatencyQueue : public QueueBase {
     switch (Reason) {
     case NoCand:         return "NOCAND";
     case PseudoKill:     return "KILL";
-    case UsesActiveFlag: return "ACTIVEFLAG";
-    case UsesNoFlag:     return "NOFLAG";
+    case AregPhyLimit:   return "AREGPHYLIMIT";
     case NodeGroup:      return "GROUP";
     case SendNoReturn:   return "SENDNORET";
     case NodePriority:   return "PRIORITY";
@@ -1241,8 +1347,10 @@ class LatencyQueue : public QueueBase {
   struct SchedCandidate {
     preNode* Node = nullptr;
     CandReason Reason = NoCand;
-    bool UsesFlag = false;
-    bool UsesActiveFlag = false;
+    // Number of flag register added if selecting the candidate. A negative
+    // number means the candidate reads an active flag and could release the
+    // flag potentially.
+    int NumFlagRegsAdded = 0;
 
     bool isValid() const { return Node; }
   };
@@ -1268,10 +1376,7 @@ class LatencyQueue : public QueueBase {
   // The register-pressure limit we use to decide sub-blocking
   unsigned GroupingPressureLimit;
 
-  // The list is to use to track active scheduled nodes that write flag.
-  std::list<preNode *> FlagWrites;
-  // Numbers of flag registers from getNumFlagRegisters()
-  const unsigned FlagRegNum;
+  AregUsage<G4_FLAG> FlagUsage;
 
 public:
   LatencyQueue(preDDD &ddd, RegisterPressure &rp, SchedConfig config,
@@ -1279,7 +1384,7 @@ public:
       : QueueBase(ddd, rp, config), LT(LT),
         HoldList([this](preNode *a, preNode *b) { return compareHold(a, b); }),
         GroupingPressureLimit(GroupingThreshold),
-        FlagRegNum(ddd.getKernel().fg.builder->getNumFlagRegisters()) {
+        FlagUsage(ddd.getKernel().fg.builder->getNumFlagRegisters()) {
     init();
   }
 
@@ -1360,45 +1465,8 @@ public:
     }
   }
 
-  bool isFlagPressureHigh() const { return FlagWrites.size() >= FlagRegNum; }
-
-  void updateFlagUsage(preNode* N) {
-    vASSERT(N && N->isScheduled);
-    // Only update flag usage when the scheduled node uses flag through
-    // condition modifier and predicate.
-    G4_INST *NInst = N->getInst();
-    if (!NInst || !NInst->usesFlag())
-      return;
-
-    auto allFlagUsesScheduled = [](preNode *FW) -> bool {
-      return std::all_of(FW->Succs.begin(), FW->Succs.end(), [FW](preEdge &E) {
-        preNode *Succ = E.getNode();
-        G4_INST *SuccInst = Succ->getInst();
-        return !SuccInst ||
-            !SuccInst->getPredicate() ||
-            (SuccInst->getPredicate()->getBase() !=
-                FW->getInst()->getCondMod()->getBase()) ||
-            Succ->isScheduled;
-      });
-    };
-
-    // Erase a flag write if
-    // 1. the new node that has the same cond mod, or
-    // 2. all of its successors that use the flag are scheduled
-    bool writeFlag = NInst->getCondMod() && NInst->opcode() != G4_sel;
-    for (auto it = FlagWrites.begin(), ie = FlagWrites.end(); it != ie; ) {
-      preNode *FW = *it;
-      if (writeFlag && NInst->getCondMod()->getBase() ==
-                           FW->getInst()->getCondMod()->getBase()) {
-        it = FlagWrites.erase(it);
-      } else if (allFlagUsesScheduled(FW)) {
-        it = FlagWrites.erase(it);
-      } else
-        ++it;
-    }
-    // Append the new flag write to the end.
-    if (writeFlag)
-      FlagWrites.push_back(N);
+  void updateAregUsage(preNode* N) {
+    FlagUsage.update(N);
   }
 
 private:
@@ -1444,37 +1512,46 @@ private:
     return std::nullopt;
   }
 
-  // Return the candidate that won't increase flag pressure and could avoid
-  // flag spills potentially. Note that currently this heuristic only considers
-  // flag uses in condition modifier and predicate, and does not consider flag
-  // in src or dst.
+  // Return the candidate that won't increase flag pressure above the physical
+  // limit and could avoid flag spills potentially. Note that currently this
+  // heuristic only considers flag uses in condition modifier and predicate,
+  // and does not consider flag in src or dst.
   std::optional<SchedCandidate *> tryCandidateToAvoidFlagSpill(
       SchedCandidate &Cand, SchedCandidate &TryCand) const {
-    // Favor a node that uses any active flag.
-    if (auto Res = tryGreater(TryCand.UsesActiveFlag, Cand.UsesActiveFlag,
-                              TryCand, Cand, UsesActiveFlag))
-      return Res;
-
-    // Favor a node that does not use flag at all.
-    if (auto Res = tryLess(TryCand.UsesFlag, Cand.UsesFlag, TryCand, Cand,
-                           UsesNoFlag))
+    // If both won't cause spill, they are equally good. Otherwise, favor the
+    // one that has lower pressure.
+    int PhyRegNum = FlagUsage.numPhysicalRegs();
+    int TryCandPressure = TryCand.NumFlagRegsAdded + FlagUsage.numRegsUsed();
+    int CandPressure = Cand.NumFlagRegsAdded + FlagUsage.numRegsUsed();
+    int TryCandVal = TryCandPressure > PhyRegNum ? TryCandPressure : PhyRegNum;
+    int CandVal = CandPressure > PhyRegNum ? CandPressure : PhyRegNum;
+    if (auto Res = tryLess(TryCandVal, CandVal, TryCand, Cand, AregPhyLimit))
       return Res;
 
     return std::nullopt;
   }
 
   void initCandidate(SchedCandidate &TryCand) {
-    // Only record the flag stat for the candidate when current flag pressure
-    // is high.
-    if (!isFlagPressureHigh())
-      return;
-
     G4_INST *Inst = TryCand.Node->getInst();
-    TryCand.UsesFlag = Inst && Inst->usesFlag();
-    TryCand.UsesActiveFlag = Inst && Inst->getPredicate() &&
-        std::any_of(FlagWrites.begin(), FlagWrites.end(), [Inst](preNode *FW) {
-            return FW->getInst()->getCondMod()->getBase() ==
-                Inst->getPredicate()->getBase();});
+    // Only record the flag stats for the candidate when it uses flag.
+    if (Inst && Inst->usesFlag()) {
+      // Update NumFlagRegsAdded if the TryCand writes to a different flag than
+      // active flag writes.
+      if (Inst->getCondMod() && Inst->opcode() != G4_sel &&
+          FlagUsage.numActiveRegWritten(Inst) == 0) {
+        TryCand.NumFlagRegsAdded +=
+            Inst->getCondMod()->getTopDcl()->getNumRegNeeded();
+        vASSERT(TryCand.NumFlagRegsAdded == 1 || TryCand.NumFlagRegsAdded == 2);
+      }
+      // Update NumFlagRegsAdded if the TryCand reads any active flag register
+      // and does not overwrite the same flag.
+      // TODO: Consider something like assigning a fractional weight if the
+      // flag has multiple uses to favor the only use or the last use in some
+      // way. Currently treat all uses as the only use to simplify the
+      // implementation.
+      if (Inst->getPredicateBase() != Inst->getCondModBase())
+        TryCand.NumFlagRegsAdded -= FlagUsage.numActiveRegRead(Inst);
+    }
   }
 
   // Return true if TryCand is better than current Cand. The heuristics should
@@ -1547,15 +1624,16 @@ private:
 
 } // namespace
 
-//
 bool BB_Scheduler::scheduleBlockForLatency(unsigned &MaxPressure,
-                                           bool ReassignID, unsigned KernelRP) {
+                                           bool ReassignID,
+                                           unsigned UpperBoundGRF) {
   auto tryLatencyHiding = [=](unsigned nr) {
     if (!config.UseLatency)
       return false;
 
-    // KernelRP == 0 means we are scheduling for the fixed number of GRF
-    if (KernelRP == 0 && MaxPressure >= getLatencyHidingThreshold(kernel, nr))
+    // UpperBoundGRF == 0 means we are scheduling for the fixed number of GRF
+    if (UpperBoundGRF == 0 &&
+        MaxPressure >= getLatencyHidingThreshold(kernel, nr))
       return false;
 
     // simple ROI check.
@@ -1578,17 +1656,14 @@ bool BB_Scheduler::scheduleBlockForLatency(unsigned &MaxPressure,
   if (!tryLatencyHiding(NumGrfs))
     return false;
 
-  // UpperBoundGRF == NumGrfs means we only schedule under single NumGRF
+  // UpperBoundGRF == 0 means we only schedule under single NumGRF
   // setting for this block instead of trying to find the best schedule
-  // among multiple NumGRF setting.
-  unsigned UpperBoundGRF = NumGrfs;
+  // among multiple NumGRF settings.
+  if (UpperBoundGRF == 0)
+    UpperBoundGRF = NumGrfs;
   unsigned SavedEstimation = 0;
   std::vector<G4_INST *> SavedSchedule;
 
-  // multiple settings are applied only to some blocks to save time
-  if (KernelRP > 0 && MaxPressure > 40 && MaxPressure * 2 > KernelRP)
-    UpperBoundGRF = std::max(256U, UpperBoundGRF);
-
   for (; NumGrfs <= UpperBoundGRF; NumGrfs += 32) {
     // try grouping-threshold decremently until we find a schedule likely won't
     // spill
@@ -1675,8 +1750,8 @@ void BB_Scheduler::LatencyScheduling(unsigned GroupingThreshold) {
       NextCycle += LT.getOccupancy(N->getInst());
     }
     N->isScheduled = true;
-    // Update flag usage after scheduling a node.
-    Q.updateFlagUsage(N);
+    // Update AReg usage after scheduling a node.
+    Q.updateAregUsage(N);
 
     for (auto I = N->succ_begin(), E = N->succ_end(); I != E; ++I) {
       preNode *Node = I->getNode();
