@@ -677,7 +677,9 @@ private:
     auto dst = builder.createDst(rtmp->getRegVar(), 0, subreg, 1, Type_UD);
     auto src0 = builder.createSrc(rtmp->getRegVar(), 0, subreg,
                                   builder.getRegionScalar(), Type_UD);
-    auto src1 = builder.createRelocImm(0, Type_UD);
+    auto src1 =
+      builder.createRelocImm(GenRelocType::R_SYM_ADDR_32,
+                             CROSS_THREAD_OFF_R0_RELOCATION_NAME, 0, Type_UD);
     auto addInst =
       builder.createBinOp(G4_add, g4::SIMD1, dst, src0, src1,
                           InstOpt_WriteEnable | InstOpt_NoCompact, false);
@@ -853,7 +855,9 @@ public:
       // case of the cross_thread_size is changed after compilation (e.g. gtpin
       // inserted argument), the relocation need to be resolved to the new
       // cross_thread_size.
-      G4_Operand *addSrc1 = builder.createRelocImm(perThreadOffsetMem, Type_UW);
+      G4_Operand *addSrc1 =
+          builder.createRelocImm(GenRelocType::R_PER_THREAD_PAYLOAD_OFFSET_32,
+                                 kernel.getName(), perThreadOffsetMem, Type_UD);
       auto addDst = builder.createDst(rtmp->getRegVar(), 0, 2, 1, Type_UD);
       // instruction has relocation must not be compacted
       auto addInst =
@@ -1094,6 +1098,7 @@ void Optimizer::insertFenceBeforeEOT() {
       builder.getOption(vISA_clearLSCUGMWritesBeforeEOT);
   bool clearHDCWritesBeforeEOT =
       builder.getOption(vISA_clearHDCWritesBeforeEOT);
+  bool clearWritesBeforeEOT = builder.needBarrierWA() && builder.supportsLSC();
   // for vector path we need this WA always, so just use table
   if (kernel.getInt32KernelAttr(Attributes::ATTR_Target) == VISA_CM) {
     clearHDCWritesBeforeEOT =
@@ -1103,7 +1108,8 @@ void Optimizer::insertFenceBeforeEOT() {
                            VISA_WA_CHECK(builder.getPWaTable(), Wa_22013689345);
   }
   if (!toRemoveFence && !clearHDCWritesBeforeEOT &&
-      !(builder.supportsLSC() && clearHdcWritesLSCUGM)) {
+      !(builder.supportsLSC() && clearHdcWritesLSCUGM) &&
+      !clearWritesBeforeEOT) {
     return;
   }
 
@@ -1115,6 +1121,7 @@ void Optimizer::insertFenceBeforeEOT() {
   bool hasUAVWrites = false;
   bool hasSLMWrites = false;
   bool hasTypedWrites = false;
+  bool hasWrites = false;
   std::list<std::pair<G4_BB *, G4_INST *>> toBeRemoved;
 
   for (auto bb : kernel.fg) {
@@ -1126,11 +1133,12 @@ void Optimizer::insertFenceBeforeEOT() {
       hasUAVWrites = true;
       hasSLMWrites = true;
       hasTypedWrites = true;
+      hasWrites = true;
       break;
     }
 
     for (auto inst : *bb) {
-      if (inst->isSend()) {
+      if (inst->isSend() && !inst->isEOT()) {
         auto msgDesc = inst->asSendInst()->getMsgDesc();
         if (msgDesc->isLSC()) {
           if (toRemoveFence && msgDesc->getSFID() == SFID::UGM &&
@@ -1144,6 +1152,7 @@ void Optimizer::insertFenceBeforeEOT() {
         }
 
         if (msgDesc->isWrite()) {
+          hasWrites = true;
           if (msgDesc->isHDC()) {
             if (msgDesc->isSLM()) {
               hasSLMWrites = true;
@@ -1200,8 +1209,9 @@ void Optimizer::insertFenceBeforeEOT() {
   }
 
   if ((!clearHDCWritesBeforeEOT &&
-       !(builder.supportsLSC() && clearHdcWritesLSCUGM)) ||
-      !(hasUAVWrites || hasSLMWrites || hasTypedWrites)) {
+       !(builder.supportsLSC() && clearHdcWritesLSCUGM) &&
+       !clearWritesBeforeEOT) ||
+      !(hasUAVWrites || hasSLMWrites || hasTypedWrites || hasWrites)) {
     return;
   }
 
@@ -1262,6 +1272,13 @@ void Optimizer::insertFenceBeforeEOT() {
           }
         }
       }
+
+      if (clearWritesBeforeEOT && hasWrites) {
+        auto fenseInst = builder.translateLscFence(
+            nullptr, SFID::UGM, LSC_FENCE_OP_EVICT, LSC_SCOPE_TILE);
+        bb->insertBefore(iter, fenseInst);
+      }
+
       builder.instList.clear();
     }
   }
