@@ -185,12 +185,12 @@ public:
     }
   }
 
-  BoundedRA(GlobalRA &ra, const LiveRangeVec &l);
+  BoundedRA(GlobalRA &ra, const LiveRangeVec *l = nullptr);
 
 private:
   GlobalRA &gra;
   G4_Kernel &kernel;
-  const LiveRangeVec &lrs;
+  const LiveRangeVec *lrs;
   static const unsigned int bitsetSz = 256;
   // Map each inst -> list of busy GRFs
   std::unordered_map<const G4_INST *, std::bitset<bitsetSz>> busyGRF;
@@ -237,30 +237,20 @@ private:
 
 class SpillManagerGRF {
 public:
-  using DECLARE_LIST = std::list<G4_Declare *>;
   using LR_LIST = std::list<LiveRange *>;
   using LSLR_LIST = std::list<LSLiveRange *>;
-  using INST_LIST = std::list<G4_INST *, INST_LIST_NODE_ALLOCATOR>;
-  typedef struct Edge {
-    unsigned first;
-    unsigned second;
-  } EDGE;
-  using INTF_LIST = std::list<EDGE>;
 
-  // Methods
+  // Construtor for GCRA.
+  SpillManagerGRF(GlobalRA &g, unsigned spillAreaOffset,
+                  const LivenessAnalysis *lvInfo, const Interference *intf,
+                  const LR_LIST *spilledLRs, bool useSpillReg,
+                  unsigned spillRegSize, unsigned indrSpillRegSize,
+                  bool enableSpillSpaceCompression, bool useScratchMsg);
 
-  SpillManagerGRF(GlobalRA &g, unsigned spillAreaOffset, unsigned varIdCount,
-                  const LivenessAnalysis *lvInfo, const LiveRangeVec& lrInfo,
-                  const Interference *intf, const LR_LIST *spilledLRs,
-                  unsigned iterationNo, bool useSpillReg, unsigned spillRegSize,
-                  unsigned indrSpillRegSize, bool enableSpillSpaceCompression,
-                  bool useScratchMsg, bool avoidDstSrcOverlap);
-
-  SpillManagerGRF(GlobalRA &g, unsigned spillAreaOffset, unsigned varIdCount,
+  // Constructor for linear scan RA.
+  SpillManagerGRF(GlobalRA &g, unsigned spillAreaOffset,
                   const LivenessAnalysis *lvInfo, LSLR_LIST *spilledLSLRs,
-                  bool enableSpillSpaceCompression, bool useScratchMsg,
-                  bool avoidDstSrcOverlap,
-                  const LiveRangeVec &emptyLRs);
+                  bool enableSpillSpaceCompression, bool useScratchMsg);
 
   ~SpillManagerGRF() {}
 
@@ -275,7 +265,6 @@ public:
 
   unsigned getNumGRFSpill() const { return numGRFSpill; }
   unsigned getNumGRFFill() const { return numGRFFill; }
-  unsigned getNumGRFMove() const { return numGRFMove; }
   // return the next cumulative logical offset. This does not non-spilled stuff
   // like private variables placed by IGC (marked by spill_mem_offset) this
   // should only be called after insertSpillFillCode()
@@ -344,17 +333,17 @@ private:
   static unsigned getSendOwordWriteType();
   unsigned getSendExDesc(bool isWrite, bool isScatter) const;
 
-  unsigned getSpillIndex(G4_RegVar *spilledRegVar);
+  unsigned getSpillIndex();
 
-  unsigned getFillIndex(G4_RegVar *spilledRegVar);
+  unsigned getFillIndex();
 
-  unsigned getTmpIndex(G4_RegVar *spilledRegVar);
+  unsigned getTmpIndex();
 
-  unsigned getMsgSpillIndex(G4_RegVar *spilledRegVar);
+  unsigned getMsgSpillIndex();
 
-  unsigned getMsgFillIndex(G4_RegVar *spilledRegVar);
+  unsigned getMsgFillIndex();
 
-  unsigned getAddrSpillFillIndex(G4_RegVar *spilledRegVar);
+  unsigned getAddrSpillFillIndex();
 
   template <class REGION_TYPE> G4_RegVar *getRegVar(REGION_TYPE *region) const;
 
@@ -372,12 +361,8 @@ private:
 
   unsigned owordMask() const;
 
-  unsigned dwordMask() const;
-
   bool owordAligned(unsigned offset) const;
 
-  bool dwordAligned(unsigned offset) const;
-
   static unsigned cdiv(unsigned dvd, unsigned dvr);
 
   G4_RegVar *getRegVar(unsigned id) const;
@@ -428,10 +413,6 @@ private:
   unsigned getRegionByteSize(G4_SrcRegRegion *region,
                              G4_ExecSize execSize) const;
 
-  bool isScalarReplication(G4_SrcRegRegion *region) const;
-
-  bool repeatSIMD16or32Source(G4_SrcRegRegion *region) const;
-
   G4_Declare *createRangeDeclare(const char *name, G4_RegFileKind regFile,
                                  unsigned short nElems, unsigned short nRows,
                                  G4_Type type, DeclareType kind,
@@ -603,66 +584,33 @@ private:
 
   void insertSpillRangeCode(INST_LIST::iterator spilledInstIter, G4_BB *bb);
 
-  INST_LIST::iterator
-  insertSendFillRangeCode(G4_SrcRegRegion *filledRegion,
-                          INST_LIST::iterator filledInstIter, G4_BB *bb);
+  void insertSendFillRangeCode(G4_SrcRegRegion *filledRegion,
+                               INST_LIST::iterator filledInstIter, G4_BB *bb);
 
   void insertFillGRFRangeCode(G4_SrcRegRegion *filledRegion,
                               INST_LIST::iterator filledInstIter, G4_BB *bb);
 
   bool useSplitSend() const;
 
-  int getHWordEncoding(int numHWord) {
-    switch (numHWord) {
-    case 1:
-      return 0;
-    case 2:
-      return 1;
-    case 4:
-      return 2;
-    case 8:
-      return 3;
-    default:
-      vISA_ASSERT_UNREACHABLE("only 1/2/4/8 HWords are supported");
-      return 0;
-    }
-  }
-
-  ChannelMask getChMaskForSpill(int numHWord) const {
-    switch (numHWord) {
-    case 1:
-    case 2:
-      return ChannelMask::createFromAPI(CHANNEL_MASK_R);
-    case 4:
-      return ChannelMask::createFromAPI(CHANNEL_MASK_RG);
-    case 8:
-      return ChannelMask::createFromAPI(CHANNEL_MASK_RGBA);
-    default:
-      vISA_ASSERT_UNREACHABLE("illegal spill size");
-      return ChannelMask::createFromAPI(CHANNEL_MASK_R);
-    }
-  }
-
   void getOverlappingIntervals(G4_Declare *dcl,
                                std::vector<G4_Declare *> &intervals) const;
 
   // Data
   GlobalRA &gra;
   IR_Builder *builder_;
-  unsigned varIdCount_;
+  const unsigned varIdCount_;
   unsigned latestImplicitVarIdCount_;
   const LivenessAnalysis *lvInfo_;
-  const LiveRangeVec &lrInfo_;
+  const LiveRangeVec *lrInfo_;
   const LR_LIST *spilledLRs_;
   LSLR_LIST *spilledLSLRs_;
-  std::vector<unsigned> spillRangeCount_;
-  std::vector<unsigned> fillRangeCount_;
-  std::vector<unsigned> tmpRangeCount_;
-  std::vector<unsigned> msgSpillRangeCount_;
-  std::vector<unsigned> msgFillRangeCount_;
-  std::vector<unsigned> addrSpillFillRangeCount_;
+  unsigned spillRangeCount_ = 0;
+  unsigned fillRangeCount_ = 0;
+  unsigned tmpRangeCount_ = 0;
+  unsigned msgSpillRangeCount_ = 0;
+  unsigned msgFillRangeCount_ = 0;
+  unsigned addrSpillFillRangeCount_ = 0;
   unsigned nextSpillOffset_;
-  unsigned iterationNo_;
   unsigned bbId_ = UINT_MAX;
   unsigned spillAreaOffset_;
   bool doSpillSpaceCompression;
@@ -682,9 +630,6 @@ private:
   // The number of GRF fill.
   unsigned numGRFFill = 0;
 
-  // The number of mov.
-  unsigned numGRFMove = 0;
-
   // CISA instruction id of current instruction
   G4_INST *curInst;
 
@@ -723,10 +668,6 @@ private:
   bool checkDefUseDomRel(G4_DstRegRegion *dst, G4_BB *bb);
   void updateRMWNeeded();
 
-  bool useLSCMsg = false;
-  bool useLscNonstackCall = false;
-  bool useLscForScatterSpill = false;
-
   // Used for new fail safe RA mechanism.
   BoundedRA context;
 
@@ -744,23 +685,7 @@ private:
     return iter == addrTakenSpillFill.end() ? nullptr : iter->second;
   }
 
-  bool headerNeeded() const {
-    bool needed = true;
-
-    if (useScratchMsg_ && builder_->getPlatform() >= GENX_SKL)
-      needed = false;
-
-    if (builder_->kernel.fg.getHasStackCalls() ||
-        builder_->kernel.fg.getIsStackCallFunc())
-      needed = false;
-
-    if (useLSCMsg)
-      needed = false;
-
-    return needed;
-  }
-
-  void initializeRangeCount(unsigned size);
+  bool headerNeeded() const;
 
   // return true if offset for spill/fill message needs to be GRF-aligned
   bool needGRFAlignedOffset() const { return useScratchMsg_ || useSplitSend(); }
