@@ -508,9 +508,11 @@ public:
                                std::string funcName, unsigned char argSize,
                                unsigned char returnSize) override;
 
-  VISA_BUILDER_API int AppendVISACFIndirectFuncCallInst(
-      VISA_PredOpnd *pred, VISA_EMask_Ctrl emask, VISA_Exec_Size executionSize,
-      VISA_VectorOpnd *funcAddr, uint8_t argSize, uint8_t returnSize) override;
+  VISA_BUILDER_API int
+  AppendVISACFIndirectFuncCallInst(VISA_PredOpnd *pred, VISA_EMask_Ctrl emask,
+                                   VISA_Exec_Size executionSize, bool isUniform,
+                                   VISA_VectorOpnd *funcAddr, uint8_t argSize,
+                                   uint8_t returnSize) override;
 
   VISA_BUILDER_API int AppendVISACFSymbolInst(std::string symbolName,
                                               VISA_VectorOpnd *dst) override;
@@ -823,44 +825,22 @@ public:
       VISA_SVM_Block_Num numBlocks, VISA_StateOpndHandle *surface,
       VISA_RawOpnd *address, VISA_RawOpnd *dst) override;
 
-  // overloaded functions for untyped instructions with/without surface index
-  VISA_BUILDER_API int AppendVISALscUntypedLoad(
-      LSC_OP op, LSC_SFID sfid, VISA_PredOpnd *pred, VISA_Exec_Size execSize,
-      VISA_EMask_Ctrl emask, LSC_CACHE_OPTS cacheOpts, LSC_ADDR addr,
-      LSC_DATA_SHAPE data, VISA_VectorOpnd *surface, VISA_RawOpnd *dstData,
-      VISA_RawOpnd *src0Addr) override;
   VISA_BUILDER_API int AppendVISALscUntypedLoad(
       LSC_OP op, LSC_SFID sfid, VISA_PredOpnd *pred, VISA_Exec_Size execSize,
       VISA_EMask_Ctrl emask, LSC_CACHE_OPTS cacheOpts, LSC_ADDR addr,
       LSC_DATA_SHAPE data, VISA_VectorOpnd *surface, unsigned surfaceIndex,
       VISA_RawOpnd *dstData, VISA_RawOpnd *src0Addr) override;
-  VISA_BUILDER_API int AppendVISALscUntypedStore(
-      LSC_OP op, LSC_SFID sfid, VISA_PredOpnd *pred, VISA_Exec_Size execSize,
-      VISA_EMask_Ctrl emask, LSC_CACHE_OPTS cacheOpts, LSC_ADDR addr,
-      LSC_DATA_SHAPE data, VISA_VectorOpnd *surface, VISA_RawOpnd *src0Addr,
-      VISA_RawOpnd *src1Data) override;
   VISA_BUILDER_API int AppendVISALscUntypedStore(
       LSC_OP op, LSC_SFID sfid, VISA_PredOpnd *pred, VISA_Exec_Size execSize,
       VISA_EMask_Ctrl emask, LSC_CACHE_OPTS cacheOpts, LSC_ADDR addr,
       LSC_DATA_SHAPE data, VISA_VectorOpnd *surface, unsigned surfaceIndex,
       VISA_RawOpnd *src0Addr,VISA_RawOpnd *src1Data) override;
-  VISA_BUILDER_API int AppendVISALscUntypedAtomic(
-      LSC_OP op, LSC_SFID sfid, VISA_PredOpnd *pred, VISA_Exec_Size execSize,
-      VISA_EMask_Ctrl emask, LSC_CACHE_OPTS cacheOpts, LSC_ADDR addr,
-      LSC_DATA_SHAPE data, VISA_VectorOpnd *surface, VISA_RawOpnd *dstReadBack,
-      VISA_RawOpnd *src0Addr, VISA_RawOpnd *src1AtomOpnd1,
-      VISA_RawOpnd *src2AtomOpnd2) override;
   VISA_BUILDER_API int AppendVISALscUntypedAtomic(
       LSC_OP op, LSC_SFID sfid, VISA_PredOpnd *pred, VISA_Exec_Size execSize,
       VISA_EMask_Ctrl emask, LSC_CACHE_OPTS cacheOpts, LSC_ADDR addr,
       LSC_DATA_SHAPE data, VISA_VectorOpnd *surface, unsigned surfaceIndex,
       VISA_RawOpnd *dstReadBack, VISA_RawOpnd *src0Addr, VISA_RawOpnd *src1AtomOpnd1,
       VISA_RawOpnd *src2AtomOpnd2) override;
-  VISA_BUILDER_API int AppendVISALscUntypedInst(
-      LSC_OP op, LSC_SFID sfid, VISA_PredOpnd *pred, VISA_Exec_Size execSize,
-      VISA_EMask_Ctrl emask, LSC_CACHE_OPTS cacheOpts, LSC_ADDR addr,
-      LSC_DATA_SHAPE data, VISA_VectorOpnd *surface, VISA_RawOpnd *dst,
-      VISA_RawOpnd *src0, VISA_RawOpnd *src1, VISA_RawOpnd *src2) override;
   VISA_BUILDER_API int AppendVISALscUntypedInst(
       LSC_OP op, LSC_SFID sfid, VISA_PredOpnd *pred, VISA_Exec_Size execSize,
       VISA_EMask_Ctrl emask, LSC_CACHE_OPTS cacheOpts, LSC_ADDR addr,
@@ -884,40 +864,44 @@ public:
   VISA_BUILDER_API int AppendVISALscTypedLoad(
       LSC_OP op, VISA_PredOpnd *pred, VISA_Exec_Size execSize,
       VISA_EMask_Ctrl emask, LSC_CACHE_OPTS cacheOpts, LSC_ADDR_TYPE addrModel,
-      LSC_ADDR_SIZE addrSize, LSC_DATA_SHAPE data, VISA_VectorOpnd *surface,
-      VISA_RawOpnd *dstData, VISA_RawOpnd *Us, VISA_RawOpnd *Vs,
-      VISA_RawOpnd *Rs, VISA_RawOpnd *LODs) override;
+      LSC_ADDR_SIZE addrSize, LSC_DATA_SHAPE data,
+      VISA_VectorOpnd *surface, unsigned surfaceIndex,
+      VISA_RawOpnd *dstData,
+      VISA_RawOpnd *Us, int uOffset,
+      VISA_RawOpnd *Vs, int vOffset,
+      VISA_RawOpnd *Rs, int rOffset, VISA_RawOpnd *LODs) override;
   VISA_BUILDER_API int AppendVISALscTypedStore(
       LSC_OP op, VISA_PredOpnd *pred, VISA_Exec_Size execSize,
       VISA_EMask_Ctrl emask, LSC_CACHE_OPTS cacheOpts, LSC_ADDR_TYPE addrType,
-      LSC_ADDR_SIZE addrSize, LSC_DATA_SHAPE data, VISA_VectorOpnd *surface,
-      VISA_RawOpnd *Us, VISA_RawOpnd *Vs, VISA_RawOpnd *Rs, VISA_RawOpnd *LODs,
+      LSC_ADDR_SIZE addrSize, LSC_DATA_SHAPE data,
+      VISA_VectorOpnd *surface, unsigned surfaceIndex,
+      VISA_RawOpnd *Us, int uOffset, VISA_RawOpnd *Vs, int vOffset,
+      VISA_RawOpnd *Rs, int rOffset, VISA_RawOpnd *LODs,
       VISA_RawOpnd *src1Data) override;
   VISA_BUILDER_API int AppendVISALscTypedAtomic(
       LSC_OP op, VISA_PredOpnd *pred, VISA_Exec_Size execSize,
       VISA_EMask_Ctrl emask, LSC_CACHE_OPTS cacheOpts, LSC_ADDR_TYPE addrType,
-      LSC_ADDR_SIZE addrSize, LSC_DATA_SHAPE data, VISA_VectorOpnd *surface,
-      VISA_RawOpnd *dst, VISA_RawOpnd *Us, VISA_RawOpnd *Vs, VISA_RawOpnd *Rs,
-      VISA_RawOpnd *LODs, VISA_RawOpnd *src1AtomicOpnd1,
-      VISA_RawOpnd *src2AtomicOpnd2) override;
+      LSC_ADDR_SIZE addrSize, LSC_DATA_SHAPE data,
+      VISA_VectorOpnd *surface, unsigned surfaceIndex,
+      VISA_RawOpnd *dst,
+      VISA_RawOpnd *Us, int uOffset,
+      VISA_RawOpnd *Vs, int vOffset,
+      VISA_RawOpnd *Rs, int rOffset,
+      VISA_RawOpnd *LODs,
+      VISA_RawOpnd *src1AtomicOpnd1, VISA_RawOpnd *src2AtomicOpnd2) override;
 
-  VISA_BUILDER_API int AppendVISALscTypedInst(
-      LSC_OP op, VISA_PredOpnd *pred, VISA_Exec_Size execSize,
-      VISA_EMask_Ctrl emask, LSC_CACHE_OPTS cacheOpts, LSC_ADDR_TYPE addrType,
-      LSC_ADDR_SIZE addrSize, LSC_DATA_SHAPE data, VISA_VectorOpnd *surface,
-      VISA_RawOpnd *dst, VISA_RawOpnd *coord0s, VISA_RawOpnd *coord1s,
-      VISA_RawOpnd *coord2s, VISA_RawOpnd *features, VISA_RawOpnd *src1, VISA_RawOpnd *src2) override;
   VISA_BUILDER_API int AppendVISALscTypedInst(
       LSC_OP op, VISA_PredOpnd *pred, VISA_Exec_Size execSize,
       VISA_EMask_Ctrl emask, LSC_CACHE_OPTS cacheOpts, LSC_ADDR_TYPE addrType,
       LSC_ADDR_SIZE addrSize, LSC_DATA_SHAPE data,
       VISA_VectorOpnd *surface, unsigned surfaceIndex,
       VISA_RawOpnd *dst,
-      VISA_RawOpnd *coord0s, int coord0Offset,
-      VISA_RawOpnd *coord1s, int coord1Offset,
-      VISA_RawOpnd *coord2s, int coord2Offset,
+      VISA_RawOpnd *Us, int uOffset,
+      VISA_RawOpnd *Vs, int vOffset,
+      VISA_RawOpnd *Rs, int rOffset,
       VISA_RawOpnd *features,
       VISA_RawOpnd *src1, VISA_RawOpnd *src2) override;
+
   VISA_BUILDER_API int AppendVISALscFence(LSC_SFID lscSfid,
                                           LSC_FENCE_OP fenceOp,
                                           LSC_SCOPE scope) override;
