@@ -155,66 +155,5 @@ public:
     return false;
   }
 };
-
-class VarSplitPass {
-public:
-  VarSplitPass(G4_Kernel &);
-
-  void run();
-  void replaceIntrinsics();
-  G4_Declare *getParentDcl(G4_Declare *);
-  std::vector<G4_Declare *> *getChildren(G4_Declare *);
-  std::vector<G4_Declare *> getSiblings(G4_Declare *);
-  bool isSplitDcl(G4_Declare *);
-  bool isPartialDcl(G4_Declare *);
-  unsigned int getSiblingNum(G4_Declare *);
-  unsigned int getIdealAllocation(G4_Declare *,
-                                  const LiveRangeVec &);
-  bool isChildDclUnused(G4_Declare *);
-  void writeHints(G4_Declare *, const LiveRangeVec &);
-  void undo(G4_Declare *);
-  bool reallocParent(G4_Declare *, const LiveRangeVec &);
-  bool isParentChildRelation(G4_Declare *, G4_Declare *);
-  bool isSplitVarLocal(G4_Declare *);
-  bool splitOccured() { return IRchanged; }
-
-private:
-  G4_Kernel &kernel;
-
-  void findSplitCandidates();
-  void split();
-  std::unordered_map<G4_Declare *, VarProperties> splitVars;
-  // split dcl, parent dcl
-  std::unordered_map<G4_Declare *, G4_Declare *> splitParentDcl;
-  // parent dcl, vector<children>
-  std::unordered_map<G4_Declare *, std::vector<G4_Declare *>> splitChildren;
-  // Store child dcls that are never referenced in CFG
-  std::unordered_set<G4_Declare *> unusedDcls;
-  // Store pre-split regions for undo
-  // <new src/dst region, <old src inst, old src rgn, old src#>>
-  std::unordered_map<G4_Operand *,
-                     std::tuple<G4_INST *, G4_Operand *, unsigned int>>
-      preSplit;
-  bool IRchanged = false;
-
-private:
-  // Split verification related declarations
-  void buildPreVerify();
-  void verify();
-  void verifyOverlap();
-
-  class InstData {
-  public:
-    G4_DstRegRegion *dst = nullptr;
-    unsigned int dstLb = 0;
-    unsigned int dstRb = 0;
-    G4_Operand *src[G4_MAX_SRCS] = {};
-    unsigned int srcLb[G4_MAX_SRCS] = {};
-    unsigned int srcRb[G4_MAX_SRCS] = {};
-
-    InstData() {}
-  };
-  std::unordered_map<G4_INST *, InstData> splitVerify;
-};
 }; // namespace vISA
 #endif
