@@ -351,8 +351,6 @@ DEF_VISA_OPTION(vISA_ScheduleStartBBID, ET_INT32, "-sched-start",
                 "USAGE: -sched-start <BB ID>\n", 0)
 DEF_VISA_OPTION(vISA_ScheduleEndBBID, ET_INT32, "-sched-end",
                 "USAGE: -sched-end <BB ID>\n", 0)
-DEF_VISA_OPTION(vISA_PreSchedGRFPressure, ET_BOOL, "-rp-aft-sched",
-                "USAGE: -rp-aft-sched\n", false)
 DEF_VISA_OPTION(vISA_preRA_MinRegThreshold, ET_INT32, "-minreg-rp",
                 "USAGE: -minreg-rp <threshold>\n", 0)
 DEF_VISA_OPTION(vISA_DumpSchedule, ET_BOOL, "-dumpSchedule", UNUSED, false)
@@ -487,7 +485,8 @@ DEF_VISA_OPTION(vISA_DumpIsaVarNames, ET_BOOL, "-dumpisavarnames", UNUSED, true)
 DEF_VISA_OPTION(vISA_UniqueLabels, ET_BOOL, "-uniqueLabel", UNUSED, false)
 DEF_VISA_OPTION(vISA_ShaderDumpFilter, ET_CSTR, "-shaderDumpFilter",
                 "USAGE: -shaderDumpFilter <regex>\n", NULL)
-DEF_VISA_OPTION(vISA_DumpvISA, ET_BOOL, "-dumpvisa", UNUSED, false)
+DEF_VISA_OPTION(vISA_DumpvISA, ET_BOOL, "-dumpvisa",
+                "-dumpvisa is deprecated and will be removed.", false)
 DEF_VISA_OPTION(vISA_StripComments, ET_BOOL, "-stripcomments", UNUSED, false)
 DEF_VISA_OPTION(vISA_dumpNewSyntax, ET_BOOL, "-disableIGASyntax", UNUSED, true)
 DEF_VISA_OPTION(vISA_NumGenBinariesWillBePatched, ET_INT32,
@@ -498,8 +497,8 @@ DEF_VISA_OPTION(vISA_noLdStAsmSyntax, ET_BOOL, "-noLdStAsmSyntax", UNUSED, false
 DEF_VISA_OPTION(vISA_ExtraIntfFile, ET_CSTR, "-extraIntfFile",
                 "USAGE: File Name with extra interference info.\n", NULL)
 DEF_VISA_OPTION(vISA_AddExtraIntfInfo, ET_BOOL, NULLSTR, UNUSED, false)
-DEF_VISA_OPTION(vISA_OutputvISABinaryName, ET_CSTR, "-outputCisaBinaryName",
-                UNUSED, NULL)
+DEF_VISA_OPTION(vISA_OutputIsaasmName, ET_CSTR, "-outputIsaasmName",
+                "USAGE: specify the name for the combined .isaasm file", NULL)
 DEF_VISA_OPTION(vISA_LabelStr, ET_CSTR, "-uniqueLabels",
                 "Label String is not provided for the -uniqueLabels option.",
                 NULL)
