@@ -9,6 +9,10 @@ SPDX-License-Identifier: MIT
 #ifndef _VISA_IGC_COMMON_HEADER_H_
 #define _VISA_IGC_COMMON_HEADER_H_
 
+#ifndef ARRAY_COUNT
+#define ARRAY_COUNT( x ) ( sizeof( x ) / sizeof( x[ 0 ] ) )
+#endif
+
 typedef enum {
   VISA_SUCCESS = 0,
   VISA_FAILURE = -1,
@@ -140,7 +144,8 @@ typedef enum {
   LABEL_BLOCK = 0x0,
   LABEL_SUBROUTINE = 0x1,
   LABEL_FC = 0x2,
-  LABEL_FUNCTION = 0x3
+  LABEL_FUNCTION = 0x3,
+  LABEL_DIVERGENT_RESOURCE_LOOP = 0x4,
 } VISA_Label_Kind;
 
 typedef enum {
@@ -406,6 +411,7 @@ typedef enum {
   ISA_MADW = 0x91,
   ISA_ADD3O = 0x92,
   ISA_RESERVED_93 = 0x93,
+  ISA_BREAKPOINT = 0x94,
   ISA_NUM_OPCODE,
   ISA_OPCODE_ENUM_SIZE = 0xFF
 } ISA_Opcode;
@@ -714,8 +720,6 @@ enum LSC_ADDR_SIZE {
   LSC_ADDR_SIZE_32b,  // [ADDR]:a32
   LSC_ADDR_SIZE_64b,  // [ADDR]:a64
 };
-static_assert(LSC_ADDR_SIZE_64b == 3,
-              "vISA binary encoding depends on enum ordinal value");
 
 enum LSC_ADDR_TYPE {
   LSC_ADDR_TYPE_INVALID,
@@ -724,8 +728,16 @@ enum LSC_ADDR_TYPE {
   LSC_ADDR_TYPE_SS,   // surface state offset
   LSC_ADDR_TYPE_BTI,  // binding table interface (legacy)
   //
-  // Pseudo address types
-  LSC_ADDR_TYPE_ARG, // built-in surface access to kernel arguments
+  LSC_ADDR_TYPE_ARG, // pseudo address type for kernel arguments
+};
+
+enum class LSC_DOC_ADDR_SPACE {
+  INVALID,
+  PRIVATE,
+  GLOBAL,
+  LOCAL,
+  GENERIC,
+  RAYSTACK,
 };
 
 //
@@ -741,7 +753,6 @@ typedef enum {
   LSC_CACHING_WRITETHROUGH,   // .wt
   LSC_CACHING_STREAMING,      // .st
   LSC_CACHING_READINVALIDATE, // .ri last use / invalidate after read
-
 } LSC_CACHE_OPT;
 // Only some combinations are legal (per platform)
 struct LSC_CACHE_OPTS {
@@ -783,6 +794,10 @@ struct LSC_ADDR {
 
   // The number of bits per address; not all address models support all sizes
   LSC_ADDR_SIZE size; // e.g. :a64, :a32, ...
+
+  // e.g private, global, local, generic, raystack
+  // used for documentation in comments
+  LSC_DOC_ADDR_SPACE addrSpace;
 };
 
 // The specific fence op
@@ -856,20 +871,23 @@ enum LSC_OP {
 
   LSC_INVALID = 0xFFFFFFFF,
 };
+
+/////////////////////////////////////////////////////////////////////
+// The vISA spec depends on the enum values assigned.  If you really
+// need to change them, fix any consumers encoders dependent on the
+// encoding (e.g. CM binary encoder) as well as the vISA spec.
+// This block just spot-checks a few of them.
+// NOTE: we can remove these constraints once we are rid of the vISA binary format.
 static_assert(LSC_LOAD == 0x0,
               "vISA binary encoding depends on enum ordinal value");
 static_assert(LSC_FENCE == 0x1F,
               "vISA binary encoding depends on enum ordinal value");
-
-/////////////////////////////////////////////////////////////////////
-// The vISA spec depends on the enum values assigned.  If you really
-// need to change then, fix any consumers encoders dependent on the
-// encoding (e.g. CM binary encoder) as well as the vISA spec.
-//
 static_assert(LSC_DATA_ELEMS_8 == 5,
               "vISA binary encoding depends on enum ordinal value");
 static_assert(LSC_ADDR_SIZE_16b == 1,
               "vISA binary encoding depends on enum ordinal value");
+static_assert(LSC_ADDR_SIZE_64b == 3,
+              "vISA binary encoding depends on enum ordinal value");
 static_assert(LSC_DATA_SIZE_8b == 1,
               "vISA binary encoding depends on enum ordinal value");
 static_assert(LSC_DATA_SIZE_16b == 2,
@@ -913,4 +931,5 @@ enum FFID {
   FFID_INVALID = 0xFF
 };
 
-#endif
+
+#endif // _VISA_IGC_COMMON_HEADER_H_
